@@ -1,3 +1,17 @@
+// Copyright 2020-2023 The Jujutsu Authors
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+// https://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
 use std::collections::HashSet;
 use std::fmt;
 use std::io::Write as _;
@@ -34,6 +48,8 @@ pub enum BranchSubcommand {
     Forget(BranchForgetArgs),
     #[command(visible_alias("l"))]
     List(BranchListArgs),
+    #[command(visible_alias("r"))]
+    Rename(BranchRenameArgs),
     #[command(visible_alias("s"))]
     Set(BranchSetArgs),
     Track(BranchTrackArgs),
@@ -122,6 +138,19 @@ pub struct BranchForgetArgs {
     pub glob: Vec<StringPattern>,
 }
 
+/// Rename `old` branch name to `new` branch name.
+///
+/// The new branch name points at the same commit as the old
+/// branch name.
+#[derive(clap::Args, Clone, Debug)]
+pub struct BranchRenameArgs {
+    /// The old name of the branch.
+    pub old: String,
+
+    /// The new name of the branch.
+    pub new: String,
+}
+
 /// Update an existing branch to point to a certain commit.
 #[derive(clap::Args, Clone, Debug)]
 pub struct BranchSetArgs {
@@ -150,7 +179,9 @@ pub struct BranchTrackArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(required = true)]
+    ///
+    /// Examples: branch@remote, glob:main@*, glob:jjfan-*@upstream
+    #[arg(required = true, value_name = "BRANCH@REMOTE")]
     pub names: Vec<RemoteBranchNamePattern>,
 }
 
@@ -165,7 +196,9 @@ pub struct BranchUntrackArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(required = true)]
+    ///
+    /// Examples: branch@remote, glob:main@*, glob:jjfan-*@upstream
+    #[arg(required = true, value_name = "BRANCH@REMOTE")]
     pub names: Vec<RemoteBranchNamePattern>,
 }
 
@@ -244,6 +277,7 @@ pub fn cmd_branch(
 ) -> Result<(), CommandError> {
     match subcommand {
         BranchSubcommand::Create(sub_args) => cmd_branch_create(ui, command, sub_args),
+        BranchSubcommand::Rename(sub_args) => cmd_branch_rename(ui, command, sub_args),
         BranchSubcommand::Set(sub_args) => cmd_branch_set(ui, command, sub_args),
         BranchSubcommand::Delete(sub_args) => cmd_branch_delete(ui, command, sub_args),
         BranchSubcommand::Forget(sub_args) => cmd_branch_forget(ui, command, sub_args),
@@ -281,16 +315,53 @@ fn cmd_branch_create(
         )?;
     }
 
-    let mut tx = workspace_command.start_transaction(&format!(
-        "create {} pointing to commit {}",
-        make_branch_term(branch_names),
-        target_commit.id().hex()
-    ));
+    let mut tx = workspace_command.start_transaction();
     for branch_name in branch_names {
         tx.mut_repo()
             .set_local_branch_target(branch_name, RefTarget::normal(target_commit.id().clone()));
     }
-    tx.finish(ui)?;
+    tx.finish(
+        ui,
+        format!(
+            "create {} pointing to commit {}",
+            make_branch_term(branch_names),
+            target_commit.id().hex()
+        ),
+    )?;
+    Ok(())
+}
+
+fn cmd_branch_rename(
+    ui: &mut Ui,
+    command: &CommandHelper,
+    args: &BranchRenameArgs,
+) -> Result<(), CommandError> {
+    let mut workspace_command = command.workspace_helper(ui)?;
+    let view = workspace_command.repo().view();
+    let old_branch = &args.old;
+    let ref_target = view.get_local_branch(old_branch).clone();
+    if ref_target.is_absent() {
+        return Err(user_error(format!("No such branch: {old_branch}")));
+    }
+
+    let new_branch = &args.new;
+    if view.get_local_branch(new_branch).is_present() {
+        return Err(user_error(format!("Branch already exists: {new_branch}")));
+    }
+
+    let mut tx = workspace_command.start_transaction();
+    tx.mut_repo()
+        .set_local_branch_target(new_branch, ref_target);
+    tx.mut_repo()
+        .set_local_branch_target(old_branch, RefTarget::absent());
+    tx.finish(
+        ui,
+        format!(
+            "rename {} to {}",
+            make_branch_term(&[old_branch]),
+            make_branch_term(&[new_branch]),
+        ),
+    )?;
     Ok(())
 }
 
@@ -335,16 +406,19 @@ fn cmd_branch_set(
         )?;
     }
 
-    let mut tx = workspace_command.start_transaction(&format!(
-        "point {} to commit {}",
-        make_branch_term(branch_names),
-        target_commit.id().hex()
-    ));
+    let mut tx = workspace_command.start_transaction();
     for branch_name in branch_names {
         tx.mut_repo()
             .set_local_branch_target(branch_name, RefTarget::normal(target_commit.id().clone()));
     }
-    tx.finish(ui)?;
+    tx.finish(
+        ui,
+        format!(
+            "point {} to commit {}",
+            make_branch_term(branch_names),
+            target_commit.id().hex()
+        ),
+    )?;
     Ok(())
 }
 
@@ -449,13 +523,12 @@ fn cmd_branch_delete(
     }
     let name_patterns = [&args.names[..], &args.glob[..]].concat();
     let names = find_local_branches(view, &name_patterns)?;
-    let mut tx =
-        workspace_command.start_transaction(&format!("delete {}", make_branch_term(&names)));
+    let mut tx = workspace_command.start_transaction();
     for branch_name in names.iter() {
         tx.mut_repo()
             .set_local_branch_target(branch_name, RefTarget::absent());
     }
-    tx.finish(ui)?;
+    tx.finish(ui, format!("delete {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(ui.stderr(), "Deleted {} branches.", names.len())?;
     }
@@ -477,12 +550,11 @@ fn cmd_branch_forget(
     }
     let name_patterns = [&args.names[..], &args.glob[..]].concat();
     let names = find_forgettable_branches(view, &name_patterns)?;
-    let mut tx =
-        workspace_command.start_transaction(&format!("forget {}", make_branch_term(&names)));
+    let mut tx = workspace_command.start_transaction();
     for branch_name in names.iter() {
         tx.mut_repo().remove_branch(branch_name);
     }
-    tx.finish(ui)?;
+    tx.finish(ui, format!("forget {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(ui.stderr(), "Forgot {} branches.", names.len())?;
     }
@@ -504,13 +576,12 @@ fn cmd_branch_track(
             names.push(name);
         }
     }
-    let mut tx =
-        workspace_command.start_transaction(&format!("track remote {}", make_branch_term(&names)));
+    let mut tx = workspace_command.start_transaction();
     for name in &names {
         tx.mut_repo()
             .track_remote_branch(&name.branch, &name.remote);
     }
-    tx.finish(ui)?;
+    tx.finish(ui, format!("track remote {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(
             ui.stderr(),
@@ -542,13 +613,12 @@ fn cmd_branch_untrack(
             names.push(name);
         }
     }
-    let mut tx = workspace_command
-        .start_transaction(&format!("untrack remote {}", make_branch_term(&names)));
+    let mut tx = workspace_command.start_transaction();
     for name in &names {
         tx.mut_repo()
             .untrack_remote_branch(&name.branch, &name.remote);
     }
-    tx.finish(ui)?;
+    tx.finish(ui, format!("untrack remote {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(
             ui.stderr(),
@@ -699,13 +769,13 @@ fn cmd_branch_list(
                 .any(|&(remote, _)| remote != git::REMOTE_NAME_FOR_LOCAL_GIT_REPO);
             if found_non_git_remote {
                 writeln!(
-                    formatter,
-                    "  (this branch will be *deleted permanently* on the remote on the\n   next \
-                     `jj git push`. Use `jj branch forget` to prevent this)"
+                    formatter.labeled("hint"),
+                    "  (this branch will be *deleted permanently* on the remote on the next `jj \
+                     git push`. Use `jj branch forget` to prevent this)"
                 )?;
             } else {
                 writeln!(
-                    formatter,
+                    formatter.labeled("hint"),
                     "  (this branch will be deleted from the underlying Git repo on the next `jj \
                      git export`)"
                 )?;
