@@ -18,12 +18,13 @@ use std::io::Write as _;
 
 use clap::Subcommand;
 use jj_lib::backend::ObjectId;
-use jj_lib::default_index_store::{DefaultIndexStore, ReadonlyIndexWrapper};
+use jj_lib::default_index::{AsCompositeIndex as _, DefaultIndexStore, DefaultReadonlyIndex};
 use jj_lib::local_working_copy::LocalWorkingCopy;
-use jj_lib::revset;
+use jj_lib::repo::Repo;
 use jj_lib::working_copy::WorkingCopy;
+use jj_lib::{op_walk, revset};
 
-use crate::cli_util::{resolve_op_for_load, user_error, CommandError, CommandHelper, RevisionArg};
+use crate::cli_util::{user_error, CommandError, CommandHelper, RevisionArg};
 use crate::template_parser;
 use crate::ui::Ui;
 
@@ -199,9 +200,10 @@ fn cmd_debug_index(
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
     let repo = workspace_command.repo();
-    let index_impl: Option<&ReadonlyIndexWrapper> = repo.readonly_index().as_any().downcast_ref();
-    if let Some(index_impl) = index_impl {
-        let stats = index_impl.as_composite().stats();
+    let default_index: Option<&DefaultReadonlyIndex> =
+        repo.readonly_index().as_any().downcast_ref();
+    if let Some(default_index) = default_index {
+        let stats = default_index.as_composite().stats();
         writeln!(ui.stdout(), "Number of commits: {}", stats.num_commits)?;
         writeln!(ui.stdout(), "Number of merges: {}", stats.num_merges)?;
         writeln!(
@@ -236,17 +238,16 @@ fn cmd_debug_reindex(
     let default_index_store: Option<&DefaultIndexStore> =
         repo.index_store().as_any().downcast_ref();
     if let Some(default_index_store) = default_index_store {
-        default_index_store.reinit();
-        let repo = repo.reload_at(repo.operation())?;
-        let index_impl: &ReadonlyIndexWrapper = repo
-            .readonly_index()
-            .as_any()
-            .downcast_ref()
-            .expect("Default index should be a ReadonlyIndexWrapper");
+        default_index_store
+            .reinit()
+            .map_err(|err| CommandError::InternalError(err.to_string()))?;
+        let default_index = default_index_store
+            .build_index_at_operation(repo.operation(), repo.store())
+            .map_err(|err| CommandError::InternalError(err.to_string()))?;
         writeln!(
             ui.stderr(),
             "Finished indexing {:?} commits.",
-            index_impl.as_composite().stats().num_commits
+            default_index.as_composite().stats().num_commits
         )?;
     } else {
         return Err(user_error(format!(
@@ -266,11 +267,7 @@ fn cmd_debug_operation(
     // even if e.g. the view object is broken.
     let workspace = command.load_workspace()?;
     let repo_loader = workspace.repo_loader();
-    let op = resolve_op_for_load(
-        repo_loader.op_store(),
-        repo_loader.op_heads_store(),
-        &args.operation,
-    )?;
+    let op = op_walk::resolve_op_for_load(repo_loader, &args.operation)?;
     if args.display == DebugOperationDisplay::Id {
         writeln!(ui.stdout(), "{}", op.id().hex())?;
         return Ok(());
