@@ -48,6 +48,11 @@ pub(crate) struct DescribeArgs {
     /// Reset the author to the configured user
     ///
     /// This resets the author name, email, and timestamp.
+    ///
+    /// You can use it in combination with the JJ_USER and JJ_EMAIL
+    /// environment variables to set a different author:
+    ///
+    /// $ JJ_USER='Foo Bar' JJ_EMAIL=foo@bar.com jj describe --reset-author
     #[arg(long)]
     reset_author: bool,
 }
@@ -77,8 +82,7 @@ pub(crate) fn cmd_describe(
     if description == *commit.description() && !args.reset_author {
         writeln!(ui.stderr(), "Nothing changed.")?;
     } else {
-        let mut tx =
-            workspace_command.start_transaction(&format!("describe commit {}", commit.id().hex()));
+        let mut tx = workspace_command.start_transaction();
         let mut commit_builder = tx
             .mut_repo()
             .rewrite_commit(command.settings(), &commit)
@@ -88,7 +92,7 @@ pub(crate) fn cmd_describe(
             commit_builder = commit_builder.set_author(new_author);
         }
         commit_builder.write()?;
-        tx.finish(ui)?;
+        tx.finish(ui, format!("describe commit {}", commit.id().hex()))?;
     }
     Ok(())
 }
