@@ -1,3 +1,17 @@
+// Copyright 2020-2023 The Jujutsu Authors
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+// https://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
 use std::collections::HashSet;
 use std::io::{Read, Write};
 use std::ops::Deref;
@@ -257,11 +271,10 @@ fn cmd_git_remote_remove(
     let mut workspace_command = command.workspace_helper(ui)?;
     let repo = workspace_command.repo();
     let git_repo = get_git_repo(repo.store())?;
-    let mut tx =
-        workspace_command.start_transaction(&format!("remove git remote {}", &args.remote));
+    let mut tx = workspace_command.start_transaction();
     git::remove_remote(tx.mut_repo(), &git_repo, &args.remote)?;
     if tx.mut_repo().has_changes() {
-        tx.finish(ui)
+        tx.finish(ui, format!("remove git remote {}", &args.remote))
     } else {
         Ok(()) // Do not print "Nothing changed."
     }
@@ -275,11 +288,13 @@ fn cmd_git_remote_rename(
     let mut workspace_command = command.workspace_helper(ui)?;
     let repo = workspace_command.repo();
     let git_repo = get_git_repo(repo.store())?;
-    let mut tx = workspace_command
-        .start_transaction(&format!("rename git remote {} to {}", &args.old, &args.new));
+    let mut tx = workspace_command.start_transaction();
     git::rename_remote(tx.mut_repo(), &git_repo, &args.old, &args.new)?;
     if tx.mut_repo().has_changes() {
-        tx.finish(ui)
+        tx.finish(
+            ui,
+            format!("rename git remote {} to {}", &args.old, &args.new),
+        )
     } else {
         Ok(()) // Do not print "Nothing changed."
     }
@@ -320,16 +335,13 @@ fn cmd_git_fetch(
     } else {
         args.remotes.clone()
     };
-    let mut tx = workspace_command.start_transaction(&format!(
-        "fetch from git remote(s) {}",
-        remotes.iter().join(",")
-    ));
-    for remote in remotes {
+    let mut tx = workspace_command.start_transaction();
+    for remote in &remotes {
         let stats = with_remote_callbacks(ui, |cb| {
             git::fetch(
                 tx.mut_repo(),
                 &git_repo,
-                &remote,
+                remote,
                 &args.branch,
                 cb,
                 &command.settings().git_settings(),
@@ -356,7 +368,10 @@ fn cmd_git_fetch(
         })?;
         print_git_import_stats(ui, &stats.import_stats)?;
     }
-    tx.finish(ui)?;
+    tx.finish(
+        ui,
+        format!("fetch from git remote(s) {}", remotes.iter().join(",")),
+    )?;
     Ok(())
 }
 
@@ -441,9 +456,6 @@ fn cmd_git_clone(
     command: &CommandHelper,
     args: &GitCloneArgs,
 ) -> Result<(), CommandError> {
-    if command.global_args().repository.is_some() {
-        return Err(user_error("'--repository' cannot be used with 'git clone'"));
-    }
     let remote_name = "origin";
     let source = absolute_git_source(command.cwd(), &args.source);
     let wc_path_str = args
@@ -505,12 +517,15 @@ fn cmd_git_clone(
             .view()
             .get_remote_branch(default_branch, remote_name);
         if let Some(commit_id) = default_branch_remote_ref.target.as_normal().cloned() {
-            let mut checkout_tx =
-                workspace_command.start_transaction("check out git remote's default branch");
+            let mut checkout_tx = workspace_command.start_transaction();
+            // For convenience, create local branch as Git would do.
+            checkout_tx
+                .mut_repo()
+                .track_remote_branch(default_branch, remote_name);
             if let Ok(commit) = checkout_tx.repo().store().get_commit(&commit_id) {
                 checkout_tx.check_out(&commit)?;
             }
-            checkout_tx.finish(ui)?;
+            checkout_tx.finish(ui, "check out git remote's default branch")?;
         }
     }
     Ok(())
@@ -538,7 +553,7 @@ fn do_git_clone(
     let mut workspace_command = command.for_loaded_repo(ui, workspace, repo)?;
     maybe_add_gitignore(&workspace_command)?;
     git_repo.remote(remote_name, source).unwrap();
-    let mut fetch_tx = workspace_command.start_transaction("fetch from git remote into empty repo");
+    let mut fetch_tx = workspace_command.start_transaction();
 
     let stats = with_remote_callbacks(ui, |cb| {
         git::fetch(
@@ -561,7 +576,7 @@ fn do_git_clone(
         }
     })?;
     print_git_import_stats(ui, &stats.import_stats)?;
-    fetch_tx.finish(ui)?;
+    fetch_tx.finish(ui, "fetch from git remote into empty repo")?;
     Ok((workspace_command, stats))
 }
 
@@ -700,7 +715,7 @@ fn cmd_git_push(
         .map(|change_str| workspace_command.resolve_single_rev(change_str, ui))
         .try_collect()?;
 
-    let mut tx = workspace_command.start_transaction("");
+    let mut tx = workspace_command.start_transaction();
     let tx_description;
     let mut branch_updates = vec![];
     if args.all {
@@ -855,8 +870,6 @@ fn cmd_git_push(
         return Ok(());
     }
 
-    tx.set_description(&tx_description);
-
     let mut new_heads = vec![];
     let mut force_pushed_branches = hashset! {};
     for (branch_name, update) in &branch_updates {
@@ -976,7 +989,7 @@ fn cmd_git_push(
         ),
         _ => user_error(err.to_string()),
     })?;
-    tx.finish(ui)?;
+    tx.finish(ui, tx_description)?;
     Ok(())
 }
 
@@ -1088,10 +1101,10 @@ fn cmd_git_import(
     _args: &GitImportArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
-    let mut tx = workspace_command.start_transaction("import git refs");
+    let mut tx = workspace_command.start_transaction();
     let stats = git::import_refs(tx.mut_repo(), &command.settings().git_settings())?;
     print_git_import_stats(ui, &stats)?;
-    tx.finish(ui)?;
+    tx.finish(ui, "import git refs")?;
     Ok(())
 }
 
@@ -1101,11 +1114,9 @@ fn cmd_git_export(
     _args: &GitExportArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
-    let repo = workspace_command.repo();
-    let git_repo = get_git_repo(repo.store())?;
-    let mut tx = workspace_command.start_transaction("export git refs");
-    let failed_branches = git::export_refs(tx.mut_repo(), &git_repo)?;
-    tx.finish(ui)?;
+    let mut tx = workspace_command.start_transaction();
+    let failed_branches = git::export_refs(tx.mut_repo())?;
+    tx.finish(ui, "export git refs")?;
     print_failed_git_export(ui, &failed_branches)?;
     Ok(())
 }
