@@ -66,11 +66,7 @@ pub(crate) fn cmd_move(
     }
     workspace_command.check_rewritable([&source, &destination])?;
     let matcher = workspace_command.matcher_from_values(&args.paths)?;
-    let mut tx = workspace_command.start_transaction(&format!(
-        "move changes from {} to {}",
-        source.id().hex(),
-        destination.id().hex()
-    ));
+    let mut tx = workspace_command.start_transaction();
     let parent_tree = merge_commit_trees(tx.repo(), &source.parents())?;
     let source_tree = source.tree()?;
     let instructions = format!(
@@ -117,9 +113,10 @@ from the source will be moved into the destination.
         // rewritten source. Otherwise it will likely already have the content
         // changes we're moving, so applying them will have no effect and the
         // changes will disappear.
-        let mut rebaser = tx.mut_repo().create_descendant_rebaser(command.settings());
-        rebaser.rebase_all()?;
-        let rebased_destination_id = rebaser.rebased().get(destination.id()).unwrap().clone();
+        let rebase_map = tx
+            .mut_repo()
+            .rebase_descendants_return_map(command.settings())?;
+        let rebased_destination_id = rebase_map.get(destination.id()).unwrap().clone();
         destination = tx.mut_repo().store().get_commit(&rebased_destination_id)?;
     }
     // Apply the selected changes onto the destination
@@ -137,6 +134,13 @@ from the source will be moved into the destination.
         .set_tree_id(new_destination_tree.id().clone())
         .set_description(description)
         .write()?;
-    tx.finish(ui)?;
+    tx.finish(
+        ui,
+        format!(
+            "move changes from {} to {}",
+            source.id().hex(),
+            destination.id().hex()
+        ),
+    )?;
     Ok(())
 }
