@@ -1,6 +1,20 @@
+// Copyright 2020-2023 The Jujutsu Authors
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+// https://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+
 use clap::Subcommand;
 use jj_lib::backend::ObjectId;
-use jj_lib::operation;
+use jj_lib::op_walk;
 use jj_lib::repo::Repo;
 
 use crate::cli_util::{user_error, CommandError, CommandHelper, LogContentFormat};
@@ -111,7 +125,7 @@ fn cmd_op_log(
     ui.request_pager();
     let mut formatter = ui.stdout_formatter();
     let formatter = formatter.as_mut();
-    let iter = operation::walk_ancestors(&head_op).take(args.limit.unwrap_or(usize::MAX));
+    let iter = op_walk::walk_ancestors(&[head_op]).take(args.limit.unwrap_or(usize::MAX));
     if !args.no_graph {
         let mut graph = get_graphlog(command.settings(), formatter.raw());
         let default_node_symbol = graph.default_node_symbol().to_owned();
@@ -200,8 +214,7 @@ pub fn cmd_op_undo(
         return Err(user_error("Cannot undo a merge operation"));
     }
 
-    let mut tx =
-        workspace_command.start_transaction(&format!("undo operation {}", bad_op.id().hex()));
+    let mut tx = workspace_command.start_transaction();
     let repo_loader = tx.base_repo().loader();
     let bad_repo = repo_loader.load_at(&bad_op)?;
     let parent_repo = repo_loader.load_at(&parent_op)?;
@@ -212,7 +225,7 @@ pub fn cmd_op_undo(
         &args.what,
     );
     tx.mut_repo().set_view(new_view);
-    tx.finish(ui)?;
+    tx.finish(ui, format!("undo operation {}", bad_op.id().hex()))?;
 
     Ok(())
 }
@@ -224,15 +237,14 @@ fn cmd_op_restore(
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
     let target_op = workspace_command.resolve_single_op(&args.operation)?;
-    let mut tx = workspace_command
-        .start_transaction(&format!("restore to operation {}", target_op.id().hex()));
+    let mut tx = workspace_command.start_transaction();
     let new_view = view_with_desired_portions_restored(
         target_op.view()?.store_view(),
         tx.base_repo().view().store_view(),
         &args.what,
     );
     tx.mut_repo().set_view(new_view);
-    tx.finish(ui)?;
+    tx.finish(ui, format!("restore to operation {}", target_op.id().hex()))?;
 
     Ok(())
 }
