@@ -95,17 +95,17 @@ pub(crate) fn cmd_resolve(
 
     let (repo_path, _) = conflicts.first().unwrap();
     workspace_command.check_rewritable([&commit])?;
-    let mut tx = workspace_command.start_transaction(&format!(
-        "Resolve conflicts in commit {}",
-        commit.id().hex()
-    ));
+    let mut tx = workspace_command.start_transaction();
     let new_tree_id = tx.run_mergetool(ui, &tree, repo_path)?;
     let new_commit = tx
         .mut_repo()
         .rewrite_commit(command.settings(), &commit)
         .set_tree_id(new_tree_id)
         .write()?;
-    tx.finish(ui)?;
+    tx.finish(
+        ui,
+        format!("Resolve conflicts in commit {}", commit.id().hex()),
+    )?;
 
     if !args.quiet {
         let new_tree = new_commit.tree()?;
