@@ -35,7 +35,7 @@ use crate::template_parser::{
     self, FunctionCallNode, TemplateAliasesMap, TemplateParseError, TemplateParseResult,
 };
 use crate::templater::{
-    IntoTemplate, PlainTextFormattedProperty, Template, TemplateFunction, TemplateProperty,
+    self, IntoTemplate, PlainTextFormattedProperty, Template, TemplateFunction, TemplateProperty,
     TemplatePropertyFn,
 };
 use crate::text_util;
@@ -79,6 +79,18 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo, '_> {
                     |item| self.wrap_commit(item),
                 )
             }
+            CommitTemplatePropertyKind::RefName(property) => {
+                build_ref_name_method(self, build_ctx, property, function)
+            }
+            CommitTemplatePropertyKind::RefNameList(property) => {
+                template_builder::build_formattable_list_method(
+                    self,
+                    build_ctx,
+                    property,
+                    function,
+                    |item| self.wrap_ref_name(item),
+                )
+            }
             CommitTemplatePropertyKind::CommitOrChangeId(property) => {
                 build_commit_or_change_id_method(self, build_ctx, property, function)
             }
@@ -106,6 +118,20 @@ impl<'repo> CommitTemplateLanguage<'repo, '_> {
         CommitTemplatePropertyKind::CommitList(Box::new(property))
     }
 
+    fn wrap_ref_name(
+        &self,
+        property: impl TemplateProperty<Commit, Output = RefName> + 'repo,
+    ) -> CommitTemplatePropertyKind<'repo> {
+        CommitTemplatePropertyKind::RefName(Box::new(property))
+    }
+
+    fn wrap_ref_name_list(
+        &self,
+        property: impl TemplateProperty<Commit, Output = Vec<RefName>> + 'repo,
+    ) -> CommitTemplatePropertyKind<'repo> {
+        CommitTemplatePropertyKind::RefNameList(Box::new(property))
+    }
+
     fn wrap_commit_or_change_id(
         &self,
         property: impl TemplateProperty<Commit, Output = CommitOrChangeId> + 'repo,
@@ -125,6 +151,8 @@ enum CommitTemplatePropertyKind<'repo> {
     Core(CoreTemplatePropertyKind<'repo, Commit>),
     Commit(Box<dyn TemplateProperty<Commit, Output = Commit> + 'repo>),
     CommitList(Box<dyn TemplateProperty<Commit, Output = Vec<Commit>> + 'repo>),
+    RefName(Box<dyn TemplateProperty<Commit, Output = RefName> + 'repo>),
+    RefNameList(Box<dyn TemplateProperty<Commit, Output = Vec<RefName>> + 'repo>),
     CommitOrChangeId(Box<dyn TemplateProperty<Commit, Output = CommitOrChangeId> + 'repo>),
     ShortestIdPrefix(Box<dyn TemplateProperty<Commit, Output = ShortestIdPrefix> + 'repo>),
 }
@@ -133,8 +161,16 @@ impl<'repo> IntoTemplateProperty<'repo, Commit> for CommitTemplatePropertyKind<'
     fn try_into_boolean(self) -> Option<Box<dyn TemplateProperty<Commit, Output = bool> + 'repo>> {
         match self {
             CommitTemplatePropertyKind::Core(property) => property.try_into_boolean(),
-            // TODO: should we allow implicit cast of List type?
-            _ => None,
+            CommitTemplatePropertyKind::Commit(_) => None,
+            CommitTemplatePropertyKind::CommitList(property) => {
+                Some(Box::new(TemplateFunction::new(property, |l| !l.is_empty())))
+            }
+            CommitTemplatePropertyKind::RefName(_) => None,
+            CommitTemplatePropertyKind::RefNameList(property) => {
+                Some(Box::new(TemplateFunction::new(property, |l| !l.is_empty())))
+            }
+            CommitTemplatePropertyKind::CommitOrChangeId(_) => None,
+            CommitTemplatePropertyKind::ShortestIdPrefix(_) => None,
         }
     }
 
@@ -162,6 +198,8 @@ impl<'repo> IntoTemplateProperty<'repo, Commit> for CommitTemplatePropertyKind<'
             CommitTemplatePropertyKind::Core(property) => property.try_into_template(),
             CommitTemplatePropertyKind::Commit(_) => None,
             CommitTemplatePropertyKind::CommitList(_) => None,
+            CommitTemplatePropertyKind::RefName(property) => Some(property.into_template()),
+            CommitTemplatePropertyKind::RefNameList(property) => Some(property.into_template()),
             CommitTemplatePropertyKind::CommitOrChangeId(property) => {
                 Some(property.into_template())
             }
@@ -273,23 +311,50 @@ fn build_commit_keyword_opt<'repo>(
         }
         "branches" => {
             let index = cache.branches_index(repo).clone();
-            language.wrap_string(wrap_fn(property, move |commit| {
-                index.get(commit.id()).join(" ") // TODO: return Vec<Branch>?
+            language.wrap_ref_name_list(wrap_fn(property, move |commit| {
+                index
+                    .get(commit.id())
+                    .iter()
+                    .filter(|ref_name| ref_name.is_local() || !ref_name.synced)
+                    .cloned()
+                    .collect()
+            }))
+        }
+        "local_branches" => {
+            let index = cache.branches_index(repo).clone();
+            language.wrap_ref_name_list(wrap_fn(property, move |commit| {
+                index
+                    .get(commit.id())
+                    .iter()
+                    .filter(|ref_name| ref_name.is_local())
+                    .cloned()
+                    .collect()
+            }))
+        }
+        "remote_branches" => {
+            let index = cache.branches_index(repo).clone();
+            language.wrap_ref_name_list(wrap_fn(property, move |commit| {
+                index
+                    .get(commit.id())
+                    .iter()
+                    .filter(|ref_name| ref_name.is_remote())
+                    .cloned()
+                    .collect()
             }))
         }
         "tags" => {
             let index = cache.tags_index(repo).clone();
-            language.wrap_string(wrap_fn(property, move |commit| {
-                index.get(commit.id()).join(" ") // TODO: return Vec<NameRef>?
+            language.wrap_ref_name_list(wrap_fn(property, move |commit| {
+                index.get(commit.id()).to_vec()
             }))
         }
         "git_refs" => {
             let index = cache.git_refs_index(repo).clone();
-            language.wrap_string(wrap_fn(property, move |commit| {
-                index.get(commit.id()).join(" ") // TODO: return Vec<NameRef>?
+            language.wrap_ref_name_list(wrap_fn(property, move |commit| {
+                index.get(commit.id()).to_vec()
             }))
         }
-        "git_head" => language.wrap_string(wrap_repo_fn(repo, property, extract_git_head)),
+        "git_head" => language.wrap_ref_name_list(wrap_repo_fn(repo, property, extract_git_head)),
         "divergent" => language.wrap_boolean(wrap_fn(property, |commit| {
             // The given commit could be hidden in e.g. obslog.
             let maybe_entries = repo.resolve_change_id(commit.change_id());
@@ -332,22 +397,94 @@ fn extract_working_copies(repo: &dyn Repo, commit: &Commit) -> String {
     names.join(" ")
 }
 
+/// Branch or tag name with metadata.
+#[derive(Clone, Debug, Eq, PartialEq)]
+struct RefName {
+    /// Local name.
+    name: String,
+    /// Remote name if this is a remote or Git-tracking ref.
+    remote: Option<String>,
+    /// Ref target has conflicts.
+    conflict: bool,
+    /// Local ref is synchronized with all tracking remotes, or tracking remote
+    /// ref is synchronized with the local.
+    synced: bool,
+}
+
+impl RefName {
+    fn is_local(&self) -> bool {
+        self.remote.is_none()
+    }
+
+    fn is_remote(&self) -> bool {
+        self.remote.is_some()
+    }
+}
+
+impl Template<()> for RefName {
+    fn format(&self, _: &(), formatter: &mut dyn Formatter) -> io::Result<()> {
+        write!(formatter.labeled("name"), "{}", self.name)?;
+        if let Some(remote) = &self.remote {
+            write!(formatter, "@")?;
+            write!(formatter.labeled("remote"), "{remote}")?;
+        }
+        // Don't show both conflict and unsynced sigils as conflicted ref wouldn't
+        // be pushed.
+        if self.conflict {
+            write!(formatter, "??")?;
+        } else if self.is_local() && !self.synced {
+            write!(formatter, "*")?;
+        }
+        Ok(())
+    }
+}
+
+impl Template<()> for Vec<RefName> {
+    fn format(&self, _: &(), formatter: &mut dyn Formatter) -> io::Result<()> {
+        templater::format_joined(&(), formatter, self, " ")
+    }
+}
+
+fn build_ref_name_method<'repo>(
+    language: &CommitTemplateLanguage<'repo, '_>,
+    _build_ctx: &BuildContext<CommitTemplatePropertyKind<'repo>>,
+    self_property: impl TemplateProperty<Commit, Output = RefName> + 'repo,
+    function: &FunctionCallNode,
+) -> TemplateParseResult<CommitTemplatePropertyKind<'repo>> {
+    let property = match function.name {
+        "name" => {
+            template_parser::expect_no_arguments(function)?;
+            language.wrap_string(TemplateFunction::new(self_property, |ref_name| {
+                ref_name.name
+            }))
+        }
+        "remote" => {
+            template_parser::expect_no_arguments(function)?;
+            language.wrap_string(TemplateFunction::new(self_property, |ref_name| {
+                ref_name.remote.unwrap_or_default()
+            }))
+        }
+        // TODO: expose conflict, synced, remote.is_some()
+        _ => return Err(TemplateParseError::no_such_method("RefName", function)),
+    };
+    Ok(property)
+}
+
 /// Cache for reverse lookup refs.
 #[derive(Clone, Debug, Default)]
 struct RefNamesIndex {
-    // TODO: store Branch/NameRef type in place of String?
-    index: HashMap<CommitId, Vec<String>>,
+    index: HashMap<CommitId, Vec<RefName>>,
 }
 
 impl RefNamesIndex {
-    fn insert<'a>(&mut self, ids: impl IntoIterator<Item = &'a CommitId>, name: String) {
+    fn insert<'a>(&mut self, ids: impl IntoIterator<Item = &'a CommitId>, name: RefName) {
         for id in ids {
             let ref_names = self.index.entry(id.clone()).or_default();
             ref_names.push(name.clone());
         }
     }
 
-    fn get(&self, id: &CommitId) -> &[String] {
+    fn get(&self, id: &CommitId) -> &[RefName] {
         if let Some(names) = self.index.get(id) {
             names
         } else {
@@ -358,31 +495,28 @@ impl RefNamesIndex {
 
 fn build_branches_index(repo: &dyn Repo) -> RefNamesIndex {
     let mut index = RefNamesIndex::default();
-    let all_branches = git::build_unified_branches_map(repo.view());
-    for (branch_name, branch_target) in &all_branches {
-        let local_target = &branch_target.local_target;
-        let mut unsynced_remote_targets = branch_target
-            .remote_targets
-            .iter()
-            .filter(|&(_, target)| target != local_target)
-            .peekable();
+    for (branch_name, branch_target) in repo.view().branches() {
+        let local_target = branch_target.local_target;
+        let remote_refs = branch_target.remote_refs;
         if local_target.is_present() {
-            let decorated_name = if local_target.has_conflict() {
-                format!("{branch_name}??")
-            } else if unsynced_remote_targets.peek().is_some() {
-                format!("{branch_name}*")
-            } else {
-                branch_name.clone()
+            let ref_name = RefName {
+                name: branch_name.to_owned(),
+                remote: None,
+                conflict: local_target.has_conflict(),
+                synced: remote_refs.iter().all(|&(_, remote_ref)| {
+                    !remote_ref.is_tracking() || remote_ref.target == *local_target
+                }),
             };
-            index.insert(local_target.added_ids(), decorated_name);
+            index.insert(local_target.added_ids(), ref_name);
         }
-        for (remote_name, target) in unsynced_remote_targets {
-            let decorated_name = if target.has_conflict() {
-                format!("{branch_name}@{remote_name}?")
-            } else {
-                format!("{branch_name}@{remote_name}")
+        for &(remote_name, remote_ref) in &remote_refs {
+            let ref_name = RefName {
+                name: branch_name.to_owned(),
+                remote: Some(remote_name.to_owned()),
+                conflict: remote_ref.target.has_conflict(),
+                synced: remote_ref.is_tracking() && remote_ref.target == *local_target,
             };
-            index.insert(target.added_ids(), decorated_name);
+            index.insert(remote_ref.target.added_ids(), ref_name);
         }
     }
     index
@@ -393,27 +527,30 @@ fn build_ref_names_index<'a>(
 ) -> RefNamesIndex {
     let mut index = RefNamesIndex::default();
     for (name, target) in ref_pairs {
-        let decorated_name = if target.has_conflict() {
-            format!("{name}?")
-        } else {
-            name.clone()
+        let ref_name = RefName {
+            name: name.to_owned(),
+            remote: None,
+            conflict: target.has_conflict(),
+            synced: true, // has no tracking remotes
         };
-        index.insert(target.added_ids(), decorated_name);
+        index.insert(target.added_ids(), ref_name);
     }
     index
 }
 
-// TODO: return NameRef?
-fn extract_git_head(repo: &dyn Repo, commit: &Commit) -> String {
+// TODO: maybe add option or nullable type?
+fn extract_git_head(repo: &dyn Repo, commit: &Commit) -> Vec<RefName> {
     let target = repo.view().git_head();
     if target.added_ids().contains(commit.id()) {
-        if target.has_conflict() {
-            "HEAD@git?".to_string()
-        } else {
-            "HEAD@git".to_string()
-        }
+        let ref_name = RefName {
+            name: "HEAD".to_owned(),
+            remote: Some(git::REMOTE_NAME_FOR_LOCAL_GIT_REPO.to_owned()),
+            conflict: target.has_conflict(),
+            synced: false, // has no local counterpart
+        };
+        vec![ref_name]
     } else {
-        "".to_string()
+        vec![]
     }
 }
 
