@@ -6,7 +6,7 @@ use std::process::{Command, ExitStatus, Stdio};
 use std::sync::Arc;
 
 use config::ConfigError;
-use futures::executor::block_on;
+use futures::StreamExt;
 use itertools::Itertools;
 use jj_lib::backend::{FileId, MergedTreeId, TreeValue};
 use jj_lib::conflicts::{self, materialize_merge_result};
@@ -15,10 +15,11 @@ use jj_lib::local_working_copy::{TreeState, TreeStateError};
 use jj_lib::matchers::Matcher;
 use jj_lib::merge::{Merge, MergedTreeValue};
 use jj_lib::merged_tree::{MergedTree, MergedTreeBuilder};
-use jj_lib::repo_path::RepoPath;
+use jj_lib::repo_path::{RepoPath, RepoPathBuf};
 use jj_lib::settings::UserSettings;
 use jj_lib::store::Store;
 use jj_lib::working_copy::{CheckoutError, SnapshotOptions};
+use pollster::FutureExt;
 use regex::{Captures, Regex};
 use tempfile::TempDir;
 use thiserror::Error;
@@ -188,7 +189,7 @@ fn check_out(
     wc_dir: PathBuf,
     state_dir: PathBuf,
     tree: &MergedTree,
-    sparse_patterns: Vec<RepoPath>,
+    sparse_patterns: Vec<RepoPathBuf>,
 ) -> Result<TreeState, DiffCheckoutError> {
     std::fs::create_dir(&wc_dir).map_err(DiffCheckoutError::SetUpDir)?;
     std::fs::create_dir(&state_dir).map_err(DiffCheckoutError::SetUpDir)?;
@@ -211,15 +212,18 @@ fn new_utf8_temp_dir(prefix: &str) -> io::Result<TempDir> {
 fn set_readonly_recursively(path: &Path) -> Result<(), std::io::Error> {
     // Directory permission is unchanged since files under readonly directory cannot
     // be removed.
-    if path.is_dir() {
+    let metadata = path.symlink_metadata()?;
+    if metadata.is_dir() {
         for entry in path.read_dir()? {
             set_readonly_recursively(&entry?.path())?;
         }
         Ok(())
-    } else {
-        let mut perms = std::fs::metadata(path)?.permissions();
+    } else if metadata.is_file() {
+        let mut perms = metadata.permissions();
         perms.set_readonly(true);
         std::fs::set_permissions(path, perms)
+    } else {
+        Ok(())
     }
 }
 
@@ -238,10 +242,11 @@ fn check_out_trees(
     matcher: &dyn Matcher,
     output_is: Option<DiffSide>,
 ) -> Result<DiffWorkingCopies, DiffCheckoutError> {
-    let changed_files = left_tree
-        .diff(right_tree, matcher)
+    let changed_files: Vec<_> = left_tree
+        .diff_stream(right_tree, matcher)
         .map(|(path, _diff)| path)
-        .collect_vec();
+        .collect()
+        .block_on();
 
     let temp_dir = new_utf8_temp_dir("jj-diff-").map_err(DiffCheckoutError::SetUpDir)?;
     let left_wc_dir = temp_dir.path().join("left");
@@ -302,12 +307,12 @@ pub fn run_mergetool_external(
     } else {
         vec![]
     };
-    let (mut removes, mut adds) = content.take();
-    let files: HashMap<&str, _> = maplit::hashmap! {
-        "base" => removes.pop().unwrap().0,
-        "right" => adds.pop().unwrap().0,
-        "left" => adds.pop().unwrap().0,
-        "output" => initial_output_content.clone(),
+    assert_eq!(content.num_sides(), 2);
+    let files: HashMap<&str, &[u8]> = maplit::hashmap! {
+        "base" => content.get_remove(0).unwrap().0.as_slice(),
+        "left" => content.get_add(0).unwrap().0.as_slice(),
+        "right" => content.get_add(1).unwrap().0.as_slice(),
+        "output" => initial_output_content.as_slice(),
     };
 
     let temp_dir = new_utf8_temp_dir("jj-resolve-").map_err(ExternalToolError::SetUpDir)?;
@@ -358,12 +363,13 @@ pub fn run_mergetool_external(
     }
 
     let new_file_ids = if editor.merge_tool_edits_conflict_markers {
-        block_on(conflicts::update_from_content(
+        conflicts::update_from_content(
             &file_merge,
             tree.store(),
             repo_path,
             output_file_contents.as_slice(),
-        ))?
+        )
+        .block_on()?
     } else {
         let new_file_id = tree
             .store()
@@ -378,7 +384,7 @@ pub fn run_mergetool_external(
         Err(new_file_ids) => conflict.with_new_file_ids(&new_file_ids),
     };
     let mut tree_builder = MergedTreeBuilder::new(tree.id());
-    tree_builder.set_or_remove(repo_path.clone(), new_tree_value);
+    tree_builder.set_or_remove(repo_path.to_owned(), new_tree_value);
     let new_tree = tree_builder.write_tree(tree.store())?;
     Ok(new_tree)
 }
