@@ -170,13 +170,13 @@ pub enum PaginationChoice {
 fn pagination_setting(config: &config::Config) -> Result<PaginationChoice, CommandError> {
     config
         .get::<PaginationChoice>("ui.paginate")
-        .map_err(|err| CommandError::ConfigError(format!("Invalid `ui.paginate`: {err:?}")))
+        .map_err(|err| CommandError::ConfigError(format!("Invalid `ui.paginate`: {err}")))
 }
 
 fn pager_setting(config: &config::Config) -> Result<CommandNameAndArgs, CommandError> {
     config
         .get::<CommandNameAndArgs>("ui.pager")
-        .map_err(|err| CommandError::ConfigError(format!("Invalid `ui.pager`: {err:?}")))
+        .map_err(|err| CommandError::ConfigError(format!("Invalid `ui.pager`: {err}")))
 }
 
 impl Ui {
@@ -218,14 +218,15 @@ impl Ui {
         match self.output {
             UiOutput::Terminal { .. } if io::stdout().is_terminal() => {
                 match UiOutput::new_paged(&self.pager_cmd) {
-                    Ok(new_output) => {
-                        self.output = new_output;
+                    Ok(pager_output) => {
+                        self.output = pager_output;
                     }
                     Err(e) => {
+                        // The pager executable couldn't be found or couldn't be run
                         writeln!(
                             self.warning(),
-                            "Failed to spawn pager '{cmd}': {e}",
-                            cmd = self.pager_cmd,
+                            "Failed to spawn pager '{name}': {e}",
+                            name = self.pager_cmd.split_name(),
                         )
                         .ok();
                     }
