@@ -12,6 +12,8 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::fs;
+
 use itertools::Itertools as _;
 
 use crate::common::TestEnvironment;
@@ -69,12 +71,40 @@ fn test_alias_bad_name() {
     insta::assert_snapshot!(stderr, @r###"
     error: unrecognized subcommand 'foo.'
 
-    Usage: jj [OPTIONS] [COMMAND]
+    Usage: jj [OPTIONS] <COMMAND>
 
     For more information, try '--help'.
     "###);
 }
 
+#[test]
+fn test_alias_calls_empty_command() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.add_config(
+        r#"
+    aliases.empty = []
+    aliases.empty_command_with_opts = ["--no-pager"]
+    "#,
+    );
+    let stderr = test_env.jj_cmd_cli_error(&repo_path, &["empty"]);
+    insta::assert_snapshot!(stderr.lines().take(3).join("\n"), @r###"
+    Jujutsu (An experimental VCS)
+
+    Usage: jj [OPTIONS] <COMMAND>
+    "###);
+    let stderr = test_env.jj_cmd_cli_error(&repo_path, &["empty", "--no-pager"]);
+    insta::assert_snapshot!(stderr.lines().next().unwrap_or_default(), @r###"
+    error: 'jj' requires a subcommand but one was not provided
+    "###);
+    let stderr = test_env.jj_cmd_cli_error(&repo_path, &["empty_command_with_opts"]);
+    insta::assert_snapshot!(stderr.lines().next().unwrap_or_default(), @r###"
+    error: 'jj' requires a subcommand but one was not provided
+    "###);
+}
+
 #[test]
 fn test_alias_calls_unknown_command() {
     let test_env = TestEnvironment::default();
@@ -88,7 +118,7 @@ fn test_alias_calls_unknown_command() {
 
       tip: a similar subcommand exists: 'next'
 
-    Usage: jj [OPTIONS] [COMMAND]
+    Usage: jj [OPTIONS] <COMMAND>
 
     For more information, try '--help'.
     "###);
@@ -125,7 +155,7 @@ fn test_alias_calls_help() {
 
     To get started, see the tutorial at https://github.com/martinvonz/jj/blob/main/docs/tutorial.md.
 
-    Usage: jj [OPTIONS] [COMMAND]
+    Usage: jj [OPTIONS] <COMMAND>
     "###);
 }
 
@@ -238,3 +268,75 @@ fn test_alias_invalid_definition() {
     Error: Alias definition for "non-string-list" must be a string list
     "###);
 }
+
+#[test]
+fn test_alias_in_repo_config() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo1", "--git"]);
+    let repo1_path = test_env.env_root().join("repo1");
+    fs::create_dir(repo1_path.join("sub")).unwrap();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo2", "--git"]);
+    let repo2_path = test_env.env_root().join("repo2");
+    fs::create_dir(repo2_path.join("sub")).unwrap();
+
+    test_env.add_config(r#"aliases.l = ['log', '-r@', '--no-graph', '-T"user alias\n"']"#);
+    fs::write(
+        repo1_path.join(".jj/repo/config.toml"),
+        r#"aliases.l = ['log', '-r@', '--no-graph', '-T"repo1 alias\n"']"#,
+    )
+    .unwrap();
+
+    // In repo1 sub directory, aliases can be loaded from the repo1 config.
+    let stdout = test_env.jj_cmd_success(&repo1_path.join("sub"), &["l"]);
+    insta::assert_snapshot!(stdout, @r###"
+    repo1 alias
+    "###);
+
+    // In repo2 directory, no repo-local aliases exist.
+    let stdout = test_env.jj_cmd_success(&repo2_path, &["l"]);
+    insta::assert_snapshot!(stdout, @r###"
+    user alias
+    "###);
+
+    // Aliases can't be loaded from the -R path due to chicken and egg problem.
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(&repo2_path, &["l", "-R", repo1_path.to_str().unwrap()]);
+    insta::assert_snapshot!(stdout, @r###"
+    user alias
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Command aliases cannot be loaded from -R/--repository path
+    "###);
+
+    // Aliases are loaded from the cwd-relative workspace even with -R.
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(&repo1_path, &["l", "-R", repo2_path.to_str().unwrap()]);
+    insta::assert_snapshot!(stdout, @r###"
+    repo1 alias
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Command aliases cannot be loaded from -R/--repository path
+    "###);
+
+    // No warning if the expanded command is identical.
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(&repo1_path, &["files", "-R", repo2_path.to_str().unwrap()]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @"");
+
+    // Config loaded from the cwd-relative workspace shouldn't persist. It's
+    // used only for command arguments expansion.
+    let stdout = test_env.jj_cmd_success(
+        &repo1_path,
+        &[
+            "config",
+            "list",
+            "aliases",
+            "-R",
+            repo2_path.to_str().unwrap(),
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    aliases.l=["log", "-r@", "--no-graph", "-T\"user alias\\n\""]
+    "###);
+}
