@@ -74,7 +74,7 @@ fn test_branch_at_root() {
     insta::assert_snapshot!(stderr, @r###"
     Nothing changed.
     Failed to export some branches:
-      fred
+      fred: Ref cannot point to the root commit in Git
     "###);
 }
 
@@ -189,6 +189,29 @@ fn test_branch_move_conflicting() {
     "###);
 }
 
+#[test]
+fn test_branch_rename() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "rename", "foo", "bar"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: No such branch: foo
+    "###);
+
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "foo"]);
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["branch", "rename", "foo", "bar"]);
+    insta::assert_snapshot!(stderr, @"");
+
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "conflictfoo"]);
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["branch", "rename", "bar", "conflictfoo"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Branch already exists: conflictfoo
+    "###);
+}
+
 #[test]
 fn test_branch_forget_glob() {
     let test_env = TestEnvironment::default();
@@ -333,16 +356,13 @@ fn test_branch_delete_glob() {
       @origin: qpvuntsm 6fbf398c (empty) commit
     foo-1 (deleted)
       @origin: qpvuntsm 6fbf398c (empty) commit
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     foo-3 (deleted)
       @origin: qpvuntsm 6fbf398c (empty) commit
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     foo-4 (deleted)
       @origin: qpvuntsm 6fbf398c (empty) commit
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     // Malformed glob
@@ -433,6 +453,7 @@ fn test_branch_forget_fetched_branch() {
 
     // Set up a git repo with a branch and a jj repo that has it as a remote.
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     let git_repo_path = test_env.env_root().join("git-repo");
@@ -548,6 +569,7 @@ fn test_branch_forget_deleted_or_nonexistent_branch() {
     // ======== Beginning of test setup ========
     // Set up a git repo with a branch and a jj repo that has it as a remote.
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     let git_repo_path = test_env.env_root().join("git-repo");
@@ -583,8 +605,7 @@ fn test_branch_forget_deleted_or_nonexistent_branch() {
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     feature1 (deleted)
       @origin: mzyxwzks 9f01a0e0 message
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     // ============ End of test setup ============
@@ -812,7 +833,7 @@ fn test_branch_track_untrack_patterns() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
     insta::assert_snapshot!(
         test_env.jj_cmd_cli_error(&repo_path, &["branch", "track", "main"]), @r###"
-    error: invalid value 'main' for '<NAMES>...': remote branch must be specified in branch@remote form
+    error: invalid value 'main' for '<BRANCH@REMOTE>...': remote branch must be specified in branch@remote form
 
     For more information, try '--help'.
     "###);
@@ -904,6 +925,7 @@ fn test_branch_track_untrack_patterns() {
 #[test]
 fn test_branch_list() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
 
     // Initialize remote refs
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "remote", "--git"]);
@@ -947,8 +969,7 @@ fn test_branch_list() {
     local-only: wqnwkozp 4e887f78 (empty) local-only
     remote-delete (deleted)
       @origin: mnmymoky 203e60eb (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-sync: zwtyzrop c761c7ea (empty) remote-sync
     remote-unsync: wqnwkozp 4e887f78 (empty) local-only
       @origin (ahead by 1 commits, behind by 1 commits): qpsqxpyq 38ef8af7 (empty) remote-unsync
@@ -959,8 +980,7 @@ fn test_branch_list() {
     local-only: wqnwkozp 4e887f78 (empty) local-only
     remote-delete (deleted)
       @origin: mnmymoky 203e60eb (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-sync: zwtyzrop c761c7ea (empty) remote-sync
       @origin: zwtyzrop c761c7ea (empty) remote-sync
     remote-unsync: wqnwkozp 4e887f78 (empty) local-only
@@ -972,6 +992,7 @@ fn test_branch_list() {
 #[test]
 fn test_branch_list_filtered() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
 
     // Initialize remote refs
@@ -1023,8 +1044,7 @@ fn test_branch_list_filtered() {
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     remote-delete (deleted)
       @origin: yxusvupt dad5f298 (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-keep: nlwprzpn 911e9120 (empty) remote-keep
     remote-rewrite: xyxluytn e31634b6 (empty) rewritten
       @origin (ahead by 1 commits, behind by 1 commits): xyxluytn hidden 3e9a5af6 (empty) remote-rewrite
@@ -1067,8 +1087,7 @@ fn test_branch_list_filtered() {
     insta::assert_snapshot!(query(&["remote-delete"]), @r###"
     remote-delete (deleted)
       @origin: yxusvupt dad5f298 (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
     insta::assert_snapshot!(query(&["-rbranches(remote-delete)"]), @r###"
     "###);
@@ -1082,8 +1101,7 @@ fn test_branch_list_filtered() {
     local-keep: kpqxywon c7b4c09c (empty) local-keep
     remote-delete (deleted)
       @origin: yxusvupt dad5f298 (empty) remote-delete
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     remote-keep: nlwprzpn 911e9120 (empty) remote-keep
     "###);
 
