@@ -12,28 +12,13 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::{self, PathBuf};
+use std::path::{self, Path, PathBuf};
 
 use crate::common::TestEnvironment;
 
 pub mod common;
 
-#[test]
-fn test_git_clone() {
-    let test_env = TestEnvironment::default();
-    let git_repo_path = test_env.env_root().join("source");
-    let git_repo = git2::Repository::init(git_repo_path).unwrap();
-
-    // Clone an empty repo
-    let (stdout, stderr) =
-        test_env.jj_cmd_ok(test_env.env_root(), &["git", "clone", "source", "empty"]);
-    insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @r###"
-    Fetching into new repo in "$TEST_ENV/empty"
-    Nothing changed.
-    "###);
-
-    // Set-up a non-empty repo
+fn set_up_non_empty_git_repo(git_repo: &git2::Repository) {
     let signature =
         git2::Signature::new("Some One", "some.one@example.com", &git2::Time::new(0, 0)).unwrap();
     let mut tree_builder = git_repo.treebuilder(None).unwrap();
@@ -54,6 +39,25 @@ fn test_git_clone() {
         )
         .unwrap();
     git_repo.set_head("refs/heads/main").unwrap();
+}
+
+#[test]
+fn test_git_clone() {
+    let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
+    let git_repo_path = test_env.env_root().join("source");
+    let git_repo = git2::Repository::init(git_repo_path).unwrap();
+
+    // Clone an empty repo
+    let (stdout, stderr) =
+        test_env.jj_cmd_ok(test_env.env_root(), &["git", "clone", "source", "empty"]);
+    insta::assert_snapshot!(stdout, @"");
+    insta::assert_snapshot!(stderr, @r###"
+    Fetching into new repo in "$TEST_ENV/empty"
+    Nothing changed.
+    "###);
+
+    set_up_non_empty_git_repo(&git_repo);
 
     // Clone with relative source path
     let (stdout, stderr) =
@@ -86,7 +90,7 @@ fn test_git_clone() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Fetching into new repo in "$TEST_ENV/failed"
-    Error: could not find repository from '$TEST_ENV/bad'; class=Repository (6)
+    Error: could not find repository at '$TEST_ENV/bad'; class=Repository (6)
     "###);
     assert!(!test_env.env_root().join("failed").exists());
 
@@ -101,7 +105,7 @@ fn test_git_clone() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Fetching into new repo in "$TEST_ENV/failed"
-    Error: could not find repository from '$TEST_ENV/bad'; class=Repository (6)
+    Error: could not find repository at '$TEST_ENV/bad'; class=Repository (6)
     "###);
     assert!(test_env.env_root().join("failed").exists());
     assert!(!test_env.env_root().join("failed").join(".jj").exists());
@@ -137,6 +141,7 @@ fn test_git_clone() {
 #[test]
 fn test_git_clone_colocate() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     let git_repo_path = test_env.env_root().join("source");
     let git_repo = git2::Repository::init(git_repo_path).unwrap();
 
@@ -160,27 +165,7 @@ fn test_git_clone_colocate() {
         git_target_file_contents.replace(path::MAIN_SEPARATOR, "/"),
         @"../../../.git");
 
-    // Set-up a non-empty repo
-    let signature =
-        git2::Signature::new("Some One", "some.one@example.com", &git2::Time::new(0, 0)).unwrap();
-    let mut tree_builder = git_repo.treebuilder(None).unwrap();
-    let file_oid = git_repo.blob(b"content").unwrap();
-    tree_builder
-        .insert("file", file_oid, git2::FileMode::Blob.into())
-        .unwrap();
-    let tree_oid = tree_builder.write().unwrap();
-    let tree = git_repo.find_tree(tree_oid).unwrap();
-    git_repo
-        .commit(
-            Some("refs/heads/main"),
-            &signature,
-            &signature,
-            "message",
-            &tree,
-            &[],
-        )
-        .unwrap();
-    git_repo.set_head("refs/heads/main").unwrap();
+    set_up_non_empty_git_repo(&git_repo);
 
     // Clone with relative source path
     let (stdout, stderr) = test_env.jj_cmd_ok(
@@ -224,17 +209,14 @@ fn test_git_clone_colocate() {
         .map(|entry| format!("{:?} {}\n", entry.status(), entry.path().unwrap()))
         .collect();
     insta::assert_snapshot!(git_statuses, @r###"
-    IGNORED .jj/.gitignore
-    IGNORED .jj/repo/
-    IGNORED .jj/working_copy/
+    Status(IGNORED) .jj/.gitignore
+    Status(IGNORED) .jj/repo/
+    Status(IGNORED) .jj/working_copy/
     "###);
 
     // The old default branch "master" shouldn't exist.
-    let stdout = test_env.jj_cmd_success(
-        &test_env.env_root().join("clone"),
-        &["branch", "list", "--all"],
-    );
-    insta::assert_snapshot!(stdout, @r###"
+    insta::assert_snapshot!(
+        get_branch_output(&test_env, &test_env.env_root().join("clone")), @r###"
     main: mzyxwzks 9f01a0e0 message
       @git: mzyxwzks 9f01a0e0 message
       @origin: mzyxwzks 9f01a0e0 message
@@ -262,7 +244,7 @@ fn test_git_clone_colocate() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Fetching into new repo in "$TEST_ENV/failed"
-    Error: could not find repository from '$TEST_ENV/bad'; class=Repository (6)
+    Error: could not find repository at '$TEST_ENV/bad'; class=Repository (6)
     "###);
     assert!(!test_env.env_root().join("failed").exists());
 
@@ -280,7 +262,7 @@ fn test_git_clone_colocate() {
     insta::assert_snapshot!(stdout, @"");
     insta::assert_snapshot!(stderr, @r###"
     Fetching into new repo in "$TEST_ENV/failed"
-    Error: could not find repository from '$TEST_ENV/bad'; class=Repository (6)
+    Error: could not find repository at '$TEST_ENV/bad'; class=Repository (6)
     "###);
     assert!(test_env.env_root().join("failed").exists());
     assert!(!test_env.env_root().join("failed").join(".git").exists());
@@ -326,3 +308,59 @@ fn test_git_clone_colocate() {
     Error: Destination path exists and is not an empty directory
     "###);
 }
+
+#[test]
+fn test_git_clone_remote_default_branch() {
+    let test_env = TestEnvironment::default();
+    let git_repo_path = test_env.env_root().join("source");
+    let git_repo = git2::Repository::init(git_repo_path).unwrap();
+    set_up_non_empty_git_repo(&git_repo);
+    // Create non-default branch in remote
+    let oid = git_repo
+        .find_reference("refs/heads/main")
+        .unwrap()
+        .target()
+        .unwrap();
+    git_repo
+        .reference("refs/heads/feature1", oid, false, "")
+        .unwrap();
+
+    // All fetched branches will be imported if auto-local-branch is on
+    test_env.add_config("git.auto-local-branch = true");
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(test_env.env_root(), &["git", "clone", "source", "clone1"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Fetching into new repo in "$TEST_ENV/clone1"
+    Working copy now at: sqpuoqvx cad212e1 (empty) (no description set)
+    Parent commit      : mzyxwzks 9f01a0e0 feature1 main | message
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+    insta::assert_snapshot!(
+        get_branch_output(&test_env, &test_env.env_root().join("clone1")), @r###"
+    feature1: mzyxwzks 9f01a0e0 message
+      @origin: mzyxwzks 9f01a0e0 message
+    main: mzyxwzks 9f01a0e0 message
+      @origin: mzyxwzks 9f01a0e0 message
+    "###);
+
+    // Only the default branch will be imported if auto-local-branch is off
+    test_env.add_config("git.auto-local-branch = false");
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(test_env.env_root(), &["git", "clone", "source", "clone2"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Fetching into new repo in "$TEST_ENV/clone2"
+    Working copy now at: pmmvwywv fa729b1e (empty) (no description set)
+    Parent commit      : mzyxwzks 9f01a0e0 feature1@origin main | message
+    Added 1 files, modified 0 files, removed 0 files
+    "###);
+    insta::assert_snapshot!(
+        get_branch_output(&test_env, &test_env.env_root().join("clone2")), @r###"
+    feature1@origin: mzyxwzks 9f01a0e0 message
+    main: mzyxwzks 9f01a0e0 message
+      @origin: mzyxwzks 9f01a0e0 message
+    "###);
+}
+
+fn get_branch_output(test_env: &TestEnvironment, repo_path: &Path) -> String {
+    test_env.jj_cmd_success(repo_path, &["branch", "list", "--all"])
+}
