@@ -73,9 +73,23 @@ fn get_log_output(test_env: &TestEnvironment, workspace_root: &Path) -> String {
     test_env.jj_cmd_success(workspace_root, &["log", "-T", template, "-r", "all()"])
 }
 
+#[test]
+fn test_git_fetch_with_default_config() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    add_git_remote(&test_env, &repo_path, "origin");
+
+    test_env.jj_cmd_ok(&repo_path, &["git", "fetch"]);
+    insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
+    origin@origin: oputwtnw ffecd2d6 message
+    "###);
+}
+
 #[test]
 fn test_git_fetch_default_remote() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
@@ -90,6 +104,7 @@ fn test_git_fetch_default_remote() {
 #[test]
 fn test_git_fetch_single_remote() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -108,6 +123,7 @@ fn test_git_fetch_single_remote() {
 #[test]
 fn test_git_fetch_single_remote_all_remotes_flag() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -125,6 +141,7 @@ fn test_git_fetch_single_remote_all_remotes_flag() {
 #[test]
 fn test_git_fetch_single_remote_from_arg() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -139,6 +156,7 @@ fn test_git_fetch_single_remote_from_arg() {
 #[test]
 fn test_git_fetch_single_remote_from_config() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -154,6 +172,7 @@ fn test_git_fetch_single_remote_from_config() {
 #[test]
 fn test_git_fetch_multiple_remotes() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -174,6 +193,7 @@ fn test_git_fetch_multiple_remotes() {
 #[test]
 fn test_git_fetch_all_remotes() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -191,6 +211,7 @@ fn test_git_fetch_all_remotes() {
 #[test]
 fn test_git_fetch_multiple_remotes_from_config() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -243,6 +264,7 @@ fn test_git_fetch_nonexistent_remote_from_config() {
 #[test]
 fn test_git_fetch_from_remote_named_git() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     let repo_path = test_env.env_root().join("repo");
     init_git_remote(&test_env, "git");
     let git_repo = git2::Repository::init(&repo_path).unwrap();
@@ -286,6 +308,7 @@ fn test_git_fetch_from_remote_named_git() {
 #[test]
 fn test_git_fetch_prune_before_updating_tips() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
@@ -313,6 +336,7 @@ fn test_git_fetch_prune_before_updating_tips() {
 #[test]
 fn test_git_fetch_conflicting_branches() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "rem1");
@@ -340,6 +364,7 @@ fn test_git_fetch_conflicting_branches() {
 #[test]
 fn test_git_fetch_conflicting_branches_colocated() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     let repo_path = test_env.env_root().join("repo");
     let _git_repo = git2::Repository::init(&repo_path).unwrap();
     // create_colocated_repo_and_branches_from_trunk1(&test_env, &repo_path);
@@ -402,6 +427,7 @@ fn create_trunk2_and_rebase_branches(test_env: &TestEnvironment, repo_path: &Pat
 #[test]
 fn test_git_fetch_all() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
@@ -541,6 +567,7 @@ fn test_git_fetch_all() {
 #[test]
 fn test_git_fetch_some_of_many_branches() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
@@ -750,6 +777,7 @@ fn test_git_fetch_some_of_many_branches() {
 #[test]
 fn test_git_fetch_undo() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
 
@@ -819,6 +847,7 @@ fn test_git_fetch_undo() {
 #[test]
 fn test_fetch_undo_what() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
 
@@ -876,8 +905,7 @@ fn test_fetch_undo_what() {
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     b (deleted)
       @origin: vpupmnsl hidden c7d4bdcb descr_for_b
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     "###);
 
     // Now, let's demo restoring just the remote-tracking branch. First, let's
@@ -886,8 +914,7 @@ fn test_fetch_undo_what() {
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     b (deleted)
       @origin: vpupmnsl hidden c7d4bdcb descr_for_b
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     newbranch: qpvuntsm 230dd059 (empty) (no description set)
     "###);
     // Restoring just the remote-tracking state will not affect `newbranch`, but
@@ -912,6 +939,7 @@ fn test_fetch_undo_what() {
 #[test]
 fn test_git_fetch_remove_fetch() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
@@ -953,6 +981,7 @@ fn test_git_fetch_remove_fetch() {
 #[test]
 fn test_git_fetch_rename_fetch() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
     add_git_remote(&test_env, &repo_path, "origin");
@@ -993,6 +1022,7 @@ fn test_git_fetch_rename_fetch() {
 #[test]
 fn test_git_fetch_removed_branch() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
 
@@ -1078,6 +1108,7 @@ fn test_git_fetch_removed_branch() {
 #[test]
 fn test_git_fetch_removed_parent_branch() {
     let test_env = TestEnvironment::default();
+    test_env.add_config("git.auto-local-branch = true");
     let source_git_repo_path = test_env.env_root().join("source");
     let _git_repo = git2::Repository::init(source_git_repo_path.clone()).unwrap();
 
@@ -1181,7 +1212,8 @@ fn test_git_fetch_remote_only_branch() {
         )
         .unwrap();
 
-    // Fetch normally
+    // Fetch using git.auto_local_branch = true
+    test_env.add_config("git.auto-local-branch = true");
     test_env.jj_cmd_ok(&repo_path, &["git", "fetch", "--remote=origin"]);
     insta::assert_snapshot!(get_branch_output(&test_env, &repo_path), @r###"
     feature1: mzyxwzks 9f01a0e0 message
