@@ -39,6 +39,7 @@ fn set_up() -> (TestEnvironment, PathBuf) {
         &[
             "git",
             "clone",
+            "--config-toml=git.auto-local-branch=true",
             origin_git_repo_path.to_str().unwrap(),
             "local",
         ],
@@ -280,8 +281,7 @@ fn test_git_push_multiple() {
     insta::assert_snapshot!(stdout, @r###"
     branch1 (deleted)
       @origin: lzmmnrxq 45a3aa29 (empty) description 1
-      (this branch will be *deleted permanently* on the remote on the
-       next `jj git push`. Use `jj branch forget` to prevent this)
+      (this branch will be *deleted permanently* on the remote on the next `jj git push`. Use `jj branch forget` to prevent this)
     branch2: yqosqzyt 15dcdaa4 (empty) foo
       @origin (ahead by 1 commits, behind by 1 commits): rlzusymt 8476341e (empty) description 2
     my-branch: yqosqzyt 15dcdaa4 (empty) foo
@@ -682,6 +682,7 @@ fn test_git_push_deleted() {
 #[test]
 fn test_git_push_conflicting_branches() {
     let (test_env, workspace_root) = set_up();
+    test_env.add_config("git.auto-local-branch = true");
     let git_repo = {
         let mut git_repo_path = workspace_root.clone();
         git_repo_path.extend([".jj", "repo", "store", "git"]);
