@@ -385,14 +385,17 @@ pub fn canonicalize_git_repo_path(path: &Path) -> io::Result<PathBuf> {
 fn gix_open_opts_from_settings(settings: &UserSettings) -> gix::open::Options {
     let user_name = settings.user_name();
     let user_email = settings.user_email();
-    gix::open::Options::default().config_overrides([
-        // Committer has to be configured to record reflog. Author isn't
-        // needed, but let's copy the same values.
-        format!("author.name={user_name}"),
-        format!("author.email={user_email}"),
-        format!("committer.name={user_name}"),
-        format!("committer.email={user_email}"),
-    ])
+    gix::open::Options::default()
+        .config_overrides([
+            // Committer has to be configured to record reflog. Author isn't
+            // needed, but let's copy the same values.
+            format!("author.name={user_name}"),
+            format!("author.email={user_email}"),
+            format!("committer.name={user_name}"),
+            format!("committer.email={user_email}"),
+        ])
+        // The git_target path should point the repository, not the working directory.
+        .open_path_as_is(true)
 }
 
 fn commit_from_git_without_root_parent(
@@ -748,12 +751,12 @@ impl Backend for GitBackend {
             let entry = entry.map_err(|err| to_read_object_err(err, id))?;
             let name =
                 str::from_utf8(entry.filename()).map_err(|err| to_invalid_utf8_err(err, id))?;
-            let (name, value) = match entry.mode() {
-                gix::object::tree::EntryMode::Tree => {
+            let (name, value) = match entry.mode().kind() {
+                gix::object::tree::EntryKind::Tree => {
                     let id = TreeId::from_bytes(entry.oid().as_bytes());
                     (name, TreeValue::Tree(id))
                 }
-                gix::object::tree::EntryMode::Blob => {
+                gix::object::tree::EntryKind::Blob => {
                     let id = FileId::from_bytes(entry.oid().as_bytes());
                     if let Some(basename) = name.strip_suffix(CONFLICT_SUFFIX) {
                         (
@@ -770,7 +773,7 @@ impl Backend for GitBackend {
                         )
                     }
                 }
-                gix::object::tree::EntryMode::BlobExecutable => {
+                gix::object::tree::EntryKind::BlobExecutable => {
                     let id = FileId::from_bytes(entry.oid().as_bytes());
                     (
                         name,
@@ -780,11 +783,11 @@ impl Backend for GitBackend {
                         },
                     )
                 }
-                gix::object::tree::EntryMode::Link => {
+                gix::object::tree::EntryKind::Link => {
                     let id = SymlinkId::from_bytes(entry.oid().as_bytes());
                     (name, TreeValue::Symlink(id))
                 }
-                gix::object::tree::EntryMode::Commit => {
+                gix::object::tree::EntryKind::Commit => {
                     let id = CommitId::from_bytes(entry.oid().as_bytes());
                     (name, TreeValue::GitSubmodule(id))
                 }
@@ -806,7 +809,7 @@ impl Backend for GitBackend {
                         id,
                         executable: false,
                     } => gix::objs::tree::Entry {
-                        mode: gix::object::tree::EntryMode::Blob,
+                        mode: gix::object::tree::EntryKind::Blob.into(),
                         filename: name.into(),
                         oid: id.as_bytes().into(),
                     },
@@ -814,27 +817,27 @@ impl Backend for GitBackend {
                         id,
                         executable: true,
                     } => gix::objs::tree::Entry {
-                        mode: gix::object::tree::EntryMode::BlobExecutable,
+                        mode: gix::object::tree::EntryKind::BlobExecutable.into(),
                         filename: name.into(),
                         oid: id.as_bytes().into(),
                     },
                     TreeValue::Symlink(id) => gix::objs::tree::Entry {
-                        mode: gix::object::tree::EntryMode::Link,
+                        mode: gix::object::tree::EntryKind::Link.into(),
                         filename: name.into(),
                         oid: id.as_bytes().into(),
                     },
                     TreeValue::Tree(id) => gix::objs::tree::Entry {
-                        mode: gix::object::tree::EntryMode::Tree,
+                        mode: gix::object::tree::EntryKind::Tree.into(),
                         filename: name.into(),
                         oid: id.as_bytes().into(),
                     },
                     TreeValue::GitSubmodule(id) => gix::objs::tree::Entry {
-                        mode: gix::object::tree::EntryMode::Commit,
+                        mode: gix::object::tree::EntryKind::Commit.into(),
                         filename: name.into(),
                         oid: id.as_bytes().into(),
                     },
                     TreeValue::Conflict(id) => gix::objs::tree::Entry {
-                        mode: gix::object::tree::EntryMode::Blob,
+                        mode: gix::object::tree::EntryKind::Blob.into(),
                         filename: (name.to_owned() + CONFLICT_SUFFIX).into(),
                         oid: id.as_bytes().into(),
                     },
@@ -1062,7 +1065,7 @@ fn write_tree_conflict(
             .map(|(i, tree_id)| (format!(".jjconflict-side-{i}"), tree_id)),
     )
     .map(|(name, tree_id)| gix::objs::tree::Entry {
-        mode: gix::object::tree::EntryMode::Tree,
+        mode: gix::object::tree::EntryKind::Tree.into(),
         filename: name.into(),
         oid: tree_id.as_bytes().into(),
     })
@@ -1167,7 +1170,7 @@ mod tests {
         let temp_dir = testutils::new_temp_dir();
         let store_path = temp_dir.path();
         let git_repo_path = temp_dir.path().join("git");
-        let git_repo = git2::Repository::init(&git_repo_path).unwrap();
+        let git_repo = git2::Repository::init(git_repo_path).unwrap();
 
         // Add a commit with some files in
         let blob1 = git_repo.blob(b"content1").unwrap();
@@ -1223,7 +1226,7 @@ mod tests {
             .unwrap();
         let commit_id2 = CommitId::from_bytes(git_commit_id2.as_bytes());
 
-        let backend = GitBackend::init_external(&settings, store_path, &git_repo_path).unwrap();
+        let backend = GitBackend::init_external(&settings, store_path, git_repo.path()).unwrap();
 
         // Import the head commit and its ancestors
         backend
@@ -1329,7 +1332,7 @@ mod tests {
         let temp_dir = testutils::new_temp_dir();
         let store_path = temp_dir.path();
         let git_repo_path = temp_dir.path().join("git");
-        let git_repo = git2::Repository::init(&git_repo_path).unwrap();
+        let git_repo = git2::Repository::init(git_repo_path).unwrap();
 
         let signature = git2::Signature::now("Someone", "someone@example.com").unwrap();
         let empty_tree_id = Oid::from_str("4b825dc642cb6eb9a060e54bf8d69288fbee4904").unwrap();
@@ -1345,7 +1348,7 @@ mod tests {
             )
             .unwrap();
 
-        let backend = GitBackend::init_external(&settings, store_path, &git_repo_path).unwrap();
+        let backend = GitBackend::init_external(&settings, store_path, git_repo.path()).unwrap();
 
         // read_commit() without import_head_commits() works as of now. This might be
         // changed later.
@@ -1369,7 +1372,7 @@ mod tests {
         let temp_dir = testutils::new_temp_dir();
         let store_path = temp_dir.path();
         let git_repo_path = temp_dir.path().join("git");
-        let git_repo = git2::Repository::init(&git_repo_path).unwrap();
+        let git_repo = git2::Repository::init(git_repo_path).unwrap();
 
         let signature = git2::Signature::now("Someone", "someone@example.com").unwrap();
         let empty_tree_id = Oid::from_str("4b825dc642cb6eb9a060e54bf8d69288fbee4904").unwrap();
@@ -1394,7 +1397,7 @@ mod tests {
             .commit_signed(commit_buf, secure_sig, None)
             .unwrap();
 
-        let backend = GitBackend::init_external(&settings, store_path, &git_repo_path).unwrap();
+        let backend = GitBackend::init_external(&settings, store_path, git_repo.path()).unwrap();
 
         let commit = backend
             .read_commit(&CommitId::from_bytes(git_commit_id.as_bytes()))
@@ -1461,9 +1464,9 @@ mod tests {
         let temp_dir = testutils::new_temp_dir();
         let store_path = temp_dir.path();
         let git_repo_path = temp_dir.path().join("git");
-        let git_repo = git2::Repository::init(&git_repo_path).unwrap();
+        let git_repo = git2::Repository::init(git_repo_path).unwrap();
 
-        let backend = GitBackend::init_external(&settings, store_path, &git_repo_path).unwrap();
+        let backend = GitBackend::init_external(&settings, store_path, git_repo.path()).unwrap();
         let mut commit = Commit {
             parents: vec![],
             predecessors: vec![],
@@ -1526,9 +1529,9 @@ mod tests {
         let temp_dir = testutils::new_temp_dir();
         let store_path = temp_dir.path();
         let git_repo_path = temp_dir.path().join("git");
-        let git_repo = git2::Repository::init(&git_repo_path).unwrap();
+        let git_repo = git2::Repository::init(git_repo_path).unwrap();
 
-        let backend = GitBackend::init_external(&settings, store_path, &git_repo_path).unwrap();
+        let backend = GitBackend::init_external(&settings, store_path, git_repo.path()).unwrap();
         let create_tree = |i| {
             let blob_id = git_repo.blob(b"content {i}").unwrap();
             let mut tree_builder = git_repo.treebuilder(None).unwrap();
