@@ -27,18 +27,26 @@ use crate::operation::Operation;
 use crate::revset::{ResolvedExpression, Revset, RevsetEvaluationError};
 use crate::store::Store;
 
+/// Error while reading index from the `IndexStore`.
 #[derive(Debug, Error)]
-pub enum IndexWriteError {
-    #[error("{0}")]
-    Other(String),
-}
+#[error(transparent)]
+pub struct IndexReadError(pub Box<dyn std::error::Error + Send + Sync>);
+
+/// Error while writing index to the `IndexStore`.
+#[derive(Debug, Error)]
+#[error(transparent)]
+pub struct IndexWriteError(pub Box<dyn std::error::Error + Send + Sync>);
 
 pub trait IndexStore: Send + Sync + Debug {
     fn as_any(&self) -> &dyn Any;
 
     fn name(&self) -> &str;
 
-    fn get_index_at_op(&self, op: &Operation, store: &Arc<Store>) -> Box<dyn ReadonlyIndex>;
+    fn get_index_at_op(
+        &self,
+        op: &Operation,
+        store: &Arc<Store>,
+    ) -> Result<Box<dyn ReadonlyIndex>, IndexReadError>;
 
     fn write_index(
         &self,
@@ -50,7 +58,7 @@ pub trait IndexStore: Send + Sync + Debug {
 pub trait Index: Send + Sync {
     fn shortest_unique_commit_id_prefix_len(&self, commit_id: &CommitId) -> usize;
 
-    fn resolve_prefix(&self, prefix: &HexPrefix) -> PrefixResolution<CommitId>;
+    fn resolve_commit_id_prefix(&self, prefix: &HexPrefix) -> PrefixResolution<CommitId>;
 
     fn has_id(&self, commit_id: &CommitId) -> bool;
 
@@ -75,10 +83,18 @@ pub trait ReadonlyIndex: Send + Sync {
 
     fn as_index(&self) -> &dyn Index;
 
+    // TODO: might be better to split Index::evaluate_revset() to
+    // Readonly/MutableIndex::evaluate_static().
+    fn evaluate_revset_static(
+        &self,
+        expression: &ResolvedExpression,
+        store: &Arc<Store>,
+    ) -> Result<Box<dyn Revset<'static>>, RevsetEvaluationError>;
+
     fn start_modification(&self) -> Box<dyn MutableIndex>;
 }
 
-pub trait MutableIndex: Any {
+pub trait MutableIndex {
     fn as_any(&self) -> &dyn Any;
 
     fn into_any(self: Box<Self>) -> Box<dyn Any>;
@@ -133,6 +149,11 @@ impl HexPrefix {
         &self.min_prefix_bytes
     }
 
+    /// Returns the bytes representation if this prefix can be a full id.
+    pub fn as_full_bytes(&self) -> Option<&[u8]> {
+        (!self.has_odd_byte).then_some(&self.min_prefix_bytes)
+    }
+
     fn split_odd_byte(&self) -> (Option<u8>, &[u8]) {
         if self.has_odd_byte {
             let (&odd, prefix) = self.min_prefix_bytes.split_last().unwrap();
