@@ -16,6 +16,7 @@
 
 #![warn(missing_docs)]
 #![deny(unused_must_use)]
+#![forbid(unsafe_code)]
 
 #[macro_use]
 pub mod content_hash;
@@ -25,7 +26,7 @@ pub mod commit;
 pub mod commit_builder;
 pub mod conflicts;
 pub mod dag_walk;
-pub mod default_index_store;
+pub mod default_index;
 pub mod default_revset_engine;
 pub mod default_revset_graph_iterator;
 pub mod default_submodule_store;
@@ -48,6 +49,7 @@ pub mod merge;
 pub mod merged_tree;
 pub mod op_heads_store;
 pub mod op_store;
+pub mod op_walk;
 pub mod operation;
 #[allow(missing_docs)]
 pub mod protos;
