@@ -28,13 +28,13 @@ use tempfile::NamedTempFile;
 
 use crate::backend::{
     make_root_commit, Backend, BackendError, BackendResult, ChangeId, Commit, CommitId, Conflict,
-    ConflictId, ConflictTerm, FileId, MergedTreeId, MillisSinceEpoch, ObjectId, Signature,
-    SymlinkId, Timestamp, Tree, TreeId, TreeValue,
+    ConflictId, ConflictTerm, FileId, MergedTreeId, MillisSinceEpoch, ObjectId, SecureSig,
+    Signature, SigningFn, SymlinkId, Timestamp, Tree, TreeId, TreeValue,
 };
 use crate::content_hash::blake2b_hash;
 use crate::file_util::persist_content_addressed_temp_file;
 use crate::merge::MergeBuilder;
-use crate::repo_path::{RepoPath, RepoPathComponent};
+use crate::repo_path::{RepoPath, RepoPathComponentBuf};
 
 const COMMIT_ID_LENGTH: usize = 64;
 const CHANGE_ID_LENGTH: usize = 16;
@@ -80,7 +80,7 @@ impl LocalBackend {
         fs::create_dir(store_path.join("conflicts")).unwrap();
         let backend = Self::load(store_path);
         let empty_tree_id = backend
-            .write_tree(&RepoPath::root(), &Tree::default())
+            .write_tree(RepoPath::root(), &Tree::default())
             .unwrap();
         assert_eq!(empty_tree_id, backend.empty_tree_id);
         backend
@@ -149,6 +149,10 @@ impl Backend for LocalBackend {
         &self.empty_tree_id
     }
 
+    fn concurrency(&self) -> usize {
+        1
+    }
+
     async fn read_file(&self, _path: &RepoPath, id: &FileId) -> BackendResult<Box<dyn Read>> {
         let path = self.file_path(id);
         let file = File::open(path).map_err(|err| map_not_found_err(err, id))?;
@@ -260,7 +264,13 @@ impl Backend for LocalBackend {
         Ok(commit_from_proto(proto))
     }
 
-    fn write_commit(&self, commit: Commit) -> BackendResult<(CommitId, Commit)> {
+    fn write_commit(
+        &self,
+        mut commit: Commit,
+        sign_with: Option<&mut SigningFn>,
+    ) -> BackendResult<(CommitId, Commit)> {
+        assert!(commit.secure_sig.is_none(), "commit.secure_sig was set");
+
         if commit.parents.is_empty() {
             return Err(BackendError::Other(
                 "Cannot write a commit with no parents".into(),
@@ -268,7 +278,14 @@ impl Backend for LocalBackend {
         }
         let temp_file = NamedTempFile::new_in(&self.path).map_err(to_other_err)?;
 
-        let proto = commit_to_proto(&commit);
+        let mut proto = commit_to_proto(&commit);
+        if let Some(sign) = sign_with {
+            let data = proto.encode_to_vec();
+            let sig = sign(&data).map_err(to_other_err)?;
+            proto.secure_sig = Some(sig.clone());
+            commit.secure_sig = Some(SecureSig { data, sig });
+        }
+
         temp_file
             .as_file()
             .write_all(&proto.encode_to_vec())
@@ -280,6 +297,10 @@ impl Backend for LocalBackend {
             .map_err(to_other_err)?;
         Ok((id, commit))
     }
+
+    fn gc(&self) -> Result<(), Box<dyn std::error::Error + Send + Sync>> {
+        Ok(())
+    }
 }
 
 pub fn commit_to_proto(commit: &Commit) -> crate::protos::local_store::Commit {
@@ -306,7 +327,14 @@ pub fn commit_to_proto(commit: &Commit) -> crate::protos::local_store::Commit {
     proto
 }
 
-fn commit_from_proto(proto: crate::protos::local_store::Commit) -> Commit {
+fn commit_from_proto(mut proto: crate::protos::local_store::Commit) -> Commit {
+    // Note how .take() sets the secure_sig field to None before we encode the data.
+    // Needs to be done first since proto is partially moved a bunch below
+    let secure_sig = proto.secure_sig.take().map(|sig| SecureSig {
+        data: proto.encode_to_vec(),
+        sig,
+    });
+
     let parents = proto.parents.into_iter().map(CommitId::new).collect();
     let predecessors = proto.predecessors.into_iter().map(CommitId::new).collect();
     let root_tree = if proto.uses_tree_conflict_format {
@@ -325,6 +353,7 @@ fn commit_from_proto(proto: crate::protos::local_store::Commit) -> Commit {
         description: proto.description,
         author: signature_from_proto(proto.author.unwrap_or_default()),
         committer: signature_from_proto(proto.committer.unwrap_or_default()),
+        secure_sig,
     }
 }
 
@@ -332,7 +361,7 @@ fn tree_to_proto(tree: &Tree) -> crate::protos::local_store::Tree {
     let mut proto = crate::protos::local_store::Tree::default();
     for entry in tree.entries() {
         proto.entries.push(crate::protos::local_store::tree::Entry {
-            name: entry.name().string(),
+            name: entry.name().as_str().to_owned(),
             value: Some(tree_value_to_proto(entry.value())),
         });
     }
@@ -343,7 +372,7 @@ fn tree_from_proto(proto: crate::protos::local_store::Tree) -> Tree {
     let mut tree = Tree::default();
     for proto_entry in proto.entries {
         let value = tree_value_from_proto(proto_entry.value.unwrap());
-        tree.set(RepoPathComponent::from(proto_entry.name), value);
+        tree.set(RepoPathComponentBuf::from(proto_entry.name), value);
     }
     tree
 }
@@ -461,7 +490,7 @@ fn conflict_term_to_proto(part: &ConflictTerm) -> crate::protos::local_store::co
 #[cfg(test)]
 mod tests {
     use assert_matches::assert_matches;
-    use futures::executor::block_on;
+    use pollster::FutureExt;
 
     use super::*;
     use crate::backend::MillisSinceEpoch;
@@ -481,37 +510,38 @@ mod tests {
             description: "".to_string(),
             author: create_signature(),
             committer: create_signature(),
+            secure_sig: None,
         };
 
         // No parents
         commit.parents = vec![];
         assert_matches!(
-            backend.write_commit(commit.clone()),
+            backend.write_commit(commit.clone(), None),
             Err(BackendError::Other(err)) if err.to_string().contains("no parents")
         );
 
         // Only root commit as parent
         commit.parents = vec![backend.root_commit_id().clone()];
-        let first_id = backend.write_commit(commit.clone()).unwrap().0;
-        let first_commit = block_on(backend.read_commit(&first_id)).unwrap();
+        let first_id = backend.write_commit(commit.clone(), None).unwrap().0;
+        let first_commit = backend.read_commit(&first_id).block_on().unwrap();
         assert_eq!(first_commit, commit);
 
         // Only non-root commit as parent
         commit.parents = vec![first_id.clone()];
-        let second_id = backend.write_commit(commit.clone()).unwrap().0;
-        let second_commit = block_on(backend.read_commit(&second_id)).unwrap();
+        let second_id = backend.write_commit(commit.clone(), None).unwrap().0;
+        let second_commit = backend.read_commit(&second_id).block_on().unwrap();
         assert_eq!(second_commit, commit);
 
         // Merge commit
         commit.parents = vec![first_id.clone(), second_id.clone()];
-        let merge_id = backend.write_commit(commit.clone()).unwrap().0;
-        let merge_commit = block_on(backend.read_commit(&merge_id)).unwrap();
+        let merge_id = backend.write_commit(commit.clone(), None).unwrap().0;
+        let merge_commit = backend.read_commit(&merge_id).block_on().unwrap();
         assert_eq!(merge_commit, commit);
 
         // Merge commit with root as one parent
         commit.parents = vec![first_id, backend.root_commit_id().clone()];
-        let root_merge_id = backend.write_commit(commit.clone()).unwrap().0;
-        let root_merge_commit = block_on(backend.read_commit(&root_merge_id)).unwrap();
+        let root_merge_id = backend.write_commit(commit.clone(), None).unwrap().0;
+        let root_merge_commit = backend.read_commit(&root_merge_id).block_on().unwrap();
         assert_eq!(root_merge_commit, commit);
     }
 
