@@ -19,7 +19,7 @@ use std::iter;
 
 use tracing::instrument;
 
-use crate::repo_path::{RepoPath, RepoPathComponent};
+use crate::repo_path::{RepoPath, RepoPathComponentBuf};
 
 #[derive(PartialEq, Eq, Debug)]
 pub enum Visit {
@@ -39,7 +39,7 @@ pub enum Visit {
 }
 
 impl Visit {
-    fn sets(dirs: HashSet<RepoPathComponent>, files: HashSet<RepoPathComponent>) -> Self {
+    fn sets(dirs: HashSet<RepoPathComponentBuf>, files: HashSet<RepoPathComponentBuf>) -> Self {
         if dirs.is_empty() && files.is_empty() {
             Self::Nothing
         } else {
@@ -58,13 +58,13 @@ impl Visit {
 #[derive(PartialEq, Eq, Debug)]
 pub enum VisitDirs {
     All,
-    Set(HashSet<RepoPathComponent>),
+    Set(HashSet<RepoPathComponentBuf>),
 }
 
 #[derive(PartialEq, Eq, Debug)]
 pub enum VisitFiles {
     All,
-    Set(HashSet<RepoPathComponent>),
+    Set(HashSet<RepoPathComponentBuf>),
 }
 
 pub trait Matcher: Sync {
@@ -104,10 +104,10 @@ pub struct FilesMatcher {
 }
 
 impl FilesMatcher {
-    pub fn new(files: &[RepoPath]) -> Self {
+    pub fn new(files: impl IntoIterator<Item = impl AsRef<RepoPath>>) -> Self {
         let mut tree = RepoPathTree::new();
         for f in files {
-            tree.add_file(f);
+            tree.add_file(f.as_ref());
         }
         FilesMatcher { tree }
     }
@@ -128,11 +128,11 @@ pub struct PrefixMatcher {
 }
 
 impl PrefixMatcher {
-    #[instrument]
-    pub fn new(prefixes: &[RepoPath]) -> Self {
+    #[instrument(skip(prefixes))]
+    pub fn new(prefixes: impl IntoIterator<Item = impl AsRef<RepoPath>>) -> Self {
         let mut tree = RepoPathTree::new();
         for prefix in prefixes {
-            let sub = tree.add(prefix);
+            let sub = tree.add(prefix.as_ref());
             sub.is_dir = true;
             sub.is_file = true;
         }
@@ -146,13 +146,13 @@ impl Matcher for PrefixMatcher {
     }
 
     fn visit(&self, dir: &RepoPath) -> Visit {
-        for (sub, tail_components) in self.tree.walk_to(dir) {
+        for (sub, tail_path) in self.tree.walk_to(dir) {
             // 'is_file' means the current path matches prefix paths
             if sub.is_file {
                 return Visit::AllRecursively;
             }
             // 'dir' found, and is an ancestor of prefix paths
-            if tail_components.is_empty() {
+            if tail_path.is_root() {
                 return sub.to_visit_sets();
             }
         }
@@ -263,7 +263,7 @@ impl Matcher for IntersectionMatcher<'_> {
 /// visited.
 #[derive(PartialEq, Eq, Debug)]
 struct RepoPathTree {
-    entries: HashMap<RepoPathComponent, RepoPathTree>,
+    entries: HashMap<RepoPathComponentBuf, RepoPathTree>,
     // is_dir/is_file aren't exclusive, both can be set to true. If entries is not empty,
     // is_dir should be set.
     is_dir: bool,
@@ -280,11 +280,11 @@ impl RepoPathTree {
     }
 
     fn add(&mut self, dir: &RepoPath) -> &mut RepoPathTree {
-        dir.components().iter().fold(self, |sub, name| {
+        dir.components().fold(self, |sub, name| {
             // Avoid name.clone() if entry already exists.
             if !sub.entries.contains_key(name) {
                 sub.is_dir = true;
-                sub.entries.insert(name.clone(), RepoPathTree::new());
+                sub.entries.insert(name.to_owned(), RepoPathTree::new());
             }
             sub.entries.get_mut(name).unwrap()
         })
@@ -300,7 +300,6 @@ impl RepoPathTree {
 
     fn get(&self, dir: &RepoPath) -> Option<&RepoPathTree> {
         dir.components()
-            .iter()
             .try_fold(self, |sub, name| sub.entries.get(name))
     }
 
@@ -310,17 +309,17 @@ impl RepoPathTree {
             .unwrap_or(Visit::Nothing)
     }
 
-    fn walk_to<'a>(
+    /// Walks the tree from the root to the given `dir`, yielding each sub tree
+    /// and remaining path.
+    fn walk_to<'a, 'b: 'a>(
         &'a self,
-        dir: &'a RepoPath,
-    ) -> impl Iterator<Item = (&RepoPathTree, &[RepoPathComponent])> + 'a {
-        iter::successors(
-            Some((self, dir.components().as_slice())),
-            |(sub, components)| {
-                let (name, tail) = components.split_first()?;
-                Some((sub.entries.get(name)?, tail))
-            },
-        )
+        dir: &'b RepoPath,
+    ) -> impl Iterator<Item = (&'a RepoPathTree, &'b RepoPath)> + 'a {
+        iter::successors(Some((self, dir)), |(sub, dir)| {
+            let mut components = dir.components();
+            let name = components.next()?;
+            Some((sub.entries.get(name)?, components.as_path()))
+        })
     }
 
     fn to_visit_sets(&self) -> Visit {
@@ -343,376 +342,319 @@ mod tests {
     use maplit::hashset;
 
     use super::*;
-    use crate::repo_path::{RepoPath, RepoPathComponent};
+
+    fn repo_path(value: &str) -> &RepoPath {
+        RepoPath::from_internal_string(value)
+    }
 
     #[test]
     fn test_repo_path_tree_empty() {
         let tree = RepoPathTree::new();
-        assert_eq!(tree.get_visit_sets(&RepoPath::root()), Visit::Nothing);
+        assert_eq!(tree.get_visit_sets(RepoPath::root()), Visit::Nothing);
     }
 
     #[test]
     fn test_repo_path_tree_root() {
         let mut tree = RepoPathTree::new();
-        tree.add_dir(&RepoPath::root());
-        assert_eq!(tree.get_visit_sets(&RepoPath::root()), Visit::Nothing);
+        tree.add_dir(RepoPath::root());
+        assert_eq!(tree.get_visit_sets(RepoPath::root()), Visit::Nothing);
     }
 
     #[test]
     fn test_repo_path_tree_dir() {
         let mut tree = RepoPathTree::new();
-        tree.add_dir(&RepoPath::from_internal_string("dir"));
+        tree.add_dir(repo_path("dir"));
         assert_eq!(
-            tree.get_visit_sets(&RepoPath::root()),
-            Visit::sets(hashset! {RepoPathComponent::from("dir")}, hashset! {}),
+            tree.get_visit_sets(RepoPath::root()),
+            Visit::sets(hashset! {RepoPathComponentBuf::from("dir")}, hashset! {}),
         );
-        tree.add_dir(&RepoPath::from_internal_string("dir/sub"));
+        tree.add_dir(repo_path("dir/sub"));
         assert_eq!(
-            tree.get_visit_sets(&RepoPath::from_internal_string("dir")),
-            Visit::sets(hashset! {RepoPathComponent::from("sub")}, hashset! {}),
+            tree.get_visit_sets(repo_path("dir")),
+            Visit::sets(hashset! {RepoPathComponentBuf::from("sub")}, hashset! {}),
         );
     }
 
     #[test]
     fn test_repo_path_tree_file() {
         let mut tree = RepoPathTree::new();
-        tree.add_file(&RepoPath::from_internal_string("dir/file"));
+        tree.add_file(repo_path("dir/file"));
         assert_eq!(
-            tree.get_visit_sets(&RepoPath::root()),
-            Visit::sets(hashset! {RepoPathComponent::from("dir")}, hashset! {}),
+            tree.get_visit_sets(RepoPath::root()),
+            Visit::sets(hashset! {RepoPathComponentBuf::from("dir")}, hashset! {}),
         );
         assert_eq!(
-            tree.get_visit_sets(&RepoPath::from_internal_string("dir")),
-            Visit::sets(hashset! {}, hashset! {RepoPathComponent::from("file")}),
+            tree.get_visit_sets(repo_path("dir")),
+            Visit::sets(hashset! {}, hashset! {RepoPathComponentBuf::from("file")}),
         );
     }
 
     #[test]
     fn test_nothingmatcher() {
         let m = NothingMatcher;
-        assert!(!m.matches(&RepoPath::from_internal_string("file")));
-        assert!(!m.matches(&RepoPath::from_internal_string("dir/file")));
-        assert_eq!(m.visit(&RepoPath::root()), Visit::Nothing);
+        assert!(!m.matches(repo_path("file")));
+        assert!(!m.matches(repo_path("dir/file")));
+        assert_eq!(m.visit(RepoPath::root()), Visit::Nothing);
     }
 
     #[test]
     fn test_filesmatcher_empty() {
-        let m = FilesMatcher::new(&[]);
-        assert!(!m.matches(&RepoPath::from_internal_string("file")));
-        assert!(!m.matches(&RepoPath::from_internal_string("dir/file")));
-        assert_eq!(m.visit(&RepoPath::root()), Visit::Nothing);
+        let m = FilesMatcher::new([] as [&RepoPath; 0]);
+        assert!(!m.matches(repo_path("file")));
+        assert!(!m.matches(repo_path("dir/file")));
+        assert_eq!(m.visit(RepoPath::root()), Visit::Nothing);
     }
 
     #[test]
     fn test_filesmatcher_nonempty() {
-        let m = FilesMatcher::new(&[
-            RepoPath::from_internal_string("dir1/subdir1/file1"),
-            RepoPath::from_internal_string("dir1/subdir1/file2"),
-            RepoPath::from_internal_string("dir1/subdir2/file3"),
-            RepoPath::from_internal_string("file4"),
+        let m = FilesMatcher::new([
+            repo_path("dir1/subdir1/file1"),
+            repo_path("dir1/subdir1/file2"),
+            repo_path("dir1/subdir2/file3"),
+            repo_path("file4"),
         ]);
 
-        assert!(!m.matches(&RepoPath::from_internal_string("dir1")));
-        assert!(!m.matches(&RepoPath::from_internal_string("dir1/subdir1")));
-        assert!(m.matches(&RepoPath::from_internal_string("dir1/subdir1/file1")));
-        assert!(m.matches(&RepoPath::from_internal_string("dir1/subdir1/file2")));
-        assert!(!m.matches(&RepoPath::from_internal_string("dir1/subdir1/file3")));
+        assert!(!m.matches(repo_path("dir1")));
+        assert!(!m.matches(repo_path("dir1/subdir1")));
+        assert!(m.matches(repo_path("dir1/subdir1/file1")));
+        assert!(m.matches(repo_path("dir1/subdir1/file2")));
+        assert!(!m.matches(repo_path("dir1/subdir1/file3")));
 
         assert_eq!(
-            m.visit(&RepoPath::root()),
+            m.visit(RepoPath::root()),
             Visit::sets(
-                hashset! {RepoPathComponent::from("dir1")},
-                hashset! {RepoPathComponent::from("file4")}
+                hashset! {RepoPathComponentBuf::from("dir1")},
+                hashset! {RepoPathComponentBuf::from("file4")}
             )
         );
         assert_eq!(
-            m.visit(&RepoPath::from_internal_string("dir1")),
+            m.visit(repo_path("dir1")),
             Visit::sets(
-                hashset! {RepoPathComponent::from("subdir1"), RepoPathComponent::from("subdir2")},
+                hashset! {
+                    RepoPathComponentBuf::from("subdir1"),
+                    RepoPathComponentBuf::from("subdir2"),
+                },
                 hashset! {}
             )
         );
         assert_eq!(
-            m.visit(&RepoPath::from_internal_string("dir1/subdir1")),
+            m.visit(repo_path("dir1/subdir1")),
             Visit::sets(
                 hashset! {},
-                hashset! {RepoPathComponent::from("file1"), RepoPathComponent::from("file2")}
+                hashset! {
+                    RepoPathComponentBuf::from("file1"),
+                    RepoPathComponentBuf::from("file2"),
+                },
             )
         );
         assert_eq!(
-            m.visit(&RepoPath::from_internal_string("dir1/subdir2")),
-            Visit::sets(hashset! {}, hashset! {RepoPathComponent::from("file3")})
+            m.visit(repo_path("dir1/subdir2")),
+            Visit::sets(hashset! {}, hashset! {RepoPathComponentBuf::from("file3")})
         );
     }
 
     #[test]
     fn test_prefixmatcher_empty() {
-        let m = PrefixMatcher::new(&[]);
-        assert!(!m.matches(&RepoPath::from_internal_string("file")));
-        assert!(!m.matches(&RepoPath::from_internal_string("dir/file")));
-        assert_eq!(m.visit(&RepoPath::root()), Visit::Nothing);
+        let m = PrefixMatcher::new([] as [&RepoPath; 0]);
+        assert!(!m.matches(repo_path("file")));
+        assert!(!m.matches(repo_path("dir/file")));
+        assert_eq!(m.visit(RepoPath::root()), Visit::Nothing);
     }
 
     #[test]
     fn test_prefixmatcher_root() {
-        let m = PrefixMatcher::new(&[RepoPath::root()]);
+        let m = PrefixMatcher::new([RepoPath::root()]);
         // Matches all files
-        assert!(m.matches(&RepoPath::from_internal_string("file")));
-        assert!(m.matches(&RepoPath::from_internal_string("dir/file")));
+        assert!(m.matches(repo_path("file")));
+        assert!(m.matches(repo_path("dir/file")));
         // Visits all directories
-        assert_eq!(m.visit(&RepoPath::root()), Visit::AllRecursively);
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(RepoPath::root()), Visit::AllRecursively);
+        assert_eq!(m.visit(repo_path("foo/bar")), Visit::AllRecursively);
     }
 
     #[test]
     fn test_prefixmatcher_single_prefix() {
-        let m = PrefixMatcher::new(&[RepoPath::from_internal_string("foo/bar")]);
+        let m = PrefixMatcher::new([repo_path("foo/bar")]);
 
         // Parts of the prefix should not match
-        assert!(!m.matches(&RepoPath::from_internal_string("foo")));
-        assert!(!m.matches(&RepoPath::from_internal_string("bar")));
+        assert!(!m.matches(repo_path("foo")));
+        assert!(!m.matches(repo_path("bar")));
         // A file matching the prefix exactly should match
-        assert!(m.matches(&RepoPath::from_internal_string("foo/bar")));
+        assert!(m.matches(repo_path("foo/bar")));
         // Files in subdirectories should match
-        assert!(m.matches(&RepoPath::from_internal_string("foo/bar/baz")));
-        assert!(m.matches(&RepoPath::from_internal_string("foo/bar/baz/qux")));
+        assert!(m.matches(repo_path("foo/bar/baz")));
+        assert!(m.matches(repo_path("foo/bar/baz/qux")));
         // Sibling files should not match
-        assert!(!m.matches(&RepoPath::from_internal_string("foo/foo")));
+        assert!(!m.matches(repo_path("foo/foo")));
         // An unrooted "foo/bar" should not match
-        assert!(!m.matches(&RepoPath::from_internal_string("bar/foo/bar")));
+        assert!(!m.matches(repo_path("bar/foo/bar")));
 
         // The matcher should only visit directory foo/ in the root (file "foo"
         // shouldn't be visited)
         assert_eq!(
-            m.visit(&RepoPath::root()),
-            Visit::sets(hashset! {RepoPathComponent::from("foo")}, hashset! {})
+            m.visit(RepoPath::root()),
+            Visit::sets(hashset! {RepoPathComponentBuf::from("foo")}, hashset! {})
         );
         // Inside parent directory "foo/", both subdirectory "bar" and file "bar" may
         // match
         assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo")),
+            m.visit(repo_path("foo")),
             Visit::sets(
-                hashset! {RepoPathComponent::from("bar")},
-                hashset! {RepoPathComponent::from("bar")}
+                hashset! {RepoPathComponentBuf::from("bar")},
+                hashset! {RepoPathComponentBuf::from("bar")}
             )
         );
         // Inside a directory that matches the prefix, everything matches recursively
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(repo_path("foo/bar")), Visit::AllRecursively);
         // Same thing in subdirectories of the prefix
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar/baz")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(repo_path("foo/bar/baz")), Visit::AllRecursively);
         // Nothing in directories that are siblings of the prefix can match, so don't
         // visit
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("bar")),
-            Visit::Nothing
-        );
+        assert_eq!(m.visit(repo_path("bar")), Visit::Nothing);
     }
 
     #[test]
     fn test_prefixmatcher_nested_prefixes() {
-        let m = PrefixMatcher::new(&[
-            RepoPath::from_internal_string("foo"),
-            RepoPath::from_internal_string("foo/bar/baz"),
-        ]);
+        let m = PrefixMatcher::new([repo_path("foo"), repo_path("foo/bar/baz")]);
 
-        assert!(m.matches(&RepoPath::from_internal_string("foo")));
-        assert!(!m.matches(&RepoPath::from_internal_string("bar")));
-        assert!(m.matches(&RepoPath::from_internal_string("foo/bar")));
+        assert!(m.matches(repo_path("foo")));
+        assert!(!m.matches(repo_path("bar")));
+        assert!(m.matches(repo_path("foo/bar")));
         // Matches because the "foo" pattern matches
-        assert!(m.matches(&RepoPath::from_internal_string("foo/baz/foo")));
+        assert!(m.matches(repo_path("foo/baz/foo")));
 
         assert_eq!(
-            m.visit(&RepoPath::root()),
+            m.visit(RepoPath::root()),
             Visit::sets(
-                hashset! {RepoPathComponent::from("foo")},
-                hashset! {RepoPathComponent::from("foo")}
+                hashset! {RepoPathComponentBuf::from("foo")},
+                hashset! {RepoPathComponentBuf::from("foo")}
             )
         );
         // Inside a directory that matches the prefix, everything matches recursively
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(repo_path("foo")), Visit::AllRecursively);
         // Same thing in subdirectories of the prefix
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar/baz")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(repo_path("foo/bar/baz")), Visit::AllRecursively);
     }
 
     #[test]
     fn test_differencematcher_remove_subdir() {
-        let m1 = PrefixMatcher::new(&[
-            RepoPath::from_internal_string("foo"),
-            RepoPath::from_internal_string("bar"),
-        ]);
-        let m2 = PrefixMatcher::new(&[RepoPath::from_internal_string("foo/bar")]);
+        let m1 = PrefixMatcher::new([repo_path("foo"), repo_path("bar")]);
+        let m2 = PrefixMatcher::new([repo_path("foo/bar")]);
         let m = DifferenceMatcher::new(&m1, &m2);
 
-        assert!(m.matches(&RepoPath::from_internal_string("foo")));
-        assert!(!m.matches(&RepoPath::from_internal_string("foo/bar")));
-        assert!(!m.matches(&RepoPath::from_internal_string("foo/bar/baz")));
-        assert!(m.matches(&RepoPath::from_internal_string("foo/baz")));
-        assert!(m.matches(&RepoPath::from_internal_string("bar")));
+        assert!(m.matches(repo_path("foo")));
+        assert!(!m.matches(repo_path("foo/bar")));
+        assert!(!m.matches(repo_path("foo/bar/baz")));
+        assert!(m.matches(repo_path("foo/baz")));
+        assert!(m.matches(repo_path("bar")));
 
         assert_eq!(
-            m.visit(&RepoPath::root()),
+            m.visit(RepoPath::root()),
             Visit::sets(
-                hashset! {RepoPathComponent::from("foo"), RepoPathComponent::from("bar")},
-                hashset! {RepoPathComponent::from("foo"), RepoPathComponent::from("bar")}
+                hashset! {
+                    RepoPathComponentBuf::from("foo"),
+                    RepoPathComponentBuf::from("bar"),
+                },
+                hashset! {
+                    RepoPathComponentBuf::from("foo"),
+                    RepoPathComponentBuf::from("bar"),
+                },
             )
         );
         assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo")),
+            m.visit(repo_path("foo")),
             Visit::Specific {
                 dirs: VisitDirs::All,
                 files: VisitFiles::All,
             }
         );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar")),
-            Visit::Nothing
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/baz")),
-            Visit::AllRecursively
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("bar")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(repo_path("foo/bar")), Visit::Nothing);
+        assert_eq!(m.visit(repo_path("foo/baz")), Visit::AllRecursively);
+        assert_eq!(m.visit(repo_path("bar")), Visit::AllRecursively);
     }
 
     #[test]
     fn test_differencematcher_shared_patterns() {
-        let m1 = PrefixMatcher::new(&[
-            RepoPath::from_internal_string("foo"),
-            RepoPath::from_internal_string("bar"),
-        ]);
-        let m2 = PrefixMatcher::new(&[RepoPath::from_internal_string("foo")]);
+        let m1 = PrefixMatcher::new([repo_path("foo"), repo_path("bar")]);
+        let m2 = PrefixMatcher::new([repo_path("foo")]);
         let m = DifferenceMatcher::new(&m1, &m2);
 
-        assert!(!m.matches(&RepoPath::from_internal_string("foo")));
-        assert!(!m.matches(&RepoPath::from_internal_string("foo/bar")));
-        assert!(m.matches(&RepoPath::from_internal_string("bar")));
-        assert!(m.matches(&RepoPath::from_internal_string("bar/foo")));
+        assert!(!m.matches(repo_path("foo")));
+        assert!(!m.matches(repo_path("foo/bar")));
+        assert!(m.matches(repo_path("bar")));
+        assert!(m.matches(repo_path("bar/foo")));
 
         assert_eq!(
-            m.visit(&RepoPath::root()),
+            m.visit(RepoPath::root()),
             Visit::sets(
-                hashset! {RepoPathComponent::from("foo"), RepoPathComponent::from("bar")},
-                hashset! {RepoPathComponent::from("foo"), RepoPathComponent::from("bar")}
+                hashset! {
+                    RepoPathComponentBuf::from("foo"),
+                    RepoPathComponentBuf::from("bar"),
+                },
+                hashset! {
+                    RepoPathComponentBuf::from("foo"),
+                    RepoPathComponentBuf::from("bar"),
+                },
             )
         );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo")),
-            Visit::Nothing
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar")),
-            Visit::Nothing
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("bar")),
-            Visit::AllRecursively
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("bar/foo")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(repo_path("foo")), Visit::Nothing);
+        assert_eq!(m.visit(repo_path("foo/bar")), Visit::Nothing);
+        assert_eq!(m.visit(repo_path("bar")), Visit::AllRecursively);
+        assert_eq!(m.visit(repo_path("bar/foo")), Visit::AllRecursively);
     }
 
     #[test]
     fn test_intersectionmatcher_intersecting_roots() {
-        let m1 = PrefixMatcher::new(&[
-            RepoPath::from_internal_string("foo"),
-            RepoPath::from_internal_string("bar"),
-        ]);
-        let m2 = PrefixMatcher::new(&[
-            RepoPath::from_internal_string("bar"),
-            RepoPath::from_internal_string("baz"),
-        ]);
+        let m1 = PrefixMatcher::new([repo_path("foo"), repo_path("bar")]);
+        let m2 = PrefixMatcher::new([repo_path("bar"), repo_path("baz")]);
         let m = IntersectionMatcher::new(&m1, &m2);
 
-        assert!(!m.matches(&RepoPath::from_internal_string("foo")));
-        assert!(!m.matches(&RepoPath::from_internal_string("foo/bar")));
-        assert!(m.matches(&RepoPath::from_internal_string("bar")));
-        assert!(m.matches(&RepoPath::from_internal_string("bar/foo")));
-        assert!(!m.matches(&RepoPath::from_internal_string("baz")));
-        assert!(!m.matches(&RepoPath::from_internal_string("baz/foo")));
+        assert!(!m.matches(repo_path("foo")));
+        assert!(!m.matches(repo_path("foo/bar")));
+        assert!(m.matches(repo_path("bar")));
+        assert!(m.matches(repo_path("bar/foo")));
+        assert!(!m.matches(repo_path("baz")));
+        assert!(!m.matches(repo_path("baz/foo")));
 
         assert_eq!(
-            m.visit(&RepoPath::root()),
+            m.visit(RepoPath::root()),
             Visit::sets(
-                hashset! {RepoPathComponent::from("bar")},
-                hashset! {RepoPathComponent::from("bar")}
+                hashset! {RepoPathComponentBuf::from("bar")},
+                hashset! {RepoPathComponentBuf::from("bar")}
             )
         );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo")),
-            Visit::Nothing
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar")),
-            Visit::Nothing
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("bar")),
-            Visit::AllRecursively
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("bar/foo")),
-            Visit::AllRecursively
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("baz")),
-            Visit::Nothing
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("baz/foo")),
-            Visit::Nothing
-        );
+        assert_eq!(m.visit(repo_path("foo")), Visit::Nothing);
+        assert_eq!(m.visit(repo_path("foo/bar")), Visit::Nothing);
+        assert_eq!(m.visit(repo_path("bar")), Visit::AllRecursively);
+        assert_eq!(m.visit(repo_path("bar/foo")), Visit::AllRecursively);
+        assert_eq!(m.visit(repo_path("baz")), Visit::Nothing);
+        assert_eq!(m.visit(repo_path("baz/foo")), Visit::Nothing);
     }
 
     #[test]
     fn test_intersectionmatcher_subdir() {
-        let m1 = PrefixMatcher::new(&[RepoPath::from_internal_string("foo")]);
-        let m2 = PrefixMatcher::new(&[RepoPath::from_internal_string("foo/bar")]);
+        let m1 = PrefixMatcher::new([repo_path("foo")]);
+        let m2 = PrefixMatcher::new([repo_path("foo/bar")]);
         let m = IntersectionMatcher::new(&m1, &m2);
 
-        assert!(!m.matches(&RepoPath::from_internal_string("foo")));
-        assert!(!m.matches(&RepoPath::from_internal_string("bar")));
-        assert!(m.matches(&RepoPath::from_internal_string("foo/bar")));
-        assert!(m.matches(&RepoPath::from_internal_string("foo/bar/baz")));
-        assert!(!m.matches(&RepoPath::from_internal_string("foo/baz")));
+        assert!(!m.matches(repo_path("foo")));
+        assert!(!m.matches(repo_path("bar")));
+        assert!(m.matches(repo_path("foo/bar")));
+        assert!(m.matches(repo_path("foo/bar/baz")));
+        assert!(!m.matches(repo_path("foo/baz")));
 
         assert_eq!(
-            m.visit(&RepoPath::root()),
-            Visit::sets(hashset! {RepoPathComponent::from("foo")}, hashset! {})
+            m.visit(RepoPath::root()),
+            Visit::sets(hashset! {RepoPathComponentBuf::from("foo")}, hashset! {})
         );
+        assert_eq!(m.visit(repo_path("bar")), Visit::Nothing);
         assert_eq!(
-            m.visit(&RepoPath::from_internal_string("bar")),
-            Visit::Nothing
-        );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo")),
+            m.visit(repo_path("foo")),
             Visit::sets(
-                hashset! {RepoPathComponent::from("bar")},
-                hashset! {RepoPathComponent::from("bar")}
+                hashset! {RepoPathComponentBuf::from("bar")},
+                hashset! {RepoPathComponentBuf::from("bar")}
             )
         );
-        assert_eq!(
-            m.visit(&RepoPath::from_internal_string("foo/bar")),
-            Visit::AllRecursively
-        );
+        assert_eq!(m.visit(repo_path("foo/bar")), Visit::AllRecursively);
     }
 }
