@@ -21,9 +21,11 @@ use std::fmt::{Debug, Formatter};
 use std::hash::Hash;
 use std::io::Write;
 use std::iter::zip;
+use std::slice;
 use std::sync::Arc;
 
 use itertools::Itertools;
+use smallvec::{smallvec_inline, SmallVec};
 
 use crate::backend;
 use crate::backend::{BackendError, FileId, ObjectId, TreeId, TreeValue};
@@ -43,17 +45,28 @@ where
         removes.len() + 1,
         "trivial_merge() requires exactly one more adds than removes"
     );
+    trivial_merge_inner(
+        itertools::interleave(adds, removes),
+        adds.len() + removes.len(),
+    )
+}
 
+fn trivial_merge_inner<T>(mut values: impl Iterator<Item = T>, values_len: usize) -> Option<T>
+where
+    T: Eq + Hash,
+{
     // Optimize the common cases of 3-way merge and 1-way (non-)merge
-    if adds.len() == 1 {
-        return Some(&adds[0]);
-    } else if adds.len() == 2 {
-        return if adds[0] == adds[1] {
-            Some(&adds[0])
-        } else if adds[0] == removes[0] {
-            Some(&adds[1])
-        } else if adds[1] == removes[0] {
-            Some(&adds[0])
+    if values_len == 1 {
+        let add = values.next().unwrap();
+        return Some(add);
+    } else if values_len == 3 {
+        let (add0, remove, add1) = values.next_tuple().unwrap();
+        return if add0 == add1 {
+            Some(add0)
+        } else if add0 == remove {
+            Some(add1)
+        } else if add1 == remove {
+            Some(add0)
         } else {
             None
         };
@@ -62,41 +75,36 @@ where
     // Number of occurrences of each value, with positive indexes counted as +1 and
     // negative as -1, thereby letting positive and negative terms with the same
     // value (i.e. key in the map) cancel each other.
-    let mut counts: HashMap<&T, i32> = HashMap::new();
-    for value in adds.iter() {
-        counts.entry(value).and_modify(|e| *e += 1).or_insert(1);
-    }
-    for value in removes.iter() {
-        counts.entry(value).and_modify(|e| *e -= 1).or_insert(-1);
+    let mut counts: HashMap<T, i32> = HashMap::new();
+    for (value, n) in zip(values, [1, -1].into_iter().cycle()) {
+        counts.entry(value).and_modify(|e| *e += n).or_insert(n);
     }
 
     // Collect non-zero value. Values with a count of 0 means that they have
     // cancelled out.
-    let counts = counts
-        .into_iter()
-        .filter(|&(_, count)| count != 0)
-        .collect_vec();
-    match counts[..] {
-        [(value, 1)] => {
-            // If there is a single value with a count of 1 left, then that is the result.
-            Some(value)
+    counts.retain(|_, count| *count != 0);
+    if counts.len() == 1 {
+        // If there is a single value with a count of 1 left, then that is the result.
+        let (value, count) = counts.into_iter().next().unwrap();
+        assert_eq!(count, 1);
+        Some(value)
+    } else if counts.len() == 2 {
+        // All sides made the same change.
+        // This matches what Git and Mercurial do (in the 3-way case at least), but not
+        // what Darcs and Pijul do. It means that repeated 3-way merging of multiple
+        // trees may give different results depending on the order of merging.
+        // TODO: Consider removing this special case, making the algorithm more strict,
+        // and maybe add a more lenient version that is used when the user explicitly
+        // asks for conflict resolution.
+        let ((value1, count1), (value2, count2)) = counts.into_iter().next_tuple().unwrap();
+        assert_eq!(count1 + count2, 1);
+        if count1 > 0 {
+            Some(value1)
+        } else {
+            Some(value2)
         }
-        [(value1, count1), (value2, count2)] => {
-            // All sides made the same change.
-            // This matches what Git and Mercurial do (in the 3-way case at least), but not
-            // what Darcs and Pijul do. It means that repeated 3-way merging of multiple
-            // trees may give different results depending on the order of merging.
-            // TODO: Consider removing this special case, making the algorithm more strict,
-            // and maybe add a more lenient version that is used when the user explicitly
-            // asks for conflict resolution.
-            assert_eq!(count1 + count2, 1);
-            if count1 > 0 {
-                Some(value1)
-            } else {
-                Some(value2)
-            }
-        }
-        _ => None,
+    } else {
+        None
     }
 }
 
@@ -107,35 +115,55 @@ where
 /// remove. The zeroth add is considered a diff from the non-existent state.
 #[derive(PartialEq, Eq, Hash, Clone)]
 pub struct Merge<T> {
-    removes: Vec<T>,
-    adds: Vec<T>,
+    /// Alternates between positive and negative terms, starting with positive.
+    values: SmallVec<[T; 1]>,
 }
 
 impl<T: Debug> Debug for Merge<T> {
     fn fmt(&self, f: &mut Formatter<'_>) -> Result<(), std::fmt::Error> {
         // Format like an enum with two variants to make it less verbose in the common
         // case of a resolved state.
-        if self.removes.is_empty() {
-            f.debug_tuple("Resolved").field(&self.adds[0]).finish()
+        if let Some(value) = self.as_resolved() {
+            f.debug_tuple("Resolved").field(value).finish()
         } else {
-            f.debug_struct("Conflicted")
-                .field("removes", &self.removes)
-                .field("adds", &self.adds)
-                .finish()
+            f.debug_tuple("Conflicted").field(&self.values).finish()
         }
     }
 }
 
 impl<T> Merge<T> {
+    /// Creates a `Merge` from the given values, in which positive and negative
+    /// terms alternate.
+    pub fn from_vec(values: impl Into<SmallVec<[T; 1]>>) -> Self {
+        let values = values.into();
+        assert!(
+            values.len() & 1 != 0,
+            "must have one more adds than removes"
+        );
+        Merge { values }
+    }
+
     /// Creates a new merge object from the given removes and adds.
-    pub fn new(removes: Vec<T>, adds: Vec<T>) -> Self {
-        assert_eq!(adds.len(), removes.len() + 1);
-        Merge { removes, adds }
+    pub fn from_removes_adds(
+        removes: impl IntoIterator<Item = T>,
+        adds: impl IntoIterator<Item = T>,
+    ) -> Self {
+        let removes = removes.into_iter();
+        let mut adds = adds.into_iter();
+        let mut values = SmallVec::with_capacity(removes.size_hint().0 * 2 + 1);
+        values.push(adds.next().expect("must have at least one add"));
+        for diff in removes.zip_longest(adds) {
+            let (remove, add) = diff.both().expect("must have one more adds than removes");
+            values.extend([remove, add]);
+        }
+        Merge { values }
     }
 
     /// Creates a `Merge` with a single resolved value.
     pub fn resolved(value: T) -> Self {
-        Merge::new(vec![], vec![value])
+        Merge {
+            values: smallvec_inline![value],
+        }
     }
 
     /// Create a `Merge` from a `removes` and `adds`, padding with `None` to
@@ -144,46 +172,67 @@ impl<T> Merge<T> {
         removes: impl IntoIterator<Item = T>,
         adds: impl IntoIterator<Item = T>,
     ) -> Merge<Option<T>> {
-        let mut removes = removes.into_iter().map(Some).collect_vec();
-        let mut adds = adds.into_iter().map(Some).collect_vec();
-        while removes.len() + 1 < adds.len() {
-            removes.push(None);
+        let removes = removes.into_iter();
+        let mut adds = adds.into_iter().fuse();
+        let mut values = smallvec_inline![adds.next()];
+        for diff in removes.zip_longest(adds) {
+            let (remove, add) = diff.map_any(Some, Some).or_default();
+            values.extend([remove, add]);
         }
-        while adds.len() < removes.len() + 1 {
-            adds.push(None);
-        }
-        Merge::new(removes, adds)
-    }
-
-    /// Returns the removes and adds as a pair.
-    pub fn take(self) -> (Vec<T>, Vec<T>) {
-        (self.removes, self.adds)
+        Merge { values }
     }
 
     /// The removed values, also called negative terms.
-    pub fn removes(&self) -> &[T] {
-        &self.removes
+    pub fn removes(&self) -> impl ExactSizeIterator<Item = &T> {
+        self.values[1..].iter().step_by(2)
     }
 
-    /// The removed values, also called positive terms.
-    pub fn adds(&self) -> &[T] {
-        &self.adds
+    /// The added values, also called positive terms.
+    pub fn adds(&self) -> impl ExactSizeIterator<Item = &T> {
+        self.values.iter().step_by(2)
+    }
+
+    /// Returns the zeroth added value, which is guaranteed to exist.
+    pub fn first(&self) -> &T {
+        &self.values[0]
+    }
+
+    /// Returns the `index`-th removed value, which is considered belonging to
+    /// the `index`-th diff pair.
+    pub fn get_remove(&self, index: usize) -> Option<&T> {
+        self.values.get(index * 2 + 1)
+    }
+
+    /// Returns the `index`-th added value, which is considered belonging to the
+    /// `index-1`-th diff pair. The zeroth add is a diff from the non-existent
+    /// state.
+    pub fn get_add(&self, index: usize) -> Option<&T> {
+        self.values.get(index * 2)
+    }
+
+    /// Removes the specified "removed"/"added" values. The removed slots are
+    /// replaced by the last "removed"/"added" values.
+    pub fn swap_remove(&mut self, remove_index: usize, add_index: usize) -> (T, T) {
+        // Swap with the last "added" and "removed" values in order.
+        let add = self.values.swap_remove(add_index * 2);
+        let remove = self.values.swap_remove(remove_index * 2 + 1);
+        (remove, add)
     }
 
     /// The number of positive terms in the conflict.
     pub fn num_sides(&self) -> usize {
-        self.adds.len()
+        self.values.len() / 2 + 1
     }
 
     /// Whether this merge is resolved. Does not resolve trivial merges.
     pub fn is_resolved(&self) -> bool {
-        self.removes.is_empty()
+        self.values.len() == 1
     }
 
     /// Returns the resolved value, if this merge is resolved. Does not
     /// resolve trivial merges.
     pub fn as_resolved(&self) -> Option<&T> {
-        if let [value] = &self.adds[..] {
+        if let [value] = &self.values[..] {
             Some(value)
         } else {
             None
@@ -193,8 +242,8 @@ impl<T> Merge<T> {
     /// Returns the resolved value, if this merge is resolved. Otherwise returns
     /// the merge itself as an `Err`. Does not resolve trivial merges.
     pub fn into_resolved(mut self) -> Result<T, Merge<T>> {
-        if self.removes.is_empty() {
-            Ok(self.adds.pop().unwrap())
+        if self.values.len() == 1 {
+            Ok(self.values.pop().unwrap())
         } else {
             Err(self)
         }
@@ -207,16 +256,16 @@ impl<T> Merge<T> {
         T: PartialEq,
     {
         let mut add_index = 0;
-        while add_index < self.adds.len() {
-            let add = &self.adds[add_index];
-            if let Some(remove_index) = self.removes.iter().position(|remove| remove == add) {
-                // Move the value to the `add_index-1`th diff, then delete the `remove_index`th
-                // diff.
-                self.adds.swap(remove_index + 1, add_index);
-                self.removes.remove(remove_index);
-                self.adds.remove(remove_index + 1);
+        while add_index < self.values.len() {
+            let add = &self.values[add_index];
+            let mut removes = self.values.iter().enumerate().skip(1).step_by(2);
+            if let Some((remove_index, _)) = removes.find(|&(_, remove)| remove == add) {
+                // Align the current "add" value to the `remove_index/2`-th diff, then
+                // delete the diff pair.
+                self.values.swap(remove_index + 1, add_index);
+                self.values.drain(remove_index..remove_index + 2);
             } else {
-                add_index += 1;
+                add_index += 2;
             }
         }
         self
@@ -228,7 +277,7 @@ impl<T> Merge<T> {
     where
         T: Eq + Hash,
     {
-        trivial_merge(&self.removes, &self.adds)
+        trivial_merge_inner(self.values.iter(), self.values.len())
     }
 
     /// Pads this merge with to the specified number of sides with the specified
@@ -237,35 +286,35 @@ impl<T> Merge<T> {
     where
         T: Clone,
     {
-        for _ in self.num_sides()..num_sides {
-            self.removes.push(value.clone());
-            self.adds.push(value.clone());
+        if num_sides <= self.num_sides() {
+            return;
         }
+        self.values.resize(num_sides * 2 - 1, value.clone());
     }
 
     /// Returns an iterator over references to the terms. The items will
     /// alternate between positive and negative terms, starting with
     /// positive (since there's one more of those).
-    pub fn iter(&self) -> impl Iterator<Item = &T> {
-        itertools::interleave(&self.adds, &self.removes)
+    pub fn iter(&self) -> slice::Iter<'_, T> {
+        self.values.iter()
     }
 
     /// A version of `Merge::iter()` that iterates over mutable references.
-    pub fn iter_mut(&mut self) -> impl Iterator<Item = &mut T> {
-        itertools::interleave(self.adds.iter_mut(), self.removes.iter_mut())
+    pub fn iter_mut(&mut self) -> slice::IterMut<'_, T> {
+        self.values.iter_mut()
     }
 
     /// Creates a new merge by applying `f` to each remove and add.
     pub fn map<'a, U>(&'a self, f: impl FnMut(&'a T) -> U) -> Merge<U> {
-        let builder: MergeBuilder<U> = self.iter().map(f).collect();
-        builder.build()
+        let values = self.values.iter().map(f).collect();
+        Merge { values }
     }
 
     /// Creates a new merge by applying `f` to each remove and add, returning
     /// `None if `f` returns `None` for any of them.
     pub fn maybe_map<'a, U>(&'a self, f: impl FnMut(&'a T) -> Option<U>) -> Option<Merge<U>> {
-        let builder: Option<MergeBuilder<U>> = self.iter().map(f).collect();
-        builder.map(MergeBuilder::build)
+        let values = self.values.iter().map(f).collect::<Option<_>>()?;
+        Some(Merge { values })
     }
 
     /// Creates a new merge by applying `f` to each remove and add, returning
@@ -274,8 +323,8 @@ impl<T> Merge<T> {
         &'a self,
         f: impl FnMut(&'a T) -> Result<U, E>,
     ) -> Result<Merge<U>, E> {
-        let builder: MergeBuilder<U> = self.iter().map(f).try_collect()?;
-        Ok(builder.build())
+        let values = self.values.iter().map(f).try_collect()?;
+        Ok(Merge { values })
     }
 }
 
@@ -289,15 +338,13 @@ impl<T> Merge<T> {
 /// the checking until after `from_iter()` (to `MergeBuilder::build()`).
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub struct MergeBuilder<T> {
-    removes: Vec<T>,
-    adds: Vec<T>,
+    values: SmallVec<[T; 1]>,
 }
 
 impl<T> Default for MergeBuilder<T> {
     fn default() -> Self {
         Self {
-            removes: Default::default(),
-            adds: Default::default(),
+            values: Default::default(),
         }
     }
 }
@@ -306,16 +353,16 @@ impl<T> MergeBuilder<T> {
     /// Requires that exactly one more "adds" than "removes" have been added to
     /// this builder.
     pub fn build(self) -> Merge<T> {
-        Merge::new(self.removes, self.adds)
+        Merge::from_vec(self.values)
     }
 }
 
 impl<T> IntoIterator for Merge<T> {
     type Item = T;
-    type IntoIter = itertools::Interleave<std::vec::IntoIter<T>, std::vec::IntoIter<T>>;
+    type IntoIter = smallvec::IntoIter<[T; 1]>;
 
     fn into_iter(self) -> Self::IntoIter {
-        itertools::interleave(self.adds, self.removes)
+        self.values.into_iter()
     }
 }
 
@@ -329,15 +376,7 @@ impl<T> FromIterator<T> for MergeBuilder<T> {
 
 impl<T> Extend<T> for MergeBuilder<T> {
     fn extend<I: IntoIterator<Item = T>>(&mut self, iter: I) {
-        let (mut curr, mut next) = if self.adds.len() != self.removes.len() {
-            (&mut self.removes, &mut self.adds)
-        } else {
-            (&mut self.adds, &mut self.removes)
-        };
-        for item in iter {
-            curr.push(item);
-            std::mem::swap(&mut curr, &mut next);
-        }
+        self.values.extend(iter)
     }
 }
 
@@ -366,10 +405,16 @@ impl<T> Merge<Option<T>> {
     /// `None` values. Note that the conversion is lossy: the order of `None`
     /// values is not preserved when converting back to a `Merge`.
     pub fn into_legacy_form(self) -> (Vec<T>, Vec<T>) {
-        (
-            self.removes.into_iter().flatten().collect(),
-            self.adds.into_iter().flatten().collect(),
-        )
+        // Allocate the maximum size assuming there would be few `None`s.
+        let mut removes = Vec::with_capacity(self.values.len() / 2);
+        let mut adds = Vec::with_capacity(self.values.len() / 2 + 1);
+        let mut values = self.values.into_iter();
+        adds.extend(values.next().unwrap());
+        while let Some(remove) = values.next() {
+            removes.extend(remove);
+            adds.extend(values.next().unwrap());
+        }
+        (removes, adds)
     }
 }
 
@@ -387,30 +432,27 @@ impl<T> Merge<Merge<T>> {
     ///
     /// 4 5 0 7 8
     ///  3 2 1 6
-    pub fn flatten(mut self) -> Merge<T> {
-        self.removes.reverse();
-        self.adds.reverse();
-        let mut result = self.adds.pop().unwrap();
-        while let Some(mut remove) = self.removes.pop() {
+    pub fn flatten(self) -> Merge<T> {
+        let mut outer_values = self.values.into_iter();
+        let mut result = outer_values.next().unwrap();
+        while let Some(mut remove) = outer_values.next() {
             // Add removes reversed, and with the first element moved last, so we preserve
             // the diffs
-            let first_add = remove.adds.remove(0);
-            result.removes.extend(remove.adds);
-            result.removes.push(first_add);
-            result.adds.extend(remove.removes);
-            let add = self.adds.pop().unwrap();
-            result.removes.extend(add.removes);
-            result.adds.extend(add.adds);
+            remove.values.rotate_left(1);
+            for i in 0..remove.values.len() / 2 {
+                remove.values.swap(i * 2, i * 2 + 1);
+            }
+            result.values.extend(remove.values);
+            let add = outer_values.next().unwrap();
+            result.values.extend(add.values);
         }
-        assert!(self.adds.is_empty());
         result
     }
 }
 
 impl<T: ContentHash> ContentHash for Merge<T> {
     fn hash(&self, state: &mut impl digest::Update) {
-        self.removes().hash(state);
-        self.adds().hash(state);
+        self.values.hash(state)
     }
 }
 
@@ -468,8 +510,8 @@ impl MergedTreeValue {
     /// Creates a new merge with the file ids from the given merge. In other
     /// words, only the executable bits from `self` will be preserved.
     pub fn with_new_file_ids(&self, file_ids: &Merge<Option<FileId>>) -> Self {
-        assert_eq!(self.removes.len(), file_ids.removes.len());
-        let builder: MergeBuilder<Option<TreeValue>> = zip(self.iter(), file_ids.iter())
+        assert_eq!(self.values.len(), file_ids.values.len());
+        let values = zip(self.iter(), file_ids.iter())
             .map(|(tree_value, file_id)| {
                 if let Some(TreeValue::File { id: _, executable }) = tree_value {
                     Some(TreeValue::File {
@@ -483,16 +525,16 @@ impl MergedTreeValue {
                 }
             })
             .collect();
-        builder.build()
+        Merge { values }
     }
 
     /// Give a summary description of the conflict's "removes" and "adds"
     pub fn describe(&self, file: &mut dyn Write) -> std::io::Result<()> {
         file.write_all(b"Conflict:\n")?;
-        for term in self.removes().iter().flatten() {
+        for term in self.removes().flatten() {
             file.write_all(format!("  Removing {}\n", describe_conflict_term(term)).as_bytes())?;
         }
-        for term in self.adds().iter().flatten() {
+        for term in self.adds().flatten() {
             file.write_all(format!("  Adding {}\n", describe_conflict_term(term)).as_bytes())?;
         }
         Ok(())
@@ -527,7 +569,7 @@ where
                 if let Some(id) = id {
                     store.get_tree(dir, id)
                 } else {
-                    Ok(Tree::null(store.clone(), dir.clone()))
+                    Ok(Tree::null(store.clone(), dir.to_owned()))
                 }
             };
             Ok(Some(tree_id_merge.try_map(get_tree)?))
@@ -571,7 +613,7 @@ mod tests {
     use super::*;
 
     fn c<T: Clone>(removes: &[T], adds: &[T]) -> Merge<T> {
-        Merge::new(removes.to_vec(), adds.to_vec())
+        Merge::from_removes_adds(removes.to_vec(), adds.to_vec())
     }
 
     #[test]
@@ -646,39 +688,48 @@ mod tests {
             assert_eq!(Merge::from_legacy_form(legacy_form.0, legacy_form.1), merge);
         }
         // Non-conflict
-        test_equivalent((vec![], vec![0]), Merge::new(vec![], vec![Some(0)]));
+        test_equivalent(
+            (vec![], vec![0]),
+            Merge::from_removes_adds(vec![], vec![Some(0)]),
+        );
         // Regular 3-way conflict
         test_equivalent(
             (vec![0], vec![1, 2]),
-            Merge::new(vec![Some(0)], vec![Some(1), Some(2)]),
+            Merge::from_removes_adds(vec![Some(0)], vec![Some(1), Some(2)]),
         );
         // Modify/delete conflict
         test_equivalent(
             (vec![0], vec![1]),
-            Merge::new(vec![Some(0)], vec![Some(1), None]),
+            Merge::from_removes_adds(vec![Some(0)], vec![Some(1), None]),
         );
         // Add/add conflict
         test_equivalent(
             (vec![], vec![0, 1]),
-            Merge::new(vec![None], vec![Some(0), Some(1)]),
+            Merge::from_removes_adds(vec![None], vec![Some(0), Some(1)]),
         );
         // 5-way conflict
         test_equivalent(
             (vec![0, 1], vec![2, 3, 4]),
-            Merge::new(vec![Some(0), Some(1)], vec![Some(2), Some(3), Some(4)]),
+            Merge::from_removes_adds(vec![Some(0), Some(1)], vec![Some(2), Some(3), Some(4)]),
         );
         // 5-way delete/delete conflict
         test_equivalent(
             (vec![0, 1], vec![]),
-            Merge::new(vec![Some(0), Some(1)], vec![None, None, None]),
+            Merge::from_removes_adds(vec![Some(0), Some(1)], vec![None, None, None]),
         );
     }
 
     #[test]
     fn test_as_resolved() {
-        assert_eq!(Merge::new(vec![], vec![0]).as_resolved(), Some(&0));
+        assert_eq!(
+            Merge::from_removes_adds(vec![], vec![0]).as_resolved(),
+            Some(&0)
+        );
         // Even a trivially resolvable merge is not resolved
-        assert_eq!(Merge::new(vec![0], vec![0, 1]).as_resolved(), None);
+        assert_eq!(
+            Merge::from_removes_adds(vec![0], vec![0, 1]).as_resolved(),
+            None
+        );
     }
 
     #[test]
@@ -753,7 +804,7 @@ mod tests {
     #[test]
     fn test_merge_invariants() {
         fn check_invariants(removes: &[u32], adds: &[u32]) {
-            let merge = Merge::new(removes.to_vec(), adds.to_vec());
+            let merge = Merge::from_removes_adds(removes.to_vec(), adds.to_vec());
             // `simplify()` is idempotent
             assert_eq!(
                 merge.clone().simplify().simplify(),
@@ -783,6 +834,17 @@ mod tests {
         }
     }
 
+    #[test]
+    fn test_swap_remove() {
+        let mut x = c(&[1, 3, 5], &[0, 2, 4, 6]);
+        assert_eq!(x.swap_remove(0, 1), (1, 2));
+        assert_eq!(x, c(&[5, 3], &[0, 6, 4]));
+        assert_eq!(x.swap_remove(1, 0), (3, 0));
+        assert_eq!(x, c(&[5], &[4, 6]));
+        assert_eq!(x.swap_remove(0, 1), (5, 6));
+        assert_eq!(x, c(&[], &[4]));
+    }
+
     #[test]
     fn test_pad_to() {
         let mut x = c(&[], &[1]);
