@@ -22,54 +22,33 @@ use itertools::Itertools;
 use thiserror::Error;
 
 use crate::dag_walk;
-use crate::op_store::{OpStore, OpStoreError, OpStoreResult, OperationId};
+use crate::op_store::{OpStore, OpStoreError, OperationId};
 use crate::operation::Operation;
 
 #[derive(Debug, Error)]
-pub enum OpHeadResolutionError<E> {
+pub enum OpHeadResolutionError {
     #[error("Operation log has no heads")]
     NoHeads,
-    #[error(transparent)]
-    OpStore(#[from] OpStoreError),
-    #[error("Op resolution error: {0}")]
-    Err(#[source] E),
 }
 
-pub trait OpHeadsStoreLock<'a> {
-    fn promote_new_op(&self, new_op: &Operation);
-}
+pub trait OpHeadsStoreLock {}
 
 /// Manages the set of current heads of the operation log.
 pub trait OpHeadsStore: Send + Sync + Debug {
     fn name(&self) -> &str;
 
-    fn add_op_head(&self, id: &OperationId);
-
-    fn remove_op_head(&self, id: &OperationId);
+    /// Remove the old op heads and add the new one.
+    ///
+    /// The old op heads must not contain the new one.
+    fn update_op_heads(&self, old_ids: &[OperationId], new_id: &OperationId);
 
     fn get_op_heads(&self) -> Vec<OperationId>;
 
-    fn lock<'a>(&'a self) -> Box<dyn OpHeadsStoreLock<'a> + 'a>;
-
-    /// Removes operations in the input that are ancestors of other operations
-    /// in the input. The ancestors are removed both from the list and from
-    /// storage.
-    // TODO: maybe introduce OpHeadsStoreError?
-    fn handle_ancestor_ops(&self, op_heads: Vec<Operation>) -> OpStoreResult<Vec<Operation>> {
-        let op_head_ids_before: HashSet<_> = op_heads.iter().map(|op| op.id().clone()).collect();
-        // Remove ancestors so we don't create merge operation with an operation and its
-        // ancestor
-        let op_heads = dag_walk::heads_ok(
-            op_heads.into_iter().map(Ok),
-            |op: &Operation| op.id().clone(),
-            |op: &Operation| op.parents().collect_vec(),
-        )?;
-        let op_head_ids_after: HashSet<_> = op_heads.iter().map(|op| op.id().clone()).collect();
-        for removed_op_head in op_head_ids_before.difference(&op_head_ids_after) {
-            self.remove_op_head(removed_op_head);
-        }
-        Ok(op_heads.into_iter().collect())
-    }
+    /// Optionally takes a lock on the op heads store. The purpose of the lock
+    /// is to prevent concurrent processes from resolving the same divergent
+    /// operations. It is not needed for correctness; implementations are free
+    /// to return a type that doesn't hold.
+    fn lock(&self) -> Box<dyn OpHeadsStoreLock + '_>;
 }
 
 // Given an OpHeadsStore, fetch and resolve its op heads down to one under a
@@ -80,12 +59,15 @@ pub fn resolve_op_heads<E>(
     op_heads_store: &dyn OpHeadsStore,
     op_store: &Arc<dyn OpStore>,
     resolver: impl FnOnce(Vec<Operation>) -> Result<Operation, E>,
-) -> Result<Operation, OpHeadResolutionError<E>> {
+) -> Result<Operation, E>
+where
+    E: From<OpHeadResolutionError> + From<OpStoreError>,
+{
     let mut op_heads = op_heads_store.get_op_heads();
 
     // TODO: De-duplicate this 'simple-resolution' code.
     if op_heads.is_empty() {
-        return Err(OpHeadResolutionError::NoHeads);
+        return Err(OpHeadResolutionError::NoHeads.into());
     }
 
     if op_heads.len() == 1 {
@@ -102,11 +84,11 @@ pub fn resolve_op_heads<E>(
     // Note that the locking isn't necessary for correctness; we take the lock
     // only to prevent other concurrent processes from doing the same work (and
     // producing another set of divergent heads).
-    let lock = op_heads_store.lock();
+    let _lock = op_heads_store.lock();
     let op_head_ids = op_heads_store.get_op_heads();
 
     if op_head_ids.is_empty() {
-        return Err(OpHeadResolutionError::NoHeads);
+        return Err(OpHeadResolutionError::NoHeads.into());
     }
 
     if op_head_ids.len() == 1 {
@@ -115,26 +97,39 @@ pub fn resolve_op_heads<E>(
         return Ok(Operation::new(op_store.clone(), op_head_id, op_head));
     }
 
-    let op_heads = op_head_ids
+    let op_heads: Vec<_> = op_head_ids
         .iter()
         .map(|op_id: &OperationId| -> Result<Operation, OpStoreError> {
             let data = op_store.read_operation(op_id)?;
             Ok(Operation::new(op_store.clone(), op_id.clone(), data))
         })
         .try_collect()?;
-    let mut op_heads = op_heads_store.handle_ancestor_ops(op_heads)?;
+    // Remove ancestors so we don't create merge operation with an operation and its
+    // ancestor
+    let op_head_ids_before: HashSet<_> = op_heads.iter().map(|op| op.id().clone()).collect();
+    let filtered_op_heads = dag_walk::heads_ok(
+        op_heads.into_iter().map(Ok),
+        |op: &Operation| op.id().clone(),
+        |op: &Operation| op.parents().collect_vec(),
+    )?;
+    let op_head_ids_after: HashSet<_> =
+        filtered_op_heads.iter().map(|op| op.id().clone()).collect();
+    let ancestor_op_heads = op_head_ids_before
+        .difference(&op_head_ids_after)
+        .cloned()
+        .collect_vec();
+    let mut op_heads = filtered_op_heads.into_iter().collect_vec();
 
     // Return without creating a merge operation
-    if op_heads.len() == 1 {
-        return Ok(op_heads.pop().unwrap());
+    if let [op_head] = &*op_heads {
+        op_heads_store.update_op_heads(&ancestor_op_heads, op_head.id());
+        return Ok(op_head.clone());
     }
 
     op_heads.sort_by_key(|op| op.store_operation().metadata.end_time.timestamp.clone());
-    match resolver(op_heads) {
-        Ok(new_op) => {
-            lock.promote_new_op(&new_op);
-            Ok(new_op)
-        }
-        Err(e) => Err(OpHeadResolutionError::Err(e)),
-    }
+    let new_op = resolver(op_heads)?;
+    let mut old_op_heads = ancestor_op_heads;
+    old_op_heads.extend_from_slice(new_op.parent_ids());
+    op_heads_store.update_op_heads(&old_op_heads, new_op.id());
+    Ok(new_op)
 }
