@@ -19,7 +19,6 @@ use std::fmt::{Debug, Formatter};
 use std::io::ErrorKind;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
-use std::pin::Pin;
 use std::sync::Arc;
 use std::{fs, slice};
 
@@ -35,7 +34,7 @@ use crate::backend::{
 };
 use crate::commit::{Commit, CommitByCommitterTimestamp};
 use crate::commit_builder::CommitBuilder;
-use crate::default_index_store::DefaultIndexStore;
+use crate::default_index::DefaultIndexStore;
 use crate::default_submodule_store::DefaultSubmoduleStore;
 use crate::file_util::{IoResultExt as _, PathError};
 use crate::git_backend::GitBackend;
@@ -49,7 +48,7 @@ use crate::operation::Operation;
 use crate::refs::{
     diff_named_ref_targets, diff_named_remote_refs, merge_ref_targets, merge_remote_refs,
 };
-use crate::revset::{self, ChangeIdIndex, Revset, RevsetExpression};
+use crate::revset::{ChangeIdIndex, RevsetExpression};
 use crate::rewrite::{DescendantRebaser, RebaseOptions};
 use crate::settings::{RepoSettings, UserSettings};
 use crate::signing::{SignInitError, Signer};
@@ -60,7 +59,7 @@ use crate::submodule_store::SubmoduleStore;
 use crate::transaction::Transaction;
 use crate::tree::TreeMergeError;
 use crate::view::View;
-use crate::{backend, dag_walk, op_store};
+use crate::{backend, dag_walk, op_store, revset};
 
 pub trait Repo {
     fn store(&self) -> &Arc<Store>;
@@ -97,8 +96,7 @@ pub struct ReadonlyRepo {
     settings: RepoSettings,
     index_store: Arc<dyn IndexStore>,
     submodule_store: Arc<dyn SubmoduleStore>,
-    index: OnceCell<Pin<Box<dyn ReadonlyIndex>>>,
-    // Declared after `change_id_index` since it must outlive it on drop.
+    index: OnceCell<Box<dyn ReadonlyIndex>>,
     change_id_index: OnceCell<Box<dyn ChangeIdIndex>>,
     // TODO: This should eventually become part of the index and not be stored fully in memory.
     view: View,
@@ -134,7 +132,7 @@ impl ReadonlyRepo {
     }
 
     pub fn default_index_store_initializer() -> &'static IndexStoreInitializer<'static> {
-        &|_settings, store_path| Box::new(DefaultIndexStore::init(store_path))
+        &|_settings, store_path| Ok(Box::new(DefaultIndexStore::init(store_path)?))
     }
 
     pub fn default_submodule_store_initializer() -> &'static SubmoduleStoreInitializer<'static> {
@@ -188,14 +186,14 @@ impl ReadonlyRepo {
         let init_operation_id = op_store.write_operation(&init_operation).unwrap();
         let init_operation = Operation::new(op_store.clone(), init_operation_id, init_operation);
         let op_heads_store = op_heads_store_initializer(user_settings, &op_heads_path);
-        op_heads_store.add_op_head(init_operation.id());
+        op_heads_store.update_op_heads(&[], init_operation.id());
         let op_heads_type_path = op_heads_path.join("type");
         fs::write(&op_heads_type_path, op_heads_store.name()).context(&op_heads_type_path)?;
         let op_heads_store = Arc::from(op_heads_store);
 
         let index_path = repo_path.join("index");
         fs::create_dir(&index_path).context(&index_path)?;
-        let index_store = index_store_initializer(user_settings, &index_path);
+        let index_store = index_store_initializer(user_settings, &index_path)?;
         let index_type_path = index_path.join("type");
         fs::write(&index_type_path, index_store.name()).context(&index_type_path)?;
         let index_store = Arc::from(index_store);
@@ -255,28 +253,28 @@ impl ReadonlyRepo {
     pub fn readonly_index(&self) -> &dyn ReadonlyIndex {
         self.index
             .get_or_init(|| {
-                Box::into_pin(
-                    self.index_store
-                        .get_index_at_op(&self.operation, &self.store),
-                )
+                // TODO: somehow propagate error, but it's weird if all callers
+                // had Result<T, IndexReadError> signature.
+                self.index_store
+                    .get_index_at_op(&self.operation, &self.store)
+                    .unwrap()
             })
             .deref()
     }
 
-    fn change_id_index<'a>(&'a self) -> &'a (dyn ChangeIdIndex + 'a) {
-        let change_id_index: &'a (dyn ChangeIdIndex + 'a) = self
-            .change_id_index
+    fn change_id_index(&self) -> &dyn ChangeIdIndex {
+        self.change_id_index
             .get_or_init(|| {
-                let revset: Box<dyn Revset<'a>> =
-                    RevsetExpression::all().evaluate_programmatic(self).unwrap();
-                let change_id_index: Box<dyn ChangeIdIndex + 'a> = revset.change_id_index();
-                // evaluate() above only borrows the index, not the whole repo
-                let change_id_index: Box<dyn ChangeIdIndex> =
-                    unsafe { std::mem::transmute(change_id_index) };
-                change_id_index
+                // TODO: maybe add abstraction over 'static/'index revset
+                // evaluation, and use it.
+                let expression = RevsetExpression::all().resolve_programmatic(self);
+                let revset = self
+                    .readonly_index()
+                    .evaluate_revset_static(&expression, self.store())
+                    .unwrap();
+                revset.change_id_index()
             })
-            .as_ref();
-        change_id_index
+            .as_ref()
     }
 
     pub fn op_heads_store(&self) -> &Arc<dyn OpHeadsStore> {
@@ -294,16 +292,15 @@ impl ReadonlyRepo {
     pub fn start_transaction(
         self: &Arc<ReadonlyRepo>,
         user_settings: &UserSettings,
-        description: &str,
     ) -> Transaction {
         let mut_repo = MutableRepo::new(self.clone(), self.readonly_index(), &self.view);
-        Transaction::new(mut_repo, user_settings, description)
+        Transaction::new(mut_repo, user_settings)
     }
 
     pub fn reload_at_head(
         &self,
         user_settings: &UserSettings,
-    ) -> Result<Arc<ReadonlyRepo>, OpHeadResolutionError<RepoLoaderError>> {
+    ) -> Result<Arc<ReadonlyRepo>, RepoLoaderError> {
         self.loader().load_at_head(user_settings)
     }
 
@@ -347,7 +344,8 @@ pub type BackendInitializer<'a> =
     dyn Fn(&UserSettings, &Path) -> Result<Box<dyn Backend>, BackendInitError> + 'a;
 pub type OpStoreInitializer<'a> = dyn Fn(&UserSettings, &Path) -> Box<dyn OpStore> + 'a;
 pub type OpHeadsStoreInitializer<'a> = dyn Fn(&UserSettings, &Path) -> Box<dyn OpHeadsStore> + 'a;
-pub type IndexStoreInitializer<'a> = dyn Fn(&UserSettings, &Path) -> Box<dyn IndexStore> + 'a;
+pub type IndexStoreInitializer<'a> =
+    dyn Fn(&UserSettings, &Path) -> Result<Box<dyn IndexStore>, BackendInitError> + 'a;
 pub type SubmoduleStoreInitializer<'a> =
     dyn Fn(&UserSettings, &Path) -> Box<dyn SubmoduleStore> + 'a;
 
@@ -355,7 +353,8 @@ type BackendFactory =
     Box<dyn Fn(&UserSettings, &Path) -> Result<Box<dyn Backend>, BackendLoadError>>;
 type OpStoreFactory = Box<dyn Fn(&UserSettings, &Path) -> Box<dyn OpStore>>;
 type OpHeadsStoreFactory = Box<dyn Fn(&UserSettings, &Path) -> Box<dyn OpHeadsStore>>;
-type IndexStoreFactory = Box<dyn Fn(&UserSettings, &Path) -> Box<dyn IndexStore>>;
+type IndexStoreFactory =
+    Box<dyn Fn(&UserSettings, &Path) -> Result<Box<dyn IndexStore>, BackendLoadError>>;
 type SubmoduleStoreFactory = Box<dyn Fn(&UserSettings, &Path) -> Box<dyn SubmoduleStore>>;
 
 pub struct StoreFactories {
@@ -395,7 +394,7 @@ impl Default for StoreFactories {
         // Index
         factories.add_index_store(
             DefaultIndexStore::name(),
-            Box::new(|_settings, store_path| Box::new(DefaultIndexStore::load(store_path))),
+            Box::new(|_settings, store_path| Ok(Box::new(DefaultIndexStore::load(store_path)))),
         );
 
         // SubmoduleStores
@@ -534,7 +533,7 @@ impl StoreFactories {
                 store: "index",
                 store_type: index_store_type.to_string(),
             })?;
-        Ok(index_store_factory(settings, store_path))
+        Ok(index_store_factory(settings, store_path)?)
     }
 
     pub fn add_submodule_store(&mut self, name: &str, factory: SubmoduleStoreFactory) {
@@ -592,6 +591,8 @@ pub enum RepoLoaderError {
     #[error(transparent)]
     TreeMerge(#[from] TreeMergeError),
     #[error(transparent)]
+    OpHeadResolution(#[from] OpHeadResolutionError),
+    #[error(transparent)]
     OpStore(#[from] OpStoreError),
 }
 
@@ -663,7 +664,7 @@ impl RepoLoader {
     pub fn load_at_head(
         &self,
         user_settings: &UserSettings,
-    ) -> Result<Arc<ReadonlyRepo>, OpHeadResolutionError<RepoLoaderError>> {
+    ) -> Result<Arc<ReadonlyRepo>, RepoLoaderError> {
         let op = op_heads_store::resolve_op_heads(
             self.op_heads_store.as_ref(),
             &self.op_store,
@@ -694,7 +695,7 @@ impl RepoLoader {
             settings: self.repo_settings.clone(),
             index_store: self.index_store.clone(),
             submodule_store: self.submodule_store.clone(),
-            index: OnceCell::with_value(Box::into_pin(index)),
+            index: OnceCell::with_value(index),
             change_id_index: OnceCell::new(),
             view,
         };
@@ -707,12 +708,14 @@ impl RepoLoader {
         user_settings: &UserSettings,
     ) -> Result<Operation, RepoLoaderError> {
         let base_repo = self.load_at(&op_heads[0])?;
-        let mut tx = base_repo.start_transaction(user_settings, "resolve concurrent operations");
+        let mut tx = base_repo.start_transaction(user_settings);
         for other_op_head in op_heads.into_iter().skip(1) {
             tx.merge_operation(other_op_head)?;
             tx.mut_repo().rebase_descendants(user_settings)?;
         }
-        let merged_repo = tx.write().leave_unpublished();
+        let merged_repo = tx
+            .write("resolve concurrent operations")
+            .leave_unpublished();
         Ok(merged_repo.operation().clone())
     }
 
@@ -797,6 +800,8 @@ impl MutableRepo {
         predecessor: &Commit,
     ) -> CommitBuilder {
         CommitBuilder::for_rewrite_from(self, settings, predecessor)
+        // CommitBuilder::write will record the rewrite in
+        // `self.rewritten_commits`
     }
 
     pub fn write_commit(
@@ -805,16 +810,47 @@ impl MutableRepo {
         sign_with: Option<&mut SigningFn>,
     ) -> BackendResult<Commit> {
         let commit = self.store().write_commit(commit, sign_with)?;
-        self.add_head(&commit);
+        self.add_head(&commit)?;
         Ok(commit)
     }
 
-    /// Record a commit as having been rewritten in this transaction. This
-    /// record is used by `rebase_descendants()`.
+    /// Record a commit as having been rewritten to one or more ids in this
+    /// transaction.
     ///
-    /// Rewritten commits don't have to be recorded here. This is just a
-    /// convenient place to record it. It won't matter after the transaction
-    /// has been committed.
+    /// This record is used by `rebase_descendants` to know which commits have
+    /// children that need to be rebased, and where to rebase them to. See the
+    /// docstring for `record_rewritten_commit` for details.
+    //  TODO(ilyagr): Make this API saner, e.g. make `self.rewritten_commits` public
+    // and make empty values correspond to abandoned commits.
+    pub fn set_rewritten_commit(
+        &mut self,
+        old_id: CommitId,
+        new_ids: impl IntoIterator<Item = CommitId>,
+    ) {
+        assert_ne!(old_id, *self.store().root_commit_id());
+        self.rewritten_commits
+            .insert(old_id, new_ids.into_iter().collect());
+    }
+
+    /// Record a commit as having been rewritten in this transaction. If it was
+    /// already rewritten, mark it as divergent (unlike `set_rewritten_commit`)
+    ///
+    /// This record is used by `rebase_descendants` to know which commits have
+    /// children that need to be rebased, and where to rebase the children (as
+    /// well as branches) to.
+    ///
+    /// The `rebase_descendants` logic treats these records as follows:
+    ///
+    /// - If a commit is recorded as rewritten to a single commit, its
+    ///   descendants would be rebased to become descendants of `new_id`. Any
+    ///   branches at `old_id` are also moved to `new_id`.
+    /// - If a commit is recorded as rewritten to more than one commit, it is
+    ///   assumed to have become divergent. Its descendants are *not* rebased.
+    ///   However, the *branches* that were at `old_id` are moved to each of the
+    ///   new ids, and thus become conflicted.
+    ///
+    /// In neither case would `rebase_descendants` modify the `old_id` commit
+    /// itself.
     pub fn record_rewritten_commit(&mut self, old_id: CommitId, new_id: CommitId) {
         assert_ne!(old_id, *self.store().root_commit_id());
         self.rewritten_commits
@@ -823,22 +859,22 @@ impl MutableRepo {
             .insert(new_id);
     }
 
-    pub fn clear_rewritten_commits(&mut self) {
-        self.rewritten_commits.clear();
-    }
-
-    /// Record a commit as having been abandoned in this transaction. This
-    /// record is used by `rebase_descendants()`.
+    /// Record a commit as having been abandoned in this transaction.
     ///
-    /// Abandoned commits don't have to be recorded here. This is just a
-    /// convenient place to record it. It won't matter after the transaction
-    /// has been committed.
+    /// This record is used by `rebase_descendants` to know which commits have
+    /// children that need to be rebased, and where to rebase the children (as
+    /// well as branches) to.
+    ///
+    /// The `rebase_descendants` logic will rebase the descendants of `old_id`
+    /// to become the descendants of parent(s) of `old_id`. Any branches at
+    /// `old_id` would be moved to the parent(s) of `old_id` as well.
     pub fn record_abandoned_commit(&mut self, old_id: CommitId) {
         assert_ne!(old_id, *self.store().root_commit_id());
         self.abandoned_commits.insert(old_id);
     }
 
-    pub fn clear_abandoned_commits(&mut self) {
+    fn clear_descendant_rebaser_plans(&mut self) {
+        self.rewritten_commits.clear();
         self.abandoned_commits.clear();
     }
 
@@ -846,20 +882,8 @@ impl MutableRepo {
         !(self.rewritten_commits.is_empty() && self.abandoned_commits.is_empty())
     }
 
-    /// Creates a `DescendantRebaser` to rebase descendants of the recorded
-    /// rewritten and abandoned commits.
-    pub fn create_descendant_rebaser<'settings, 'repo>(
-        &'repo mut self,
-        settings: &'settings UserSettings,
-    ) -> DescendantRebaser<'settings, 'repo> {
-        DescendantRebaser::new(
-            settings,
-            self,
-            self.rewritten_commits.clone(),
-            self.abandoned_commits.clone(),
-        )
-    }
-
+    /// After the rebaser returned by this function is dropped,
+    /// self.clear_descendant_rebaser_plans() needs to be called.
     fn rebase_descendants_return_rebaser<'settings, 'repo>(
         &'repo mut self,
         settings: &'settings UserSettings,
@@ -869,20 +893,58 @@ impl MutableRepo {
             // Optimization
             return Ok(None);
         }
-        let mut rebaser = self.create_descendant_rebaser(settings);
+        let mut rebaser = DescendantRebaser::new(
+            settings,
+            self,
+            self.rewritten_commits.clone(),
+            self.abandoned_commits.clone(),
+        );
         *rebaser.mut_options() = options;
         rebaser.rebase_all()?;
         Ok(Some(rebaser))
     }
 
+    // TODO(ilyagr): Either document that this also moves branches (rename the
+    // function and the related functions?) or change things so that this only
+    // rebases descendants.
     pub fn rebase_descendants_with_options(
         &mut self,
         settings: &UserSettings,
         options: RebaseOptions,
     ) -> Result<usize, TreeMergeError> {
-        Ok(self
+        let result = self
             .rebase_descendants_return_rebaser(settings, options)?
-            .map_or(0, |rebaser| rebaser.rebased().len()))
+            .map_or(0, |rebaser| rebaser.into_map().len());
+        self.clear_descendant_rebaser_plans();
+        Ok(result)
+    }
+
+    /// This is similar to `rebase_descendants_return_map`, but the return value
+    /// needs more explaining.
+    ///
+    /// If the `options.empty` is the default, this function will only
+    /// rebase commits, and the return value is what you'd expect it to be.
+    ///
+    /// Otherwise, this function may rebase some commits and abandon others. The
+    /// behavior is such that only commits with a single parent will ever be
+    /// abandoned. In the returned map, an abandoned commit will look as a
+    /// key-value pair where the key is the abandoned commit and the value is
+    /// **its parent**. One can tell this case apart since the change ids of the
+    /// key and the value will not match. The parent will inherit the
+    /// descendants and the branches of the abandoned commit.
+    pub fn rebase_descendants_with_options_return_map(
+        &mut self,
+        settings: &UserSettings,
+        options: RebaseOptions,
+    ) -> Result<HashMap<CommitId, CommitId>, TreeMergeError> {
+        let result = Ok(self
+            // We do not set RebaseOptions here, since this function does not currently return
+            // enough information to describe the results of a rebase if some commits got
+            // abandoned
+            .rebase_descendants_return_rebaser(settings, options)?
+            .map_or(HashMap::new(), |rebaser| rebaser.into_map()));
+        self.clear_descendant_rebaser_plans();
+        result
     }
 
     pub fn rebase_descendants(&mut self, settings: &UserSettings) -> Result<usize, TreeMergeError> {
@@ -893,12 +955,7 @@ impl MutableRepo {
         &mut self,
         settings: &UserSettings,
     ) -> Result<HashMap<CommitId, CommitId>, TreeMergeError> {
-        Ok(self
-            // We do not set RebaseOptions here, since this function does not currently return
-            // enough information to describe the results of a rebase if some commits got
-            // abandoned
-            .rebase_descendants_return_rebaser(settings, Default::default())?
-            .map_or(HashMap::new(), |rebaser| rebaser.rebased().clone()))
+        self.rebase_descendants_with_options_return_map(settings, Default::default())
     }
 
     pub fn set_wc_commit(
@@ -984,11 +1041,19 @@ impl MutableRepo {
             .collect();
     }
 
-    pub fn add_head(&mut self, head: &Commit) {
-        self.add_heads(slice::from_ref(head));
+    /// Ensures that the given `head` and ancestor commits are reachable from
+    /// the visible heads.
+    pub fn add_head(&mut self, head: &Commit) -> BackendResult<()> {
+        self.add_heads(slice::from_ref(head))
     }
 
-    pub fn add_heads(&mut self, heads: &[Commit]) {
+    /// Ensures that the given `heads` and ancestor commits are reachable from
+    /// the visible heads.
+    ///
+    /// The `heads` may contain redundant commits such as already visible ones
+    /// and ancestors of the other heads. The `heads` and ancestor commits
+    /// should exist in the store.
+    pub fn add_heads(&mut self, heads: &[Commit]) -> BackendResult<()> {
         let current_heads = self.view.get_mut().heads();
         // Use incremental update for common case of adding a single commit on top a
         // current head. TODO: Also use incremental update when adding a single
@@ -1008,19 +1073,23 @@ impl MutableRepo {
                 }
             }
             _ => {
-                let missing_commits = dag_walk::topo_order_reverse_ord(
-                    heads.iter().cloned().map(CommitByCommitterTimestamp),
+                let missing_commits = dag_walk::topo_order_reverse_ord_ok(
+                    heads
+                        .iter()
+                        .cloned()
+                        .map(CommitByCommitterTimestamp)
+                        .map(Ok),
                     |CommitByCommitterTimestamp(commit)| commit.id().clone(),
                     |CommitByCommitterTimestamp(commit)| {
                         commit
                             .parent_ids()
                             .iter()
                             .filter(|id| !self.index().has_id(id))
-                            .map(|id| self.store().get_commit(id).unwrap())
-                            .map(CommitByCommitterTimestamp)
+                            .map(|id| self.store().get_commit(id))
+                            .map_ok(CommitByCommitterTimestamp)
                             .collect_vec()
                     },
-                );
+                )?;
                 for CommitByCommitterTimestamp(missing_commit) in missing_commits.iter().rev() {
                     self.index.add_commit(missing_commit);
                 }
@@ -1030,6 +1099,7 @@ impl MutableRepo {
                 self.view.mark_dirty();
             }
         }
+        Ok(())
     }
 
     pub fn remove_head(&mut self, head: &CommitId) {
