@@ -47,7 +47,7 @@ pub struct GitSettings {
 impl GitSettings {
     pub fn from_config(config: &config::Config) -> Self {
         GitSettings {
-            auto_local_branch: config.get_bool("git.auto-local-branch").unwrap_or(true),
+            auto_local_branch: config.get_bool("git.auto-local-branch").unwrap_or(false),
             abandon_unreachable_commits: config
                 .get_bool("git.abandon-unreachable-commits")
                 .unwrap_or(true),
@@ -58,7 +58,7 @@ impl GitSettings {
 impl Default for GitSettings {
     fn default() -> Self {
         GitSettings {
-            auto_local_branch: true,
+            auto_local_branch: false,
             abandon_unreachable_commits: true,
         }
     }
