@@ -30,7 +30,7 @@ use crate::backend::{
 use crate::files::MergeResult;
 use crate::matchers::{EverythingMatcher, Matcher};
 use crate::merge::{trivial_merge, Merge, MergedTreeValue};
-use crate::repo_path::{RepoPath, RepoPathComponent, RepoPathJoin};
+use crate::repo_path::{RepoPath, RepoPathBuf, RepoPathComponent, RepoPathComponentsIter};
 use crate::store::Store;
 use crate::{backend, files};
 
@@ -48,7 +48,7 @@ pub enum TreeMergeError {
 #[derive(Clone)]
 pub struct Tree {
     store: Arc<Store>,
-    dir: RepoPath,
+    dir: RepoPathBuf,
     id: TreeId,
     data: Arc<backend::Tree>,
 }
@@ -78,7 +78,7 @@ impl Hash for Tree {
 }
 
 impl Tree {
-    pub fn new(store: Arc<Store>, dir: RepoPath, id: TreeId, data: Arc<backend::Tree>) -> Self {
+    pub fn new(store: Arc<Store>, dir: RepoPathBuf, id: TreeId, data: Arc<backend::Tree>) -> Self {
         Tree {
             store,
             dir,
@@ -87,7 +87,7 @@ impl Tree {
         }
     }
 
-    pub fn null(store: Arc<Store>, dir: RepoPath) -> Self {
+    pub fn null(store: Arc<Store>, dir: RepoPathBuf) -> Self {
         Tree {
             store,
             dir,
@@ -136,7 +136,7 @@ impl Tree {
     }
 
     pub fn path_value(&self, path: &RepoPath) -> Option<TreeValue> {
-        assert_eq!(self.dir(), &RepoPath::root());
+        assert_eq!(self.dir(), RepoPath::root());
         match path.split() {
             Some((dir, basename)) => self
                 .sub_tree_recursive(dir.components())
@@ -159,10 +159,9 @@ impl Tree {
         self.store.get_tree(subdir, id).unwrap()
     }
 
-    fn sub_tree_recursive(&self, components: &[RepoPathComponent]) -> Option<Tree> {
-        if let Some((first, tail)) = components.split_first() {
-            tail.iter()
-                .try_fold(self.sub_tree(first)?, |tree, name| tree.sub_tree(name))
+    fn sub_tree_recursive(&self, mut components: RepoPathComponentsIter) -> Option<Tree> {
+        if let Some(first) = components.next() {
+            components.try_fold(self.sub_tree(first)?, |tree, name| tree.sub_tree(name))
         } else {
             // TODO: It would be nice to be able to return a reference here, but
             // then we would have to figure out how to share Tree instances
@@ -171,7 +170,7 @@ impl Tree {
         }
     }
 
-    pub fn conflicts_matching(&self, matcher: &dyn Matcher) -> Vec<(RepoPath, ConflictId)> {
+    pub fn conflicts_matching(&self, matcher: &dyn Matcher) -> Vec<(RepoPathBuf, ConflictId)> {
         let mut conflicts = vec![];
         for (name, value) in self.entries_matching(matcher) {
             if let TreeValue::Conflict(id) = value {
@@ -182,7 +181,7 @@ impl Tree {
     }
 
     #[instrument]
-    pub fn conflicts(&self) -> Vec<(RepoPath, ConflictId)> {
+    pub fn conflicts(&self) -> Vec<(RepoPathBuf, ConflictId)> {
         self.conflicts_matching(&EverythingMatcher)
     }
 
@@ -197,21 +196,18 @@ pub struct TreeEntriesIterator<'matcher> {
 }
 
 struct TreeEntriesDirItem {
-    entry_iterator: TreeEntriesNonRecursiveIterator<'static>,
-    // On drop, tree must outlive entry_iterator
-    tree: Box<Tree>,
+    tree: Tree,
+    entries: Vec<(RepoPathBuf, TreeValue)>,
 }
 
-impl TreeEntriesDirItem {
-    fn new(tree: Tree) -> Self {
-        let tree = Box::new(tree);
-        let entry_iterator = tree.entries_non_recursive();
-        let entry_iterator: TreeEntriesNonRecursiveIterator<'static> =
-            unsafe { std::mem::transmute(entry_iterator) };
-        Self {
-            entry_iterator,
-            tree,
-        }
+impl From<Tree> for TreeEntriesDirItem {
+    fn from(tree: Tree) -> Self {
+        let mut entries = tree
+            .entries_non_recursive()
+            .map(|entry| (tree.dir().join(entry.name()), entry.value().clone()))
+            .collect_vec();
+        entries.reverse();
+        Self { tree, entries }
     }
 }
 
@@ -219,31 +215,30 @@ impl<'matcher> TreeEntriesIterator<'matcher> {
     fn new(tree: Tree, matcher: &'matcher dyn Matcher) -> Self {
         // TODO: Restrict walk according to Matcher::visit()
         Self {
-            stack: vec![TreeEntriesDirItem::new(tree)],
+            stack: vec![TreeEntriesDirItem::from(tree)],
             matcher,
         }
     }
 }
 
 impl Iterator for TreeEntriesIterator<'_> {
-    type Item = (RepoPath, TreeValue);
+    type Item = (RepoPathBuf, TreeValue);
 
     fn next(&mut self) -> Option<Self::Item> {
         while let Some(top) = self.stack.last_mut() {
-            if let Some(entry) = top.entry_iterator.next() {
-                let path = top.tree.dir().join(entry.name());
-                match entry.value() {
+            if let Some((path, value)) = top.entries.pop() {
+                match value {
                     TreeValue::Tree(id) => {
                         // TODO: Handle the other cases (specific files and trees)
                         if self.matcher.visit(&path).is_nothing() {
                             continue;
                         }
-                        let subtree = top.tree.known_sub_tree(&path, id);
-                        self.stack.push(TreeEntriesDirItem::new(subtree));
+                        let subtree = top.tree.known_sub_tree(&path, &id);
+                        self.stack.push(TreeEntriesDirItem::from(subtree));
                     }
                     value => {
                         if self.matcher.matches(&path) {
-                            return Some((path, value.clone()));
+                            return Some((path, value));
                         }
                     }
                 };
@@ -373,10 +368,11 @@ fn merge_tree_value(
         _ => {
             // Start by creating a Merge object. Merges can cleanly represent a single
             // resolved state, the absence of a state, or a conflicted state.
-            let conflict = Merge::new(
-                vec![maybe_base.cloned()],
-                vec![maybe_side1.cloned(), maybe_side2.cloned()],
-            );
+            let conflict = Merge::from_vec(vec![
+                maybe_side1.cloned(),
+                maybe_base.cloned(),
+                maybe_side2.cloned(),
+            ]);
             let filename = dir.join(basename);
             let expanded = conflict.try_map(|term| match term {
                 Some(TreeValue::Conflict(id)) => store.read_conflict(&filename, id),
@@ -400,6 +396,10 @@ fn merge_tree_value(
     })
 }
 
+/// Resolves file-level conflict by merging content hunks.
+///
+/// The input `conflict` is supposed to be simplified. It shouldn't contain
+/// non-file values that cancel each other.
 pub fn try_resolve_file_conflict(
     store: &Store,
     filename: &RepoPath,
@@ -434,7 +434,9 @@ pub fn try_resolve_file_conflict(
         }));
     }
 
-    // Simplify the conflict for two reasons:
+    // While the input conflict should be simplified by caller, it might contain
+    // terms which only differ in executable bits. Simplify the conflict further
+    // for two reasons:
     // 1. Avoid reading unchanged file contents
     // 2. The simplified conflict can sometimes be resolved when the unsimplfied one
     //    cannot
@@ -453,7 +455,7 @@ pub fn try_resolve_file_conflict(
             Ok(content)
         })?;
     let slices = contents.map(|content| content.as_slice());
-    let merge_result = files::merge(slices);
+    let merge_result = files::merge(&slices);
     match merge_result {
         MergeResult::Resolved(merged_content) => {
             let id = store.write_file(filename, &mut merged_content.0.as_slice())?;
