@@ -28,7 +28,7 @@ use crate::fsmonitor::FsmonitorKind;
 use crate::gitignore::GitIgnoreFile;
 use crate::merged_tree::MergedTree;
 use crate::op_store::{OperationId, WorkspaceId};
-use crate::repo_path::RepoPath;
+use crate::repo_path::{RepoPath, RepoPathBuf};
 use crate::settings::HumanByteSize;
 
 /// The trait all working-copy implementations must implement.
@@ -56,7 +56,7 @@ pub trait WorkingCopy {
     /// out in the working copy. An empty list means that no paths should be
     /// checked out in the working copy. A single `RepoPath::root()` entry means
     /// that all files should be checked out.
-    fn sparse_patterns(&self) -> Result<&[RepoPath], WorkingCopyStateError>;
+    fn sparse_patterns(&self) -> Result<&[RepoPathBuf], WorkingCopyStateError>;
 
     /// Locks the working copy and returns an instance with methods for updating
     /// the working copy files and state.
@@ -87,7 +87,7 @@ pub trait LockedWorkingCopy {
     fn reset(&mut self, new_tree: &MergedTree) -> Result<(), ResetError>;
 
     /// See `WorkingCopy::sparse_patterns()`
-    fn sparse_patterns(&self) -> Result<&[RepoPath], WorkingCopyStateError>;
+    fn sparse_patterns(&self) -> Result<&[RepoPathBuf], WorkingCopyStateError>;
 
     /// Updates the patterns that decide which paths from the current tree
     /// should be checked out in the working copy.
@@ -97,7 +97,7 @@ pub trait LockedWorkingCopy {
     // to use sparse).
     fn set_sparse_patterns(
         &mut self,
-        new_sparse_patterns: Vec<RepoPath>,
+        new_sparse_patterns: Vec<RepoPathBuf>,
     ) -> Result<CheckoutStats, CheckoutError>;
 
     /// Finish the modifications to the working copy by writing the updated
@@ -141,7 +141,7 @@ pub enum SnapshotError {
         max_size: HumanByteSize,
     },
     /// Some other error happened while snapshotting the working copy.
-    #[error("{message}: {err:?}")]
+    #[error("{message}: {err}")]
     Other {
         /// Error message.
         message: String,
@@ -224,7 +224,7 @@ pub enum CheckoutError {
     #[error("Internal backend error: {0}")]
     InternalBackendError(#[from] BackendError),
     /// Some other error happened while checking out the working copy.
-    #[error("{message}: {err:?}")]
+    #[error("{message}: {err}")]
     Other {
         /// Error message.
         message: String,
@@ -248,7 +248,7 @@ pub enum ResetError {
     #[error("Internal error: {0}")]
     InternalBackendError(#[from] BackendError),
     /// Some other error happened while checking out the working copy.
-    #[error("{message}: {err:?}")]
+    #[error("{message}: {err}")]
     Other {
         /// Error message.
         message: String,
@@ -260,7 +260,7 @@ pub enum ResetError {
 
 /// An error while reading the working copy state.
 #[derive(Debug, Error)]
-#[error("{message}: {err:?}")]
+#[error("{message}: {err}")]
 pub struct WorkingCopyStateError {
     /// Error message.
     pub message: String,
