@@ -54,9 +54,9 @@ fn test_commit_parallel(backend: TestRepoBackend) {
             let settings = settings.clone();
             let repo = repo.clone();
             s.spawn(move || {
-                let mut tx = repo.start_transaction(&settings, "test");
+                let mut tx = repo.start_transaction(&settings);
                 write_random_commit(tx.mut_repo(), &settings);
-                tx.commit();
+                tx.commit("test");
             });
         }
     });
@@ -85,9 +85,9 @@ fn test_commit_parallel_instances(backend: TestRepoBackend) {
             let settings = settings.clone();
             let repo = load_repo_at_head(&settings, repo.repo_path());
             s.spawn(move || {
-                let mut tx = repo.start_transaction(&settings, "test");
+                let mut tx = repo.start_transaction(&settings);
                 write_random_commit(tx.mut_repo(), &settings);
-                tx.commit();
+                tx.commit("test");
             });
         }
     });
