@@ -12,7 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use futures::executor::block_on;
 use jj_lib::backend::FileId;
 use jj_lib::conflicts::{
     extract_as_single_hunk, materialize_merge_result, parse_conflict, update_from_content,
@@ -21,6 +20,7 @@ use jj_lib::merge::Merge;
 use jj_lib::repo::Repo;
 use jj_lib::repo_path::RepoPath;
 use jj_lib::store::Store;
+use pollster::FutureExt;
 use testutils::TestRepo;
 
 #[test]
@@ -31,7 +31,7 @@ fn test_materialize_conflict_basic() {
     let path = RepoPath::from_internal_string("file");
     let base_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2
 line 3
@@ -41,7 +41,7 @@ line 5
     );
     let left_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2
 left 3.1
@@ -53,7 +53,7 @@ line 5
     );
     let right_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2
 right 3.1
@@ -64,12 +64,12 @@ line 5
 
     // The left side should come first. The diff should be use the smaller (right)
     // side, and the left side should be a snapshot.
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone())],
         vec![Some(left_id.clone()), Some(right_id.clone())],
     );
     insta::assert_snapshot!(
-        &materialize_conflict_string(store, &path, &conflict),
+        &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
     line 2
@@ -88,12 +88,12 @@ line 5
     );
     // Swap the positive terms in the conflict. The diff should still use the right
     // side, but now the right side should come first.
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone())],
         vec![Some(right_id.clone()), Some(left_id.clone())],
     );
     insta::assert_snapshot!(
-        &materialize_conflict_string(store, &path, &conflict),
+        &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
     line 2
@@ -121,7 +121,7 @@ fn test_materialize_conflict_multi_rebase_conflicts() {
     let path = RepoPath::from_internal_string("file");
     let base_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2 base
 line 3
@@ -129,7 +129,7 @@ line 3
     );
     let a_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2 a.1
 line 2 a.2
@@ -139,7 +139,7 @@ line 3
     );
     let b_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2 b.1
 line 2 b.2
@@ -148,7 +148,7 @@ line 3
     );
     let c_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2 c.1
 line 3
@@ -157,12 +157,12 @@ line 3
 
     // The order of (a, b, c) should be preserved. For all cases, the "a" side
     // should be a snapshot.
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone()), Some(base_id.clone())],
         vec![Some(a_id.clone()), Some(b_id.clone()), Some(c_id.clone())],
     );
     insta::assert_snapshot!(
-        &materialize_conflict_string(store, &path, &conflict),
+        &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
     <<<<<<<
@@ -181,12 +181,12 @@ line 3
     line 3
     "###
     );
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone()), Some(base_id.clone())],
         vec![Some(c_id.clone()), Some(b_id.clone()), Some(a_id.clone())],
     );
     insta::assert_snapshot!(
-        &materialize_conflict_string(store, &path, &conflict),
+        &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
     <<<<<<<
@@ -205,12 +205,12 @@ line 3
     line 3
     "###
     );
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone()), Some(base_id.clone())],
         vec![Some(c_id.clone()), Some(a_id.clone()), Some(b_id.clone())],
     );
     insta::assert_snapshot!(
-        &materialize_conflict_string(store, &path, &conflict),
+        &materialize_conflict_string(store, path, &conflict),
         @r###"
     line 1
     <<<<<<<
@@ -239,7 +239,7 @@ fn test_materialize_parse_roundtrip() {
     let path = RepoPath::from_internal_string("file");
     let base_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2
 line 3
@@ -249,7 +249,7 @@ line 5
     );
     let left_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1 left
 line 2 left
 line 3
@@ -259,7 +259,7 @@ line 5 left
     );
     let right_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1 right
 line 2
 line 3
@@ -268,11 +268,11 @@ line 5 right
 ",
     );
 
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone())],
         vec![Some(left_id.clone()), Some(right_id.clone())],
     );
-    let materialized = materialize_conflict_string(store, &path, &conflict);
+    let materialized = materialize_conflict_string(store, path, &conflict);
     insta::assert_snapshot!(
         materialized,
         @r###"
@@ -304,27 +304,23 @@ line 5 right
         @r###"
     Some(
         [
-            Conflicted {
-                removes: [
-                    "line 1\nline 2\n",
-                ],
-                adds: [
+            Conflicted(
+                [
                     "line 1 left\nline 2 left\n",
+                    "line 1\nline 2\n",
                     "line 1 right\nline 2\n",
                 ],
-            },
+            ),
             Resolved(
                 "line 3\n",
             ),
-            Conflicted {
-                removes: [
-                    "line 4\nline 5\n",
-                ],
-                adds: [
+            Conflicted(
+                [
                     "line 4\nline 5 left\n",
+                    "line 4\nline 5\n",
                     "line 4 right\nline 5 right\n",
                 ],
-            },
+            ),
         ],
     )
     "###);
@@ -338,7 +334,7 @@ fn test_materialize_conflict_modify_delete() {
     let path = RepoPath::from_internal_string("file");
     let base_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2
 line 3
@@ -348,7 +344,7 @@ line 5
     );
     let modified_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2
 modified
@@ -358,7 +354,7 @@ line 5
     );
     let deleted_id = testutils::write_file(
         store,
-        &path,
+        path,
         "line 1
 line 2
 line 4
@@ -367,11 +363,11 @@ line 5
     );
 
     // left modifies a line, right deletes the same line.
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone())],
         vec![Some(modified_id.clone()), Some(deleted_id.clone())],
     );
-    insta::assert_snapshot!(&materialize_conflict_string(store, &path, &conflict), @r###"
+    insta::assert_snapshot!(&materialize_conflict_string(store, path, &conflict), @r###"
     line 1
     line 2
     <<<<<<<
@@ -386,11 +382,11 @@ line 5
     );
 
     // right modifies a line, left deletes the same line.
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone())],
         vec![Some(deleted_id.clone()), Some(modified_id.clone())],
     );
-    insta::assert_snapshot!(&materialize_conflict_string(store, &path, &conflict), @r###"
+    insta::assert_snapshot!(&materialize_conflict_string(store, path, &conflict), @r###"
     line 1
     line 2
     <<<<<<<
@@ -405,11 +401,11 @@ line 5
     );
 
     // modify/delete conflict at the file level
-    let conflict = Merge::new(
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_id.clone())],
         vec![Some(modified_id.clone()), None],
     );
-    insta::assert_snapshot!(&materialize_conflict_string(store, &path, &conflict), @r###"
+    insta::assert_snapshot!(&materialize_conflict_string(store, path, &conflict), @r###"
     <<<<<<<
     %%%%%%%
      line 1
@@ -464,15 +460,13 @@ line 5
             Resolved(
                 "line 1\n",
             ),
-            Conflicted {
-                removes: [
-                    "line 2\nline 3\nline 4\n",
-                ],
-                adds: [
+            Conflicted(
+                [
                     "line 2\nleft\nline 4\n",
+                    "line 2\nline 3\nline 4\n",
                     "right\n",
                 ],
-            },
+            ),
             Resolved(
                 "line 5\n",
             ),
@@ -511,17 +505,15 @@ line 5
             Resolved(
                 "line 1\n",
             ),
-            Conflicted {
-                removes: [
-                    "line 2\nline 3\nline 4\n",
-                    "line 2\nline 3\nline 4\n",
-                ],
-                adds: [
+            Conflicted(
+                [
                     "line 2\nleft\nline 4\n",
+                    "line 2\nline 3\nline 4\n",
                     "right\n",
+                    "line 2\nline 3\nline 4\n",
                     "line 2\nforward\nline 3\nline 4\n",
                 ],
-            },
+            ),
             Resolved(
                 "line 5\n",
             ),
@@ -604,22 +596,26 @@ fn test_update_conflict_from_content() {
     let store = test_repo.repo.store();
 
     let path = RepoPath::from_internal_string("dir/file");
-    let base_file_id = testutils::write_file(store, &path, "line 1\nline 2\nline 3\n");
-    let left_file_id = testutils::write_file(store, &path, "left 1\nline 2\nleft 3\n");
-    let right_file_id = testutils::write_file(store, &path, "right 1\nline 2\nright 3\n");
-    let conflict = Merge::new(
+    let base_file_id = testutils::write_file(store, path, "line 1\nline 2\nline 3\n");
+    let left_file_id = testutils::write_file(store, path, "left 1\nline 2\nleft 3\n");
+    let right_file_id = testutils::write_file(store, path, "right 1\nline 2\nright 3\n");
+    let conflict = Merge::from_removes_adds(
         vec![Some(base_file_id.clone())],
         vec![Some(left_file_id.clone()), Some(right_file_id.clone())],
     );
 
     // If the content is unchanged compared to the materialized value, we get the
     // old conflict id back.
-    let materialized = materialize_conflict_string(store, &path, &conflict);
-    let parse = |content| block_on(update_from_content(&conflict, store, &path, content)).unwrap();
+    let materialized = materialize_conflict_string(store, path, &conflict);
+    let parse = |content| {
+        update_from_content(&conflict, store, path, content)
+            .block_on()
+            .unwrap()
+    };
     assert_eq!(parse(materialized.as_bytes()), conflict);
 
     // If the conflict is resolved, we get None back to indicate that.
-    let expected_file_id = testutils::write_file(store, &path, "resolved 1\nline 2\nresolved 3\n");
+    let expected_file_id = testutils::write_file(store, path, "resolved 1\nline 2\nresolved 3\n");
     assert_eq!(
         parse(b"resolved 1\nline 2\nresolved 3\n"),
         Merge::normal(expected_file_id)
@@ -631,12 +627,12 @@ fn test_update_conflict_from_content() {
     );
     assert_ne!(new_conflict, conflict);
     // Calculate expected new FileIds
-    let new_base_file_id = testutils::write_file(store, &path, "resolved 1\nline 2\nline 3\n");
-    let new_left_file_id = testutils::write_file(store, &path, "resolved 1\nline 2\nleft 3\n");
-    let new_right_file_id = testutils::write_file(store, &path, "resolved 1\nline 2\nright 3\n");
+    let new_base_file_id = testutils::write_file(store, path, "resolved 1\nline 2\nline 3\n");
+    let new_left_file_id = testutils::write_file(store, path, "resolved 1\nline 2\nleft 3\n");
+    let new_right_file_id = testutils::write_file(store, path, "resolved 1\nline 2\nright 3\n");
     assert_eq!(
         new_conflict,
-        Merge::new(
+        Merge::from_removes_adds(
             vec![Some(new_base_file_id.clone())],
             vec![
                 Some(new_left_file_id.clone()),
@@ -652,18 +648,23 @@ fn test_update_conflict_from_content_modify_delete() {
     let store = test_repo.repo.store();
 
     let path = RepoPath::from_internal_string("dir/file");
-    let before_file_id = testutils::write_file(store, &path, "line 1\nline 2 before\nline 3\n");
-    let after_file_id = testutils::write_file(store, &path, "line 1\nline 2 after\nline 3\n");
-    let conflict = Merge::new(vec![Some(before_file_id)], vec![Some(after_file_id), None]);
+    let before_file_id = testutils::write_file(store, path, "line 1\nline 2 before\nline 3\n");
+    let after_file_id = testutils::write_file(store, path, "line 1\nline 2 after\nline 3\n");
+    let conflict =
+        Merge::from_removes_adds(vec![Some(before_file_id)], vec![Some(after_file_id), None]);
 
     // If the content is unchanged compared to the materialized value, we get the
     // old conflict id back.
-    let materialized = materialize_conflict_string(store, &path, &conflict);
-    let parse = |content| block_on(update_from_content(&conflict, store, &path, content)).unwrap();
+    let materialized = materialize_conflict_string(store, path, &conflict);
+    let parse = |content| {
+        update_from_content(&conflict, store, path, content)
+            .block_on()
+            .unwrap()
+    };
     assert_eq!(parse(materialized.as_bytes()), conflict);
 
     // If the conflict is resolved, we get None back to indicate that.
-    let expected_file_id = testutils::write_file(store, &path, "resolved\n");
+    let expected_file_id = testutils::write_file(store, path, "resolved\n");
     assert_eq!(parse(b"resolved\n"), Merge::normal(expected_file_id));
 
     // If the conflict is modified, we get a new conflict back.
@@ -671,13 +672,13 @@ fn test_update_conflict_from_content_modify_delete() {
         b"<<<<<<<\n%%%%%%%\n line 1\n-line 2 before\n+line 2 modified after\n line 3\n+++++++\n>>>>>>>\n",
     );
     // Calculate expected new FileIds
-    let new_base_file_id = testutils::write_file(store, &path, "line 1\nline 2 before\nline 3\n");
+    let new_base_file_id = testutils::write_file(store, path, "line 1\nline 2 before\nline 3\n");
     let new_left_file_id =
-        testutils::write_file(store, &path, "line 1\nline 2 modified after\nline 3\n");
+        testutils::write_file(store, path, "line 1\nline 2 modified after\nline 3\n");
 
     assert_eq!(
         new_conflict,
-        Merge::new(
+        Merge::from_removes_adds(
             vec![Some(new_base_file_id.clone())],
             vec![Some(new_left_file_id.clone()), None]
         )
@@ -690,7 +691,7 @@ fn materialize_conflict_string(
     conflict: &Merge<Option<FileId>>,
 ) -> String {
     let mut result: Vec<u8> = vec![];
-    let contents = block_on(extract_as_single_hunk(conflict, store, path));
+    let contents = extract_as_single_hunk(conflict, store, path).block_on();
     materialize_merge_result(&contents, &mut result).unwrap();
     String::from_utf8(result).unwrap()
 }
