@@ -12,19 +12,24 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::fs;
 use std::sync::Arc;
 
-use jj_lib::backend::CommitId;
+use assert_matches::assert_matches;
+use jj_lib::backend::{CommitId, ObjectId as _};
 use jj_lib::commit::Commit;
 use jj_lib::commit_builder::CommitBuilder;
-use jj_lib::default_index_store::{
-    CompositeIndex, IndexPosition, MutableIndexImpl, ReadonlyIndexWrapper,
+use jj_lib::default_index::{
+    AsCompositeIndex as _, CompositeIndex, DefaultIndexStore, DefaultIndexStoreError,
+    DefaultMutableIndex, DefaultReadonlyIndex, IndexPosition,
 };
 use jj_lib::index::Index as _;
 use jj_lib::repo::{MutableRepo, ReadonlyRepo, Repo};
 use jj_lib::settings::UserSettings;
+use testutils::test_backend::TestBackend;
 use testutils::{
-    create_random_commit, load_repo_at_head, write_random_commit, CommitGraphBuilder, TestRepo,
+    commit_transactions, create_random_commit, load_repo_at_head, write_random_commit,
+    CommitGraphBuilder, TestRepo,
 };
 
 fn child_commit<'repo>(
@@ -81,7 +86,7 @@ fn test_index_commits_standard_cases() {
     // o root
 
     let root_commit_id = repo.store().root_commit_id();
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
@@ -91,7 +96,7 @@ fn test_index_commits_standard_cases() {
     let commit_f = graph_builder.commit_with_parents(&[&commit_b, &commit_e]);
     let commit_g = graph_builder.commit_with_parents(&[&commit_f]);
     let commit_h = graph_builder.commit_with_parents(&[&commit_e]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     let index = as_readonly_composite(&repo);
     // There should be the root commit, plus 8 more
@@ -138,7 +143,7 @@ fn test_index_commits_criss_cross() {
     // Create a long chain of criss-crossed merges. If they were traversed without
     // keeping track of visited nodes, it would be 2^50 visits, so if this test
     // finishes in reasonable time, we know that we don't do a naive traversal.
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let mut left_commits = vec![graph_builder.initial_commit()];
     let mut right_commits = vec![graph_builder.initial_commit()];
@@ -150,7 +155,7 @@ fn test_index_commits_criss_cross() {
         left_commits.push(new_left);
         right_commits.push(new_right);
     }
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     let index = as_readonly_composite(&repo);
     // There should the root commit, plus 2 for each generation
@@ -275,16 +280,16 @@ fn test_index_commits_previous_operations() {
     // |/
     // o root
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     let commit_c = graph_builder.commit_with_parents(&[&commit_b]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     tx.mut_repo().remove_head(commit_c.id());
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     // Delete index from disk
     let index_operations_dir = repo.repo_path().join("index").join("operations");
@@ -323,24 +328,24 @@ fn test_index_commits_incremental() {
     // o root
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let commit_a = child_commit(tx.mut_repo(), &settings, &root_commit)
         .write()
         .unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     let index = as_readonly_composite(&repo);
     // There should be the root commit, plus 1 more
     assert_eq!(index.num_commits(), 1 + 1);
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let commit_b = child_commit(tx.mut_repo(), &settings, &commit_a)
         .write()
         .unwrap();
     let commit_c = child_commit(tx.mut_repo(), &settings, &commit_b)
         .write()
         .unwrap();
-    tx.commit();
+    tx.commit("test");
 
     let repo = load_repo_at_head(&settings, repo.repo_path());
     let index = as_readonly_composite(&repo);
@@ -374,17 +379,17 @@ fn test_index_commits_incremental_empty_transaction() {
     // o root
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let commit_a = child_commit(tx.mut_repo(), &settings, &root_commit)
         .write()
         .unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     let index = as_readonly_composite(&repo);
     // There should be the root commit, plus 1 more
     assert_eq!(index.num_commits(), 1 + 1);
 
-    repo.start_transaction(&settings, "test").commit();
+    repo.start_transaction(&settings).commit("test");
 
     let repo = load_repo_at_head(&settings, repo.repo_path());
     let index = as_readonly_composite(&repo);
@@ -416,17 +421,17 @@ fn test_index_commits_incremental_already_indexed() {
     // o root
 
     let root_commit = repo.store().root_commit();
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let commit_a = child_commit(tx.mut_repo(), &settings, &root_commit)
         .write()
         .unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     assert!(repo.index().has_id(commit_a.id()));
     assert_eq!(as_readonly_composite(&repo).num_commits(), 1 + 1);
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
-    mut_repo.add_head(&commit_a);
+    mut_repo.add_head(&commit_a).unwrap();
     assert_eq!(as_mutable_composite(mut_repo).num_commits(), 1 + 1);
 }
 
@@ -436,33 +441,27 @@ fn create_n_commits(
     repo: &Arc<ReadonlyRepo>,
     num_commits: i32,
 ) -> Arc<ReadonlyRepo> {
-    let mut tx = repo.start_transaction(settings, "test");
+    let mut tx = repo.start_transaction(settings);
     for _ in 0..num_commits {
         write_random_commit(tx.mut_repo(), settings);
     }
-    tx.commit()
-}
-
-fn as_readonly_wrapper(repo: &Arc<ReadonlyRepo>) -> &ReadonlyIndexWrapper {
-    repo.readonly_index()
-        .as_any()
-        .downcast_ref::<ReadonlyIndexWrapper>()
-        .unwrap()
+    tx.commit("test")
 }
 
 fn as_readonly_composite(repo: &Arc<ReadonlyRepo>) -> CompositeIndex<'_> {
-    as_readonly_wrapper(repo).as_composite()
-}
-
-fn as_mutable_impl(repo: &MutableRepo) -> &MutableIndexImpl {
-    repo.mutable_index()
+    repo.readonly_index()
         .as_any()
-        .downcast_ref::<MutableIndexImpl>()
+        .downcast_ref::<DefaultReadonlyIndex>()
         .unwrap()
+        .as_composite()
 }
 
 fn as_mutable_composite(repo: &MutableRepo) -> CompositeIndex<'_> {
-    as_mutable_impl(repo).as_composite()
+    repo.mutable_index()
+        .as_any()
+        .downcast_ref::<DefaultMutableIndex>()
+        .unwrap()
+        .as_composite()
 }
 
 fn commits_by_level(repo: &Arc<ReadonlyRepo>) -> Vec<u32> {
@@ -537,6 +536,87 @@ fn test_index_commits_incremental_squashed() {
     assert_eq!(commits_by_level(&repo), vec![71, 20]);
 }
 
+#[test]
+fn test_reindex_from_merged_operation() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    // The following operation log:
+    // x (add head, index will be missing)
+    // x (add head, index will be missing)
+    // |\
+    // o o (remove head)
+    // o o (add head)
+    // |/
+    // o
+    let mut txs = Vec::new();
+    for _ in 0..2 {
+        let mut tx = repo.start_transaction(&settings);
+        let commit = write_random_commit(tx.mut_repo(), &settings);
+        let repo = tx.commit("test");
+        let mut tx = repo.start_transaction(&settings);
+        tx.mut_repo().remove_head(commit.id());
+        txs.push(tx);
+    }
+    let repo = commit_transactions(&settings, txs);
+    let mut op_ids_to_delete = Vec::new();
+    op_ids_to_delete.push(repo.op_id());
+    let mut tx = repo.start_transaction(&settings);
+    write_random_commit(tx.mut_repo(), &settings);
+    let repo = tx.commit("test");
+    op_ids_to_delete.push(repo.op_id());
+    let operation_to_reload = repo.operation();
+
+    // Sanity check before corrupting the index store
+    let index = as_readonly_composite(&repo);
+    assert_eq!(index.num_commits(), 4);
+
+    let index_operations_dir = repo.repo_path().join("index").join("operations");
+    for &op_id in &op_ids_to_delete {
+        fs::remove_file(index_operations_dir.join(op_id.hex())).unwrap();
+    }
+
+    // When re-indexing, one of the merge parent operations will be selected as
+    // the parent index segment. The commits in the other side should still be
+    // reachable.
+    let repo = repo.reload_at(operation_to_reload).unwrap();
+    let index = as_readonly_composite(&repo);
+    assert_eq!(index.num_commits(), 4);
+}
+
+#[test]
+fn test_reindex_missing_commit() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction(&settings);
+    let missing_commit = write_random_commit(tx.mut_repo(), &settings);
+    let repo = tx.commit("test");
+    let bad_op_id = repo.op_id();
+
+    let mut tx = repo.start_transaction(&settings);
+    tx.mut_repo().remove_head(missing_commit.id());
+    let repo = tx.commit("test");
+
+    // Remove historical head commit to simulate bad GC.
+    let test_backend: &TestBackend = repo.store().backend_impl().downcast_ref().unwrap();
+    test_backend.remove_commit_unchecked(missing_commit.id());
+    let repo = load_repo_at_head(&settings, repo.repo_path()); // discard cache
+    assert!(repo.store().get_commit(missing_commit.id()).is_err());
+
+    // Reindexing error should include the operation id where the commit
+    // couldn't be found.
+    let default_index_store: &DefaultIndexStore =
+        repo.index_store().as_any().downcast_ref().unwrap();
+    default_index_store.reinit().unwrap();
+    let err = default_index_store
+        .build_index_at_operation(repo.operation(), repo.store())
+        .unwrap_err();
+    assert_matches!(err, DefaultIndexStoreError::IndexCommits { op_id, .. } if op_id == *bad_op_id);
+}
+
 /// Test that .jj/repo/index/type is created when the repo is created, and that
 /// it is created when an old repo is loaded.
 #[test]
