@@ -17,7 +17,7 @@ use jj_lib::op_store::{RefTarget, RemoteRef, RemoteRefState, WorkspaceId};
 use jj_lib::repo::Repo;
 use maplit::hashset;
 use testutils::{
-    assert_rebased, create_random_commit, write_random_commit, CommitGraphBuilder, TestRepo,
+    assert_rebased_onto, create_random_commit, write_random_commit, CommitGraphBuilder, TestRepo,
 };
 
 #[test]
@@ -27,14 +27,14 @@ fn test_edit() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let wc_commit = write_random_commit(tx.mut_repo(), &settings);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let ws_id = WorkspaceId::default();
     tx.mut_repo().edit(ws_id.clone(), &wc_commit).unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     assert_eq!(repo.view().get_wc_commit_id(&ws_id), Some(wc_commit.id()));
 }
 
@@ -45,11 +45,11 @@ fn test_checkout() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let wc_commit_parent = write_random_commit(tx.mut_repo(), &settings);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let ws_id = WorkspaceId::default();
     let wc_commit = tx
         .mut_repo()
@@ -58,7 +58,7 @@ fn test_checkout() {
     assert_eq!(wc_commit.tree_id(), wc_commit_parent.tree_id());
     assert_eq!(wc_commit.parents().len(), 1);
     assert_eq!(wc_commit.parents()[0].id(), wc_commit_parent.id());
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     assert_eq!(repo.view().get_wc_commit_id(&ws_id), Some(wc_commit.id()));
 }
 
@@ -70,14 +70,14 @@ fn test_checkout_previous_not_empty() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let old_wc_commit = write_random_commit(mut_repo, &settings);
     let ws_id = WorkspaceId::default();
     mut_repo.edit(ws_id.clone(), &old_wc_commit).unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let new_wc_commit = write_random_commit(mut_repo, &settings);
     mut_repo.edit(ws_id, &new_wc_commit).unwrap();
@@ -93,7 +93,7 @@ fn test_checkout_previous_empty() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let old_wc_commit = mut_repo
         .new_commit(
@@ -105,9 +105,9 @@ fn test_checkout_previous_empty() {
         .unwrap();
     let ws_id = WorkspaceId::default();
     mut_repo.edit(ws_id.clone(), &old_wc_commit).unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let new_wc_commit = write_random_commit(mut_repo, &settings);
     mut_repo.edit(ws_id, &new_wc_commit).unwrap();
@@ -123,7 +123,7 @@ fn test_checkout_previous_empty_with_description() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let old_wc_commit = mut_repo
         .new_commit(
@@ -136,9 +136,9 @@ fn test_checkout_previous_empty_with_description() {
         .unwrap();
     let ws_id = WorkspaceId::default();
     mut_repo.edit(ws_id.clone(), &old_wc_commit).unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let new_wc_commit = write_random_commit(mut_repo, &settings);
     mut_repo.edit(ws_id, &new_wc_commit).unwrap();
@@ -154,7 +154,7 @@ fn test_checkout_previous_empty_with_local_branch() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let old_wc_commit = mut_repo
         .new_commit(
@@ -167,9 +167,9 @@ fn test_checkout_previous_empty_with_local_branch() {
     mut_repo.set_local_branch_target("b", RefTarget::normal(old_wc_commit.id().clone()));
     let ws_id = WorkspaceId::default();
     mut_repo.edit(ws_id.clone(), &old_wc_commit).unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let new_wc_commit = write_random_commit(mut_repo, &settings);
     mut_repo.edit(ws_id, &new_wc_commit).unwrap();
@@ -185,7 +185,7 @@ fn test_checkout_previous_empty_non_head() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let old_wc_commit = mut_repo
         .new_commit(
@@ -205,9 +205,9 @@ fn test_checkout_previous_empty_non_head() {
         .unwrap();
     let ws_id = WorkspaceId::default();
     mut_repo.edit(ws_id.clone(), &old_wc_commit).unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let new_wc_commit = write_random_commit(mut_repo, &settings);
     mut_repo.edit(ws_id, &new_wc_commit).unwrap();
@@ -226,16 +226,16 @@ fn test_edit_initial() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let wc_commit = write_random_commit(tx.mut_repo(), &settings);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let workspace_id = WorkspaceId::new("new-workspace".to_string());
     tx.mut_repo()
         .edit(workspace_id.clone(), &wc_commit)
         .unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     assert_eq!(
         repo.view().get_wc_commit_id(&workspace_id),
         Some(wc_commit.id())
@@ -252,18 +252,18 @@ fn test_add_head_success() {
 
     // Create a commit outside of the repo by using a temporary transaction. Then
     // add that as a head.
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let new_commit = write_random_commit(tx.mut_repo(), &settings);
     drop(tx);
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     assert!(!mut_repo.view().heads().contains(new_commit.id()));
     assert!(!mut_repo.index().has_id(new_commit.id()));
-    mut_repo.add_head(&new_commit);
+    mut_repo.add_head(&new_commit).unwrap();
     assert!(mut_repo.view().heads().contains(new_commit.id()));
     assert!(mut_repo.index().has_id(new_commit.id()));
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     assert!(repo.view().heads().contains(new_commit.id()));
     assert!(repo.index().has_id(new_commit.id()));
 }
@@ -276,17 +276,17 @@ fn test_add_head_ancestor() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     let commit3 = graph_builder.commit_with_parents(&[&commit2]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     assert_eq!(repo.view().heads(), &hashset! {commit3.id().clone()});
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
-    mut_repo.add_head(&commit1);
+    mut_repo.add_head(&commit1).unwrap();
     assert_eq!(repo.view().heads(), &hashset! {commit3.id().clone()});
 }
 
@@ -298,13 +298,13 @@ fn test_add_head_not_immediate_child() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let initial = write_random_commit(tx.mut_repo(), &settings);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     // Create some commits outside of the repo by using a temporary transaction.
     // Then add one of them as a head.
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let rewritten = create_random_commit(tx.mut_repo(), &settings)
         .set_change_id(initial.change_id().clone())
         .set_predecessors(vec![initial.id().clone()])
@@ -317,9 +317,9 @@ fn test_add_head_not_immediate_child() {
     drop(tx);
 
     assert_eq!(repo.view().heads(), &hashset! {initial.id().clone()});
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
-    mut_repo.add_head(&child);
+    mut_repo.add_head(&child).unwrap();
     assert_eq!(
         mut_repo.view().heads(),
         &hashset! {initial.id().clone(), child.id().clone()}
@@ -338,14 +338,14 @@ fn test_remove_head() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     let commit3 = graph_builder.commit_with_parents(&[&commit2]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     assert!(mut_repo.view().heads().contains(commit3.id()));
     mut_repo.remove_head(commit3.id());
@@ -356,7 +356,7 @@ fn test_remove_head() {
     assert!(mut_repo.index().has_id(commit1.id()));
     assert!(mut_repo.index().has_id(commit2.id()));
     assert!(mut_repo.index().has_id(commit3.id()));
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     let heads = repo.view().heads().clone();
     assert!(!heads.contains(commit3.id()));
     assert!(!heads.contains(commit2.id()));
@@ -374,16 +374,16 @@ fn test_add_public_head() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let commit1 = write_random_commit(tx.mut_repo(), &settings);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     assert!(!mut_repo.view().public_heads().contains(commit1.id()));
     mut_repo.add_public_head(&commit1);
     assert!(mut_repo.view().public_heads().contains(commit1.id()));
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     assert!(repo.view().public_heads().contains(commit1.id()));
 }
 
@@ -395,19 +395,19 @@ fn test_add_public_head_ancestor() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     tx.mut_repo().add_public_head(&commit2);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     assert!(!mut_repo.view().public_heads().contains(commit1.id()));
     mut_repo.add_public_head(&commit1);
     assert!(!mut_repo.view().public_heads().contains(commit1.id()));
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     assert!(!repo.view().public_heads().contains(commit1.id()));
 }
 
@@ -419,18 +419,18 @@ fn test_remove_public_head() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let commit1 = write_random_commit(mut_repo, &settings);
     mut_repo.add_public_head(&commit1);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     assert!(mut_repo.view().public_heads().contains(commit1.id()));
     mut_repo.remove_public_head(commit1.id());
     assert!(!mut_repo.view().public_heads().contains(commit1.id()));
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     assert!(!repo.view().public_heads().contains(commit1.id()));
 }
 
@@ -447,7 +447,7 @@ fn test_has_changed() {
         state: RemoteRefState::Tracking, // doesn't matter
     };
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let commit1 = write_random_commit(mut_repo, &settings);
     let commit2 = write_random_commit(mut_repo, &settings);
@@ -459,16 +459,16 @@ fn test_has_changed() {
         .unwrap();
     mut_repo.set_local_branch_target("main", RefTarget::normal(commit1.id().clone()));
     mut_repo.set_remote_branch("main", "origin", normal_remote_ref(commit1.id()));
-    let repo = tx.commit();
+    let repo = tx.commit("test");
     // Test the setup
     assert_eq!(repo.view().heads(), &hashset! {commit1.id().clone()});
     assert_eq!(repo.view().public_heads(), &hashset! {commit1.id().clone()});
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
 
     mut_repo.add_public_head(&commit1);
-    mut_repo.add_head(&commit1);
+    mut_repo.add_head(&commit1).unwrap();
     mut_repo
         .set_wc_commit(ws_id.clone(), commit1.id().clone())
         .unwrap();
@@ -482,7 +482,7 @@ fn test_has_changed() {
     mut_repo.set_remote_branch("stable", "origin", RemoteRef::absent());
     assert!(!mut_repo.has_changes());
 
-    mut_repo.add_head(&commit2);
+    mut_repo.add_head(&commit2).unwrap();
     assert!(mut_repo.has_changes());
     mut_repo.remove_head(commit2.id());
     assert!(!mut_repo.has_changes());
@@ -517,74 +517,74 @@ fn test_has_changed() {
 
 #[test]
 fn test_rebase_descendants_simple() {
-    // Tests that MutableRepo::create_descendant_rebaser() creates a
-    // DescendantRebaser that rebases descendants of rewritten and abandoned
-    // commits.
+    // There are many additional tests of this functionality in `test_rewrite.rs`.
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     let commit3 = graph_builder.commit_with_parents(&[&commit2]);
     let commit4 = graph_builder.commit_with_parents(&[&commit1]);
     let commit5 = graph_builder.commit_with_parents(&[&commit4]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit6 = graph_builder.commit_with_parents(&[&commit1]);
     mut_repo.record_rewritten_commit(commit2.id().clone(), commit6.id().clone());
     mut_repo.record_abandoned_commit(commit4.id().clone());
-    let mut rebaser = mut_repo.create_descendant_rebaser(&settings);
+    let rebase_map = tx
+        .mut_repo()
+        .rebase_descendants_return_map(&settings)
+        .unwrap();
     // Commit 3 got rebased onto commit 2's replacement, i.e. commit 6
-    assert_rebased(rebaser.rebase_next().unwrap(), &commit3, &[&commit6]);
+    assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit3, &[commit6.id()]);
     // Commit 5 got rebased onto commit 4's parent, i.e. commit 1
-    assert_rebased(rebaser.rebase_next().unwrap(), &commit5, &[&commit1]);
-    assert!(rebaser.rebase_next().unwrap().is_none());
+    assert_rebased_onto(tx.mut_repo(), &rebase_map, &commit5, &[commit1.id()]);
+    assert_eq!(rebase_map.len(), 2);
+
     // No more descendants to rebase if we try again.
-    assert!(mut_repo
-        .create_descendant_rebaser(&settings)
-        .rebase_next()
-        .unwrap()
-        .is_none());
+    let rebase_map = tx
+        .mut_repo()
+        .rebase_descendants_return_map(&settings)
+        .unwrap();
+    assert_eq!(rebase_map.len(), 0);
 }
 
 #[test]
 fn test_rebase_descendants_conflicting_rewrite() {
-    // Tests MutableRepo::create_descendant_rebaser() when a commit has been marked
-    // as rewritten to several other commits.
+    // Test rebasing descendants when one commit was rewritten to several other
+    // commits. There are many additional tests of this functionality in
+    // `test_rewrite.rs`.
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let commit1 = graph_builder.initial_commit();
     let commit2 = graph_builder.commit_with_parents(&[&commit1]);
     let _commit3 = graph_builder.commit_with_parents(&[&commit2]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let mut graph_builder = CommitGraphBuilder::new(&settings, mut_repo);
     let commit4 = graph_builder.commit_with_parents(&[&commit1]);
     let commit5 = graph_builder.commit_with_parents(&[&commit1]);
     mut_repo.record_rewritten_commit(commit2.id().clone(), commit4.id().clone());
     mut_repo.record_rewritten_commit(commit2.id().clone(), commit5.id().clone());
-    let mut rebaser = mut_repo.create_descendant_rebaser(&settings);
     // Commit 3 does *not* get rebased because it's unclear if it should go onto
     // commit 4 or commit 5
-    assert!(rebaser.rebase_next().unwrap().is_none());
-    // No more descendants to rebase if we try again.
-    assert!(mut_repo
-        .create_descendant_rebaser(&settings)
-        .rebase_next()
-        .unwrap()
-        .is_none());
+    let rebase_map = tx
+        .mut_repo()
+        .rebase_descendants_return_map(&settings)
+        .unwrap();
+    assert!(rebase_map.is_empty());
 }
 
 #[test]
@@ -592,7 +592,7 @@ fn test_rename_remote() {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let commit = write_random_commit(mut_repo, &settings);
     let remote_ref = RemoteRef {
