@@ -42,13 +42,13 @@ fn test_heads_fork() {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
 
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let initial = graph_builder.initial_commit();
     let child1 = graph_builder.commit_with_parents(&[&initial]);
     let child2 = graph_builder.commit_with_parents(&[&initial]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     assert_eq!(
         *repo.view().heads(),
@@ -64,14 +64,14 @@ fn test_heads_merge() {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
 
     let mut graph_builder = CommitGraphBuilder::new(&settings, tx.mut_repo());
     let initial = graph_builder.initial_commit();
     let child1 = graph_builder.commit_with_parents(&[&initial]);
     let child2 = graph_builder.commit_with_parents(&[&initial]);
     let merge = graph_builder.commit_with_parents(&[&child1, &child2]);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
     assert_eq!(*repo.view().heads(), hashset! {merge.id().clone()});
 }
@@ -83,7 +83,7 @@ fn test_merge_views_heads() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let head_unchanged = write_random_commit(mut_repo, &settings);
     let head_remove_tx1 = write_random_commit(mut_repo, &settings);
@@ -94,9 +94,9 @@ fn test_merge_views_heads() {
     mut_repo.add_public_head(&public_head_remove_tx1);
     let public_head_remove_tx2 = write_random_commit(mut_repo, &settings);
     mut_repo.add_public_head(&public_head_remove_tx2);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     tx1.mut_repo().remove_head(head_remove_tx1.id());
     tx1.mut_repo()
         .remove_public_head(public_head_remove_tx1.id());
@@ -104,7 +104,7 @@ fn test_merge_views_heads() {
     let public_head_add_tx1 = write_random_commit(tx1.mut_repo(), &settings);
     tx1.mut_repo().add_public_head(&public_head_add_tx1);
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     tx2.mut_repo().remove_head(head_remove_tx2.id());
     tx2.mut_repo()
         .remove_public_head(public_head_remove_tx2.id());
@@ -148,7 +148,7 @@ fn test_merge_views_checkout() {
     // Workspace 5 gets deleted in tx2 and modified in tx1.
     // Workspace 6 gets added in tx1.
     // Workspace 7 gets added in tx2.
-    let mut initial_tx = repo.start_transaction(&settings, "test");
+    let mut initial_tx = repo.start_transaction(&settings);
     let commit1 = write_random_commit(initial_tx.mut_repo(), &settings);
     let commit2 = write_random_commit(initial_tx.mut_repo(), &settings);
     let commit3 = write_random_commit(initial_tx.mut_repo(), &settings);
@@ -179,9 +179,9 @@ fn test_merge_views_checkout() {
         .mut_repo()
         .set_wc_commit(ws5_id.clone(), commit1.id().clone())
         .unwrap();
-    let repo = initial_tx.commit();
+    let repo = initial_tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     tx1.mut_repo()
         .set_wc_commit(ws1_id.clone(), commit2.id().clone())
         .unwrap();
@@ -196,7 +196,7 @@ fn test_merge_views_checkout() {
         .set_wc_commit(ws6_id.clone(), commit2.id().clone())
         .unwrap();
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     tx2.mut_repo()
         .set_wc_commit(ws1_id.clone(), commit3.id().clone())
         .unwrap();
@@ -232,7 +232,7 @@ fn test_merge_views_branches() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let main_branch_local_tx0 = write_random_commit(mut_repo, &settings);
     let main_branch_origin_tx0 = write_random_commit(mut_repo, &settings);
@@ -260,9 +260,9 @@ fn test_merge_views_branches() {
         "feature",
         RefTarget::normal(feature_branch_local_tx0.id().clone()),
     );
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let main_branch_local_tx1 = write_random_commit(tx1.mut_repo(), &settings);
     tx1.mut_repo().set_local_branch_target(
         "main",
@@ -274,7 +274,7 @@ fn test_merge_views_branches() {
         RefTarget::normal(feature_branch_tx1.id().clone()),
     );
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     let main_branch_local_tx2 = write_random_commit(tx2.mut_repo(), &settings);
     let main_branch_origin_tx2 = write_random_commit(tx2.mut_repo(), &settings);
     let main_branch_origin_tx2_remote_ref = RemoteRef {
@@ -324,15 +324,15 @@ fn test_merge_views_tags() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let v1_tx0 = write_random_commit(mut_repo, &settings);
     mut_repo.set_tag_target("v1.0", RefTarget::normal(v1_tx0.id().clone()));
     let v2_tx0 = write_random_commit(mut_repo, &settings);
     mut_repo.set_tag_target("v2.0", RefTarget::normal(v2_tx0.id().clone()));
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let v1_tx1 = write_random_commit(tx1.mut_repo(), &settings);
     tx1.mut_repo()
         .set_tag_target("v1.0", RefTarget::normal(v1_tx1.id().clone()));
@@ -340,7 +340,7 @@ fn test_merge_views_tags() {
     tx1.mut_repo()
         .set_tag_target("v2.0", RefTarget::normal(v2_tx1.id().clone()));
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     let v1_tx2 = write_random_commit(tx2.mut_repo(), &settings);
     tx2.mut_repo()
         .set_tag_target("v1.0", RefTarget::normal(v1_tx2.id().clone()));
@@ -368,7 +368,7 @@ fn test_merge_views_git_refs() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx = repo.start_transaction(&settings, "test");
+    let mut tx = repo.start_transaction(&settings);
     let mut_repo = tx.mut_repo();
     let main_branch_tx0 = write_random_commit(mut_repo, &settings);
     mut_repo.set_git_ref_target(
@@ -380,9 +380,9 @@ fn test_merge_views_git_refs() {
         "refs/heads/feature",
         RefTarget::normal(feature_branch_tx0.id().clone()),
     );
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let main_branch_tx1 = write_random_commit(tx1.mut_repo(), &settings);
     tx1.mut_repo().set_git_ref_target(
         "refs/heads/main",
@@ -394,7 +394,7 @@ fn test_merge_views_git_refs() {
         RefTarget::normal(feature_branch_tx1.id().clone()),
     );
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     let main_branch_tx2 = write_random_commit(tx2.mut_repo(), &settings);
     tx2.mut_repo().set_git_ref_target(
         "refs/heads/main",
@@ -424,18 +424,18 @@ fn test_merge_views_git_heads() {
     let test_repo = TestRepo::init();
     let repo = &test_repo.repo;
 
-    let mut tx0 = repo.start_transaction(&settings, "test");
+    let mut tx0 = repo.start_transaction(&settings);
     let tx0_head = write_random_commit(tx0.mut_repo(), &settings);
     tx0.mut_repo()
         .set_git_head_target(RefTarget::normal(tx0_head.id().clone()));
-    let repo = tx0.commit();
+    let repo = tx0.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let tx1_head = write_random_commit(tx1.mut_repo(), &settings);
     tx1.mut_repo()
         .set_git_head_target(RefTarget::normal(tx1_head.id().clone()));
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     let tx2_head = write_random_commit(tx2.mut_repo(), &settings);
     tx2.mut_repo()
         .set_git_head_target(RefTarget::normal(tx2_head.id().clone()));
@@ -455,11 +455,11 @@ fn test_merge_views_divergent() {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
 
-    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let mut tx = test_repo.repo.start_transaction(&settings);
     let commit_a = write_random_commit(tx.mut_repo(), &settings);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let commit_a2 = tx1
         .mut_repo()
         .rewrite_commit(&settings, &commit_a)
@@ -468,7 +468,7 @@ fn test_merge_views_divergent() {
         .unwrap();
     tx1.mut_repo().rebase_descendants(&settings).unwrap();
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     let commit_a3 = tx2
         .mut_repo()
         .rewrite_commit(&settings, &commit_a)
@@ -494,17 +494,17 @@ fn test_merge_views_child_on_rewritten(child_first: bool) {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
 
-    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let mut tx = test_repo.repo.start_transaction(&settings);
     let commit_a = write_random_commit(tx.mut_repo(), &settings);
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let commit_b = create_random_commit(tx1.mut_repo(), &settings)
         .set_parents(vec![commit_a.id().clone()])
         .write()
         .unwrap();
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     let commit_a2 = tx2
         .mut_repo()
         .rewrite_commit(&settings, &commit_a)
@@ -540,22 +540,22 @@ fn test_merge_views_child_on_rewritten_divergent(on_rewritten: bool, child_first
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
 
-    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let mut tx = test_repo.repo.start_transaction(&settings);
     let commit_a2 = write_random_commit(tx.mut_repo(), &settings);
     let commit_a3 = create_random_commit(tx.mut_repo(), &settings)
         .set_change_id(commit_a2.change_id().clone())
         .write()
         .unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let parent = if on_rewritten { &commit_a2 } else { &commit_a3 };
     let commit_b = create_random_commit(tx1.mut_repo(), &settings)
         .set_parents(vec![parent.id().clone()])
         .write()
         .unwrap();
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     let commit_a4 = tx2
         .mut_repo()
         .rewrite_commit(&settings, &commit_a2)
@@ -596,21 +596,21 @@ fn test_merge_views_child_on_abandoned(child_first: bool) {
     let settings = testutils::user_settings();
     let test_repo = TestRepo::init();
 
-    let mut tx = test_repo.repo.start_transaction(&settings, "test");
+    let mut tx = test_repo.repo.start_transaction(&settings);
     let commit_a = write_random_commit(tx.mut_repo(), &settings);
     let commit_b = create_random_commit(tx.mut_repo(), &settings)
         .set_parents(vec![commit_a.id().clone()])
         .write()
         .unwrap();
-    let repo = tx.commit();
+    let repo = tx.commit("test");
 
-    let mut tx1 = repo.start_transaction(&settings, "test");
+    let mut tx1 = repo.start_transaction(&settings);
     let commit_c = create_random_commit(tx1.mut_repo(), &settings)
         .set_parents(vec![commit_b.id().clone()])
         .write()
         .unwrap();
 
-    let mut tx2 = repo.start_transaction(&settings, "test");
+    let mut tx2 = repo.start_transaction(&settings);
     tx2.mut_repo()
         .record_abandoned_commit(commit_b.id().clone());
     tx2.mut_repo().rebase_descendants(&settings).unwrap();
