@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::collections::HashMap;
 use std::env;
 use std::fs::{self, OpenOptions};
 use std::io::{Read, Write};
@@ -20,8 +21,8 @@ use std::sync::{Arc, Once};
 
 use itertools::Itertools;
 use jj_lib::backend::{
-    self, Backend, BackendInitError, ChangeId, FileId, MergedTreeId, MillisSinceEpoch, ObjectId,
-    Signature, Timestamp, TreeValue,
+    self, Backend, BackendInitError, ChangeId, CommitId, FileId, MergedTreeId, MillisSinceEpoch,
+    ObjectId, Signature, Timestamp, TreeValue,
 };
 use jj_lib::commit::Commit;
 use jj_lib::commit_builder::CommitBuilder;
@@ -30,7 +31,6 @@ use jj_lib::local_backend::LocalBackend;
 use jj_lib::merged_tree::MergedTree;
 use jj_lib::repo::{MutableRepo, ReadonlyRepo, Repo, RepoLoader, StoreFactories};
 use jj_lib::repo_path::{RepoPath, RepoPathBuf};
-use jj_lib::rewrite::RebasedDescendant;
 use jj_lib::settings::UserSettings;
 use jj_lib::signing::Signer;
 use jj_lib::store::Store;
@@ -131,17 +131,27 @@ impl TestRepo {
     }
 
     pub fn init_with_backend(backend: TestRepoBackend) -> Self {
-        let settings = user_settings();
+        Self::init_with_backend_and_settings(backend, &user_settings())
+    }
+
+    pub fn init_with_settings(settings: &UserSettings) -> Self {
+        Self::init_with_backend_and_settings(TestRepoBackend::Test, settings)
+    }
+
+    pub fn init_with_backend_and_settings(
+        backend: TestRepoBackend,
+        settings: &UserSettings,
+    ) -> Self {
         let temp_dir = new_temp_dir();
 
         let repo_dir = temp_dir.path().join("repo");
         fs::create_dir(&repo_dir).unwrap();
 
         let repo = ReadonlyRepo::init(
-            &settings,
+            settings,
             &repo_dir,
             &move |settings, store_path| backend.init_backend(settings, store_path),
-            Signer::from_settings(&settings).unwrap(),
+            Signer::from_settings(settings).unwrap(),
             ReadonlyRepo::default_op_store_initializer(),
             ReadonlyRepo::default_op_heads_store_initializer(),
             ReadonlyRepo::default_index_store_initializer(),
@@ -241,7 +251,7 @@ pub fn commit_transactions(settings: &UserSettings, txs: Vec<Transaction>) -> Ar
     let repo_loader = txs[0].base_repo().loader();
     let mut op_ids = vec![];
     for tx in txs {
-        op_ids.push(tx.commit().op_id().clone());
+        op_ids.push(tx.commit("test").op_id().clone());
         std::thread::sleep(std::time::Duration::from_millis(1));
     }
     let repo = repo_loader.load_at_head(settings).unwrap();
@@ -442,27 +452,55 @@ impl<'settings, 'repo> CommitGraphBuilder<'settings, 'repo> {
     }
 }
 
-pub fn assert_rebased(
-    rebased: Option<RebasedDescendant>,
+fn assert_in_rebased_map(
+    repo: &impl Repo,
+    rebased: &HashMap<CommitId, CommitId>,
     expected_old_commit: &Commit,
-    expected_new_parents: &[&Commit],
 ) -> Commit {
-    if let Some(RebasedDescendant {
-        old_commit,
-        new_commit,
-    }) = rebased
-    {
-        assert_eq!(old_commit, *expected_old_commit);
-        assert_eq!(new_commit.change_id(), expected_old_commit.change_id());
-        assert_eq!(
-            new_commit.parent_ids(),
-            expected_new_parents
-                .iter()
-                .map(|commit| commit.id().clone())
-                .collect_vec()
-        );
-        new_commit
-    } else {
-        panic!("expected rebased commit: {rebased:?}");
-    }
+    let new_commit_id = rebased.get(expected_old_commit.id()).unwrap_or_else(|| {
+        panic!(
+            "Expected commit to have been rebased: {}",
+            expected_old_commit.id().hex()
+        )
+    });
+    let new_commit = repo.store().get_commit(new_commit_id).unwrap().clone();
+    new_commit
+}
+
+pub fn assert_rebased_onto(
+    repo: &impl Repo,
+    rebased: &HashMap<CommitId, CommitId>,
+    expected_old_commit: &Commit,
+    expected_new_parent_ids: &[&CommitId],
+) -> Commit {
+    let new_commit = assert_in_rebased_map(repo, rebased, expected_old_commit);
+    assert_eq!(
+        new_commit.parent_ids().to_vec(),
+        expected_new_parent_ids
+            .iter()
+            .map(|x| (*x).clone())
+            .collect_vec()
+    );
+    assert_eq!(new_commit.change_id(), expected_old_commit.change_id());
+    new_commit
+}
+
+/// If `expected_old_commit` was abandoned, the `rebased` map indicates the
+/// commit the children of `expected_old_commit` should be rebased to, which
+/// would have a different change id. This happens when the EmptyBehavior in
+/// RebaseOptions is not the default; because of the details of the
+/// implementation this returned parent commit is always singular.
+pub fn assert_abandoned_with_parent(
+    repo: &impl Repo,
+    rebased: &HashMap<CommitId, CommitId>,
+    expected_old_commit: &Commit,
+    expected_new_parent_id: &CommitId,
+) -> Commit {
+    let new_parent_commit = assert_in_rebased_map(repo, rebased, expected_old_commit);
+    assert_eq!(new_parent_commit.id(), expected_new_parent_id);
+    assert_ne!(
+        new_parent_commit.change_id(),
+        expected_old_commit.change_id()
+    );
+    new_parent_commit
 }
